/*-----------------------------------------------------------------------
 * File: DCODEBIT.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"
#include "ber_internal.h"

/* 
 * Function to decode a primitive BER bit string into an array of bytes. 
 * The function returns -2 if the input is not a valid primitive bit string.
 * The function result is the length of the decoded bit string in *BITS*. 
 * If the target buffer (Bytes) is not sufficiently large to hold the result,
 * the function returns -1 and does not alter the output buffer. 
 * Any odd bits are in the most significant bits of the last byte.
 */
sint32 
BER_DecodeBitString(const uint8 *Octets,  /* pointer to input BER_BIT_STRING octets */
    uint8 *Bytes,                   /* pointer to output buffer */
    uint32 SizeofBytes)             /* size of output buffer in bytes */
{
    sint32 result;
    uint32 length;
    BER_PARSED_ITEM item;

    length = BER_SizeofObject(Octets);
    if (length > 1000000 /* detect trash by ridiculous length */
        || !BER_ExpandItem(Octets, length, &item))
        return -2;
    if (item.Tag[0] != BER_BIT_STRING || item.Content[0] > 7) 
        return -2;
    length = item.ContentLength - 1;
    result = length * 8 - item.Content[0];
    if (length > SizeofBytes) 
        return -1;
    cssm_memcpy(Bytes, item.Content + 1, length);
    if (item.Content[0])  /* clear padding bits just for cleanliness */
        Bytes[length] &= (0xff << item.Content[0]); 
    return result;
}
