/*-----------------------------------------------------------------------
 * File: FINDATTR.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"
#include "ber_internal.h"

static const uint8 *BER_SearchTree(DER_NODE_PTR Tree, const uint8 *Oid)
{
    BER_PARSED_ITEM parsedOID;
    const uint8 *result;
    sint16 i;

    if (!Tree || Tree->Count < 2) 
        return 0;
    if (Tree->Child[0].X.Input.Tag[0] == BER_OBJECT_IDENTIFIER) {
        if (!BER_ExpandItem(Oid, BER_SizeofObject(Oid), &parsedOID)) 
            return 0;
        if (Tree->Child[0].X.Input.ContentLength == parsedOID.ContentLength &&
            !cssm_memcmp(parsedOID.Content, 
                Tree->Child[0].X.Input.Content, 
                parsedOID.ContentLength)) {
            return Tree->Child[1].X.Input.Tag;
        }
    }
    for (i = 0; i < Tree->Count; i++) {
        if (!Tree->Child[i].IsLeaf) {
            result = BER_SearchTree(Tree->Child[i].X.Node, Oid);
            if (result) 
                return result;
        }
    }
    return 0;
}

/*
**  find a BER attribute value (still BER), given an OID and a BER-encoded attribute list
**  returns NIL if not found. Assumes SEQ(OID, value) structure somewhere in subtree.
*/
const uint8 *
BER_FindAttribute(  /* find BER attribute value for given OID in attributes */
    BER_HANDLE AppHandle,
    BER_MEMORY_FUNCS_PTR MemoryFunctions,
    const uint8 *Oid,         /* desired BER OID */
    const uint8 *Attributes)  /* BER with SEQ(OID, value) pairs somewhere */
{
    DER_NODE_PTR tree;
    BER_PARSED_ITEM attr;
    const uint8 *result;

    if (!BER_ExpandItem(Attributes, BER_SizeofObject(Attributes), &attr)) 
        return 0;
    tree = BER_ParseComplexObject(AppHandle, MemoryFunctions, 0, &attr);
    result = BER_SearchTree(tree, Oid);
    DER_RecycleTree(AppHandle, MemoryFunctions, tree);

    return result;
}
