/*-----------------------------------------------------------------------
 * File: INTS2OID.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"

/* encode an array of unsigned long to a DER object identifier octets */
sint32 
DER_EncodeOid(              /* return number of bytes used to encode it */
    const uint32 *Numbers,        /* the array of integers */
    sint32 Dimension,       /* number of integers in OID */
    uint8 *Octets)          /* output buffer */
{
    sint32 size, i, j, result;
    uint32 value;

    if (Dimension < 2) 
        return 0;
    /* precompute the sizes */
    size = 1;   /* tag plus first two numbers */
    for (i = 2; i < Dimension; i++) {
        for (j = 1; j < 5; j++)
            if ((1UL << (j * 7)) > Numbers[i]) break;
        size += j;
    }
    result = 1 + BER_LengthOfLength(size) + size;
    /* now construct the DER */
    *Octets++ = BER_OBJECT_IDENTIFIER;
    i = BER_OutputLength(size, Octets);
    Octets += i;
    *Octets++ = (uint8) (Numbers[0] * 40 + Numbers[1]);
    for (i = 2; i < Dimension; i++) {
        value = Numbers[i];
        for (j = 4; j > 0; j--) 
            if ((1UL << (j * 7)) <= value) break;
        if (j == 4) {   /* special for MSFT C++ bug */
            *Octets++ = (uint8) (((value >> 28) & 15) | 128);
            j--;
        }
        while (j > 0) {
            *Octets++ = (uint8) (((value >> (j * 7)) & 127) | 128);
            j--;
        }
        *Octets++ = (uint8) (value & 127);
    }

    return result;
}
