/*-----------------------------------------------------------------------
 * File: ALGOIDS.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
#include "cssm.h"
#include "oidsalg.h"
#include "algoid.h"
#include "cssmport.h"
#include "clutil.h"
#include "guids.h"
#include <cssmMemTrack.h>

typedef struct {
    CSSM_OID_PTR AlgOID;
    CSSM_ALGORITHMS AlgID;
} CSSM_OID_ID;


/*-----------------------------------------------------------------------------
 * Name: CSSM_CLUTIL_X509_AlgIdToAlgorithmOid
 *
 * Description:
 * This function maps the specified AlgID to the corresponding
 * Algorithm OID.
 *
 * Parameters: 
 * alg_int - AlgID to be mapped.
 *
 * Return value:
 * Pointer to OID structure and NULL in case of failure.
 * 
 *---------------------------------------------------------------------------*/
CSSM_OID_PTR 
CSSM_CLUTIL_X509_AlgIdToAlgorithmOid(CSSM_ALGORITHMS alg_int)
{
    return X509_AlgIdToAlgorithmOid(alg_int);
}

/*-----------------------------------------------------------------------------
 * Name: CSSM_CLUTIL_X509_AlgorithmOidToAlgId
 *
 * Description:
 * This function maps the specified Algorithm OID to the corresponding
 * AlgID.
 *
 * Parameters: 
 * Oid - OID to be mapped.
 *
 * Return value:
 * Algorithm ID and CSSM_ALGID_NONE in case of failures.
 * 
 *---------------------------------------------------------------------------*/
CSSM_ALGORITHMS CSSM_CLUTIL_X509_AlgorithmOidToAlgId (CSSM_OID_PTR Oid)
{
    return X509_AlgorithmOidToAlgId(Oid);
}

/*-----------------------------------------------------------------------------
 * Name: CSSM_CLUTIL_X509_IsEqualOid
 *
 * Description:
 * This function checks for the equality of specified OID"s.
 *
 * Parameters:
 * Oid1 - First OID to be compared.
 * Oid2 - Second OID to be compared.
 *
 * Return value:
 * CSSM_TRUE in case of equal oid's, CSSM_FALSE otherwise.
 * 
 * Error Codes:
 * None
 *---------------------------------------------------------------------------*/
CSSM_BOOL 
CSSM_CLUTIL_X509_IsEqualOid(const CSSM_OID *Oid1, const CSSM_OID *Oid2)
{
    return X509_IsEqualOid(Oid1, Oid2);
}
