/*-----------------------------------------------------------------------
 * File: API_SHIM.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/* Implements MAF style API shims to call CSSM API functions for the 
 * proper CSSM.
 */

/* MAF stuff */
#include "maf_include.h"



/* Generic CSSM stuff */
#include "cssm.h"
#include "cssmspi.h"
#include "cssmcli.h"

/* Local headers */

#ifdef ISL_INCLUDED
#include "eislapi.h"
#endif

#include <cssmMemTrack.h>

CSSM_RETURN 
addin_SignData (CSSM_CC_HANDLE CCHandle,
               const CSSM_DATA *DataBufs,
               uint32 DataBufCount,
               CSSM_ALGORITHMS DigestAlgorithm,
               CSSM_DATA_PTR Signature)
{
#if (ADDIN_NEED_LOAD_WRITER_LOCK == 0)
    const
#endif
    ADDIN_LOAD_DATA *pLoadData = NULL;

    /* Fetch the function pointer and check validity */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        return CSSMERR_CSSM_MODULE_NOT_LOADED;
    }

    /* Call through the pointer */
    return pLoadData->SignData (CCHandle,
                                DataBufs,
                                DataBufCount,
                                DigestAlgorithm,
                                Signature);
}

CSSM_RETURN 
addin_VerifyData (CSSM_CC_HANDLE CCHandle,
                   const CSSM_DATA *DataBufs,
                   uint32 DataBufCount,
                   CSSM_ALGORITHMS DigestAlgorithm,
                   const CSSM_DATA *Signature)
{
#if (ADDIN_NEED_LOAD_WRITER_LOCK == 0)
    const
#endif
    ADDIN_LOAD_DATA *pLoadData = NULL;

    /* Fetch the function pointer and check validity */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        return CSSMERR_CSSM_MODULE_NOT_LOADED;
    }

    /* Call through the pointer */
    return pLoadData->VerifyData (CCHandle,
                                  DataBufs,
                                  DataBufCount,
                                  DigestAlgorithm,
                                  Signature);
}

CSSM_RETURN 
addin_DigestData (CSSM_CC_HANDLE CCHandle,
                  const CSSM_DATA *DataBufs,
                  uint32 DataBufCount,
                  CSSM_DATA_PTR Digest)
{
#if (ADDIN_NEED_LOAD_WRITER_LOCK == 0)
    const
#endif
    ADDIN_LOAD_DATA *pLoadData = NULL;

    /* Fetch the function pointer and check validity */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        return CSSMERR_CSSM_MODULE_NOT_LOADED;
    }

    /* Call through the pointer */
    return pLoadData->DigestData (CCHandle,
                                  DataBufs,
                                  DataBufCount,
                                  Digest);
}


CSSM_RETURN 
addin_DeleteContext (CSSM_CC_HANDLE CCHandle)
{
#if (ADDIN_NEED_LOAD_WRITER_LOCK == 0)
    const
#endif
    ADDIN_LOAD_DATA *pLoadData = NULL;

    /* Fetch the function pointer and check validity */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        return CSSMERR_CSSM_MODULE_NOT_LOADED;
    }

    /* Call through the pointer */
    return pLoadData->DeleteContext (CCHandle);
}

CSSM_RETURN
addin_GetContext (CSSM_CC_HANDLE CCHandle,
                  CSSM_CONTEXT_PTR *Context)
{
#if (ADDIN_NEED_LOAD_WRITER_LOCK == 0)
    const
#endif
    ADDIN_LOAD_DATA *pLoadData = NULL;

    /* Fetch the function pointer and check validity */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        return CSSMERR_CSSM_MODULE_NOT_LOADED;
    }

    /* Call through the pointer */
    return pLoadData->GetContext (CCHandle, Context);
}

CSSM_RETURN 
addin_FreeContext (CSSM_CONTEXT_PTR Context)
{
#if (ADDIN_NEED_LOAD_WRITER_LOCK == 0)
    const
#endif
    ADDIN_LOAD_DATA *pLoadData = NULL;

    /* Fetch the function pointer and check validity */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        return CSSMERR_CSSM_MODULE_NOT_LOADED;
    }

    /* Call through the pointer */
    return pLoadData->FreeContext (Context);
}
 


/* CSP interfaces */
CSSM_RETURN 
addin_csp_CreateSignatureContext (CSSM_CSP_HANDLE CSPHandle,
                                   uint32 AlgorithmID,
                                   const CSSM_ACCESS_CREDENTIALS *AccessCred,
                                   const CSSM_KEY *Key,
                                   CSSM_CC_HANDLE *NewContextHandle )
{
#if (ADDIN_NEED_LOAD_WRITER_LOCK == 0)
    const
#endif
    ADDIN_LOAD_DATA *pLoadData = NULL;

    /* Fetch the function pointer and check validity */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        return CSSMERR_CSSM_MODULE_NOT_LOADED;
    }

    /* Call through the pointer */
    return pLoadData->csp_CreateSignatureContext (CSPHandle,
                                                  AlgorithmID,
                                                  AccessCred,
                                                  Key,
                                                  NewContextHandle);
}

CSSM_RETURN 
addin_csp_CreateDigestContext (CSSM_CSP_HANDLE CSPHandle,
                               CSSM_ALGORITHMS AlgorithmID,
                               CSSM_CC_HANDLE *NewContextHandle)
{
#if (ADDIN_NEED_LOAD_WRITER_LOCK == 0)
    const
#endif
    ADDIN_LOAD_DATA *pLoadData = NULL;

    /* Fetch the function pointer and check validity */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        return CSSMERR_CSSM_MODULE_NOT_LOADED;
    }

    /* Call through the pointer */
    return pLoadData->csp_CreateDigestContext (CSPHandle,
                                                  AlgorithmID,
                                                  NewContextHandle);
}
