/*-----------------------------------------------------------------------
 * File: CERTUP2C.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
#include "maf_include.h"
#include "cssm.h"
#include "cssmport.h"
#include "internal.h"
#include "spkiutil.h"

/***************************************************************************/
/* certup2c                                                                */
/*                                                                         */
/* Take a Cert tuple, and form a packed canonical S-expression ACL from it.*/
/* Then store that in a CSSM_ENCODED_CERT structure                        */
/***************************************************************************/

CSSM_RETURN certup2c( CSSM_TP_HANDLE TPHandle,
                      CSSM_TUPLE_PTR certuple, 
                      CSSM_ENCODED_CERT *certptr)
{
    uint32 size = 11 ;            /* to count the size of the output */
                                /* init with room for (4:cert) & slop */
    uint8 *w ;                    /* memory walker */

    if (certuple == NULL)
    {
        return ( CSSMERR_TP_INVALID_INPUT_POINTER ) ;
    }
    if (certptr == NULL)
    {
        return ( CSSMERR_TP_INVALID_OUTPUT_POINTER ) ;
    }

    /* init result */
    port_memset(certptr, 0, sizeof(CSSM_ENCODED_CERT));

    size += size_tuple(certuple) ;

    w = certptr->CertBlob.Data = (uint8 *) App_Malloc( TPHandle, size ) ;

    if (!w) 
    {
        return ( CSSMERR_TP_MEMORY_ERROR ) ;
    }
    port_memcpy( w, "(4:cert", 7 ) ; 
    w += 7 ; /* init the output */

    pack_certuple( &w, certuple ) ;
    *(w++) = ')' ;
    certptr->CertBlob.Length = w - certptr->CertBlob.Data ; /* note the length */
    certptr->CertType = CSSM_CERT_SPKI;
    certptr->CertEncoding = CSSM_CERT_ENCODING_SEXPR;
    return ( CSSM_OK ) ;
} /* certup2c */



