/*-----------------------------------------------------------------------
 * File: SPKI_MAF.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* A SPKI CL/TP Module: spkicl
 *
 * This module provides stub functions for the services in all the service categories.
 */

#define CSSM_LEAN
#include "cssm.h"
#include "cssmport.h"
#include "cssmlock.h"
#include "cssmspi.h"
#include "maf_config.h"
#include "maf_integ.h"
#include "maf_collectn.h"
#include "maf_util.h"
#include "maf_interface.h"
#include "mempool.h"

CSSM_GUID intel_spki_clm_guid_3_0_0 = intel_spki_clm_guid_3_0_0_init;

/* Declare the custom attach functions for each service category. */
CSSM_RETURN CSSMAPI cl_ModuleAttach(CSSM_MODULE_FUNCS_PTR *FuncTbl);
CSSM_RETURN CSSMAPI tp_ModuleAttach(CSSM_MODULE_FUNCS_PTR *FuncTbl);

PORT_TLS_INDEX TLS_MemoryPool;
MEMORY_POOL pool;
MEMORY_POOL_PTR poolptr = &pool;
extern MEMORY_POOL_FUNCS memory_pool_funcs;
extern const CSSM_API_MEMORY_FUNCS Addin_APIMemFuncs;
/*-----------------------------------------------------------------------------
 * Name: Addin_callout_Initialize
 *
 * Description:
 * Preforms initialization that takes places when the module is loaded into
 * the process space
 *
 * Parameters: 
 * None
 * 
 * Return Code:
 * CSSM_OK if successful, otherwise CSSMERR_TP_INTERNAL_ERROR
 *---------------------------------------------------------------------------*/
CSSM_RETURN
Addin_callout_Initialize(void)
{
    CSSM_RETURN ret;
    ret = port_AllocateTlsIndex(&TLS_MemoryPool, NULL);
    if (CSSM_OK != ret)
    {
        return ret;
    }
    ret = (*memory_pool_funcs.init)((CSSM_MEMORY_FUNCS_PTR)&Addin_APIMemFuncs, NULL, poolptr);
    return ret;
    
}

/*-----------------------------------------------------------------------------
 * Name: Addin_callout_Terminate
 *
 * Description:
 * Terminates state before the module is unloaded from the process space
 *
 * Parameters: 
 * None.
 * 
 * Return Code:
 * CSSM_OK if successful, otherwise CSSMERR_TP_INTERNAL_ERROR
 *---------------------------------------------------------------------------*/
CSSM_RETURN
Addin_callout_Terminate(void)
{
    (*memory_pool_funcs.recycle)(poolptr);
    return port_DeleteTlsIndex(TLS_MemoryPool);
}

/*-------------------------------------------------------------------------
 * Name: Addin_callout_LoadDataConstruct
 *
 * Description:
 * Constructs a ADDIN_LOAD_DATA structure
 *
 * Parameters: 
 * pAddinLoadData (output) : variable to initialize
 * 
 * Note:
 * If this function fails, Addin_callout_LoadDataDestroy will still be
 * called.
 *
 * Return Code:
 * CSSM_OK if successful, otherwise CSSMERR_TP_INTERNAL_ERROR
 *-------------------------------------------------------------------------
 */
CSSM_RETURN Addin_callout_LoadDataConstruct(
						ADDIN_MODULE_HANDLE hCssm,
						ADDIN_LOAD_DATA * pAddinLoadData )
{
	if (!pAddinLoadData)
	{
		goto FAIL;
	}
	
	/* obtain needed CSSM entry points */
	/* CSSM API */
	if( Addin_GetProcAddress( hCssm,
							  "CSSM_SignData",
							  (CSSM_PROC_ADDR*)&pAddinLoadData->SignData ) 
		!= CSSM_OK )
		goto FAIL;
	if( Addin_GetProcAddress( hCssm,
							  "CSSM_VerifyData",
							  (CSSM_PROC_ADDR*)&pAddinLoadData->VerifyData ) 
		!= CSSM_OK )
		goto FAIL;	
	if( Addin_GetProcAddress( hCssm,
							  "CSSM_DigestData",
							  (CSSM_PROC_ADDR*)&pAddinLoadData->DigestData ) 
		!= CSSM_OK )
		goto FAIL;	
	
    if( Addin_GetProcAddress( hCssm,
							  "CSSM_DeleteContext",
							  (CSSM_PROC_ADDR*)&pAddinLoadData->DeleteContext ) 
		!= CSSM_OK )
		goto FAIL;	 
	if( Addin_GetProcAddress( hCssm,
							  "CSSM_GetContext",
							  (CSSM_PROC_ADDR*)&pAddinLoadData->GetContext ) 
		!= CSSM_OK )
		goto FAIL;	 
	if( Addin_GetProcAddress( hCssm,
							  "CSSM_FreeContext",
							  (CSSM_PROC_ADDR*)&pAddinLoadData->FreeContext ) 
		!= CSSM_OK )
		goto FAIL;	 
	if( Addin_GetProcAddress( hCssm, 
							  "CSSM_CSP_CreateSignatureContext",
							  (CSSM_PROC_ADDR*)&pAddinLoadData->csp_CreateSignatureContext ) 
		!= CSSM_OK )
		goto FAIL;
	if( Addin_GetProcAddress( hCssm, 
							  "CSSM_CSP_CreateDigestContext",
							  (CSSM_PROC_ADDR*)&pAddinLoadData->csp_CreateDigestContext ) 
		!= CSSM_OK )
		goto FAIL;

	return CSSM_OK;

FAIL:
	if (pAddinLoadData)
	{
		/* Clear the structure, with a known bad value (for debugging) */
		port_memset( pAddinLoadData, 0x33, sizeof(*pAddinLoadData) );
	}

	return CSSMERR_TP_INTERNAL_ERROR;
}

/*-------------------------------------------------------------------------
 * Name: Addin_callout_LoadDataDestroy
 *
 * Description:
 * Constructs a ADDIN_LOAD_DATA structure
 *
 * Parameters: 
 * pAddinLoadData (output) : ADDIN_LOAD_DATA structure to destroy
 * 
 * Return Code:
 * none
 *-------------------------------------------------------------------------
 */
void Addin_callout_LoadDataDestroy(
						ADDIN_LOAD_DATA * pAddinLoadData )
{
	if (pAddinLoadData)
	{
		/* Clear the structure */
		port_memset( pAddinLoadData, 0, sizeof(*pAddinLoadData) );
	}

	return;
}

/*-------------------------------------------------------------------------
 * Name: Addin_callout_ModuleLoad
 *
 * Description:
 * Call used to indicate that the application has requested a module load
 *
 * Parameters: 
 * pLoadTracker (input/output) : The load tracker node that was created. 
 *  This node may be writen to.
 * CssmNotifyCallback (input) : Callback function to indicate to CSSM that
 *  a subservice is ready for attach (must be called before an attach can 
 *  occurr).
 * CssmNotifyCallbackCtx (input) : Opaque data to pass to the callback
 *  function.
 * 
 * Return Code:
 * CSSM_OK if successful, otherwise CSSMERR_TP_INTERNAL_ERROR
 *-------------------------------------------------------------------------
 */
CSSM_RETURN Addin_callout_ModuleLoad(
						MAF_MODULE_LOAD_TRACKER_PTR pLoadTracker,
						CSSM_SPI_ModuleEventHandler CssmNotifyCallback,
						const void* CssmNotifyCallbackCtx )
{
    if (CssmNotifyCallback)
    {
        CssmNotifyCallback(&ADDIN_GUID, (void *)CssmNotifyCallbackCtx, 0, CSSM_SERVICE_CL|CSSM_SERVICE_TP,
                           CSSM_NOTIFY_INSERT);
    }

    return CSSM_OK;
}

/*-------------------------------------------------------------------------
 * Name: Addin_callout_ModuleUnload
 *
 * Description:
 * Function is called as part of the unload process to give indication to
 * the addin.
 *
 * Parameters: 
 * pLoadTracker (input/output) : The load tracker node that was created. 
 *  This node may be written to.
 * CssmNotifyCallback (input) : Identifies the corresponding ModuleLoad 
 *  call CssmNotifyCallbackCtx (input) : Identifies the corresponding 
 *  ModuleLoad call
 * 
 * Note: 
 * There is no need to call the callback function on unload.
 *
 * Return Code:
 * CSSM_OK if successful, otherwise CSSMERR_TP_INTERNAL_ERROR
 *-------------------------------------------------------------------------
 */
CSSM_RETURN Addin_callout_ModuleUnload(
						MAF_MODULE_LOAD_TRACKER_PTR pLoadTracker,
						CSSM_SPI_ModuleEventHandler CssmNotifyCallback,
						const void* CssmNotifyCallbackCtx )
{
    return CSSM_OK;
}

/*-------------------------------------------------------------------------
 * Name: Addin_callout_ModuleAttach
 *
 * Description:
 * Functions is called to indicate that a module attach has been reqested. 
 *
 * Parameters: 
 * pLoadTracker (input/output) : An initialized load tracker, the 
 *  ADDIN_LOAD_DATA structure can be modified
 * pAttachTracker (input/output) : An construcuted attach tracker, the 
 *  ADDIN_ATTACH_DATA structure can be modified
 * FuncTbl (outpt) : Buffer to hold the SPI functions that the addin wants
 * to register with CSSM
 * 
 * Return Code:
 * CSSM_OK if successful, otherwise CSSMERR_TP_INTERNAL_ERROR
 *-------------------------------------------------------------------------
 */
CSSM_RETURN Addin_callout_ModuleAttach(
						MAF_MODULE_LOAD_TRACKER_PTR pLoadTracker, 
						MAF_MODULE_ATTACH_TRACKER_PTR pAttachTracker,
						CSSM_MODULE_FUNCS_PTR *FuncTbl )
{
    if (!pAttachTracker)
    {
        return CSSMERR_TP_INVALID_POINTER;
    }

    switch (pAttachTracker->SubServiceType)
    {
    case CSSM_SERVICE_CL:  return cl_ModuleAttach(FuncTbl);
    case CSSM_SERVICE_TP:  return tp_ModuleAttach(FuncTbl);
    default:               return CSSMERR_CSSM_INVALID_SUBSERVICEID;
    }
    return CSSMERR_TP_INTERNAL_ERROR;
}
