/*-----------------------------------------------------------------------
 * File: CRTGETNE.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
#include "maf_include.h"
    /* Windows & CSSM includes */
#ifndef VMS
#pragma warning (disable:4201 4214 4115)
#include <stdio.h>
#pragma warning (default:4201 4214 4115)
#else
#include <stdio.h>
#endif
#include "cssm.h"
#include "cssmspi.h"
#include "guids.h"
#include "cssmport.h"
#include "cssmutil.h"

    /* Certificate-specific includes */
#include "x509defs.h"
#include "x_fndefs.h"

#include "oidsx509.h"
#include "oidsalg.h"
#include "x509hlpr.h"
#include "clutil.h"
#include "x_qtable.h"
#include "internal.h"

#include "api_shim.h"
#include "maf_include.h"

#include <cssmMemTrack.h>

#define X509_FORMAT_VERSION 2

/*-----------------------------------------------------------------------------
 * Name: CL_CertGetNextFieldValue
 *
 * Description:
 * This function returns the next element in the array of elements that matched
 * the initial query.
 *
 * Parameters: 
 * ResultsHandle (input) : The handle used to identify the desired query results
 *
 * result_val (output):
 * The next element in the array of elements that matched the initial query.
 *---------------------------------------------------------------------------*/
CSSM_RETURN CSSMCLI CL_CertGetNextFieldValue (CSSM_CL_HANDLE CLHandle,
                                                CSSM_HANDLE ResultsHandle,
                                                CSSM_DATA_PTR *result_val)
{
    CSSM_DATA_PTR datap = NULL;
    CSSM_RETURN ret = CSSM_OK;

    CSSM_CLUTIL_CONTEXT *contextPtr;
    
    ADDIN_ATTACH_DATA *pAttachData = NULL;

    /* Check to see that we know our execution environment */
    pAttachData = (ADDIN_ATTACH_DATA *)Addin_GetAttachData();
    if ( pAttachData == NULL )
    {
      return CSSMERR_CL_INTERNAL_ERROR;
  
    } 

    contextPtr = (CSSM_CLUTIL_CONTEXT *)pAttachData;
    /* Obtain the next value from a previous query */
    if (port_IsBadWritePtr (result_val, sizeof(CSSM_DATA_PTR)))
    {
        return CSSMERR_CL_INVALID_OUTPUT_POINTER;
    }
    *result_val = NULL;
    ret = cl_QT_GetNextResult(contextPtr, CLHandle, ResultsHandle, &datap);

    if(CSSM_OK != ret)
    {
        return ret;
    }
    else
    {
        if (datap == NULL)
        {
            return CSSMERR_CL_INTERNAL_ERROR;
        }
    }

    *result_val = datap;
    return CSSM_OK;
}

