/*-----------------------------------------------------------------------
 * File: FUNCPTRS.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

/* This header should only be included by the maf_config.h file to
 * complete the declaration of the ADDIN_LOAD_DATA structure.
 *
/* CSSM interfaces */


CSSM_RETURN 
(CSSMAPI *GetAPIMemoryFunctions) (CSSM_HANDLE AddInHandle,
                                      CSSM_API_MEMORY_FUNCS_PTR AppMemoryFuncs);

CSSM_RETURN 
(CSSMAPI *GenerateKeyPair) (CSSM_CC_HANDLE CCHandle,
                                          uint32 PublicKeyUsage,
                                          uint32 PublicKeyAttr,
                                          const CSSM_DATA *PublicKeyLabel,
                                          CSSM_KEY_PTR PublicKey,
                                          uint32 PrivateKeyUsage,
                                          uint32 PrivateKeyAttr,
                                          const CSSM_DATA *PrivateKeyLabel,
                                          const CSSM_RESOURCE_CONTROL_CONTEXT *CredAndAclEntry,
                                          CSSM_KEY_PTR PrivateKey);
CSSM_RETURN 
(CSSMAPI *QueryKeySizeInBits) (CSSM_CSP_HANDLE CSPHandle,
                                   CSSM_CC_HANDLE CCHandle,
                                   const CSSM_KEY *Key,
                                   CSSM_KEY_SIZE_PTR KeySize);



CSSM_RETURN 
(CSSMAPI *SignData) (CSSM_CC_HANDLE CCHandle,
                                   const CSSM_DATA *DataBufs,
                                   uint32 DataBufCount,
                                   CSSM_ALGORITHMS DigestAlgorithm,
                                   CSSM_DATA_PTR Signature);
CSSM_RETURN 
(CSSMAPI *VerifyData) (CSSM_CC_HANDLE CCHandle,
                                   const CSSM_DATA *DataBufs,
                                   uint32 DataBufCount,
                                   CSSM_ALGORITHMS DigestAlgorithm,
                                   const CSSM_DATA *Signature);

CSSM_RETURN 
(CSSMAPI *DeleteContext) (CSSM_CC_HANDLE CCHandle);

CSSM_RETURN
(CSSMAPI *GetContext) (CSSM_CC_HANDLE CCHandle, 
                                    CSSM_CONTEXT_PTR *Context);

CSSM_RETURN 
(CSSMAPI *FreeContext) (CSSM_CONTEXT_PTR Context);

CSSM_RETURN 
(CSSMAPI *ModuleLoad) (const CSSM_GUID *ModuleGuid,
                           CSSM_KEY_HIERARCHY KeyHierarchy,
                           CSSM_API_ModuleEventHandler AppNotifyCallback,
                           const void* AppNotifyCallbackCtx);

CSSM_RETURN 
(CSSMAPI *ModuleUnload) (const CSSM_GUID *ModuleGuid,
                              CSSM_API_ModuleEventHandler AppNotifyCallback,
                              const void* AppNotifyCallbackCtx);

CSSM_RETURN 
(CSSMAPI *ModuleAttach) (const CSSM_GUID *ModuleGuid,
                        const CSSM_VERSION *Version,
                        const CSSM_API_MEMORY_FUNCS *MemoryFuncs,
                        uint32 SubserviceID,
                        CSSM_SERVICE_TYPE SubServiceType,
                        CSSM_ATTACH_FLAGS AttachFlags,
                        CSSM_KEY_HIERARCHY KeyHierarchy,
                        CSSM_FUNC_NAME_ADDR *FunctionTable,
                        uint32 NumFunctionTable,
                        const void *Reserved,
                        CSSM_MODULE_HANDLE_PTR NewModuleHandle );

CSSM_RETURN 
(CSSMAPI *ModuleDetach) (CSSM_MODULE_HANDLE ModuleHandle);


/* CSP interfaces */
CSSM_RETURN 
(CSSMAPI *csp_CreateSignatureContext )(CSSM_CSP_HANDLE CSPHandle,
                                           uint32 AlgorithmID,
                                           const CSSM_ACCESS_CREDENTIALS *AccessCred,
                                           const CSSM_KEY *Key,
                                           CSSM_CC_HANDLE *NewContextHandle );

CSSM_RETURN 
(CSSMAPI *csp_ObtainPrivateKeyFromPublicKey) (CSSM_CSP_HANDLE CSPHandle,
                                                  const CSSM_KEY *PublicKey,
                                                  CSSM_KEY_PTR PrivateKey);

CSSM_RETURN 
(CSSMAPI *csp_CreateKeyGenContext) (  CSSM_CSP_HANDLE CSPHandle,
                                          CSSM_ALGORITHMS AlgorithmID,
                                          uint32 KeySizeInBits,
                                          const CSSM_CRYPTO_DATA *Seed,
                                          const CSSM_DATA *Salt,
                                          const CSSM_DATE *StartDate,
                                          const CSSM_DATE *EndDate,
                                          const CSSM_DATA *Params,
                                          CSSM_CC_HANDLE *NewContextHandle );

CSSM_RETURN 
(CSSMAPI *GetContextAttribute) (const CSSM_CONTEXT *Context,
                                    uint32 AttributeType,
                                    CSSM_CONTEXT_ATTRIBUTE_PTR *ContextAttribute );

CSSM_RETURN 
(CSSMAPI *DeleteContextAttributes) (CSSM_CC_HANDLE CCHandle,
                                uint32 NumberAttributes,
                                const CSSM_CONTEXT_ATTRIBUTE *ContextAttributes);
