/*-----------------------------------------------------------------------
 * File: X509HLPR.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

#ifndef _X509HLPR_H
#define _X509HLPR_H    

#include "cssm.h"
#include "cssmport.h"
#include "x509defs.h"

#ifdef __cplusplus
extern "C" {
#endif

CSSM_BOOL x509_cl_IsEqualCssmData( const CSSM_DATA *Data1, const CSSM_DATA *Data2);
CSSM_BOOL x509_cl_FreeCssmKey(CSSM_HANDLE AppHandle,
                              CSSM_KEY_PTR KeyPtr);
uint32 x509_cl_CopyCssmData(CSSM_HANDLE AppHandle,
                                 const CSSM_DATA *Src,
                                 CSSM_DATA_PTR Dest);
CSSM_RETURN x509_cl_CopyCssmDataToOneField(CSSM_HANDLE AppHandle,
                                              const CSSM_DATA *Message,
                                              uint32 *NumberOfFields,
                                              CSSM_FIELD_PTR *result_val);
CSSM_RETURN x509_cl_CopyOneFieldToCssmData(CSSM_HANDLE AppHandle,
                                             CSSM_FIELD_PTR Fields,
                                             CSSM_DATA_PTR *result_val);

CSSM_SUBSERVICE_UID_PTR x509_cl_FindCSPSupportingThisAlg(CSSM_HANDLE AppHandle,
                                                         CSSM_UPCALLS_PTR MemoryFunctions, 
                                                         CSSM_ALGORITHMS AlgorithmId,
                                                         uint32 EffectiveKeySizeInBits,
                                                         CSSM_USEE_TAG *Tag);

CSSM_RETURN
x509_cl_FindCSPUseeTagSupportingThisAlgAndKeysize(CSSM_HANDLE AppHandle,
                                                  CSSM_UPCALLS_PTR MemoryFunctions, 
                                                  CSSM_ALGORITHMS AlgorithmId, 
                                                  uint32 EffectiveKeySizeInBits,
                                                  CSSM_SUBSERVICE_UID *CSPUid,
                                                  CSSM_USEE_TAG *Tag);

CSSM_X509_SUBJECT_PUBLIC_KEY_INFO_PTR x509_cl_CSSMKeyToSpki  (CSSM_HANDLE AppHandle, 
                                                         CSSM_UPCALLS_PTR MemoryFunctions, 
                                                         CSSM_KEY_PTR CSSMKey);

CSSM_RETURN            x509_cl_SpkiToCSSMKey           (CSSM_HANDLE AppHandle, 
                                                         CSSM_UPCALLS_PTR MemoryFunctions, 
                                                         CSSM_X509_SUBJECT_PUBLIC_KEY_INFO_PTR SPKI,
                                                         CSSM_KEY_PTR *key_ptr);

CSSM_RETURN               x509_cl_GetKeyDataAndParams     (CSSM_CL_HANDLE AppHandle, 
                                                         CSSM_UPCALLS_PTR MemoryFunctions, 
                                                         CSSM_KEY_PTR  KeyPtr, 
                                                         CSSM_DATA_PTR KeyDataPtr, 
                                                         CSSM_DATA_PTR ParamsPtr);

/* This routine creates a DER-encoded X509 distinguished name structure */
CSSM_RETURN x509_cl_CreateDerEncodedName   
                                 (CSSM_HANDLE AppHandle, 
                                  CSSM_UPCALLS_PTR MemoryFunctions, 
                                  const CSSM_X509_TYPE_VALUE_PAIR *DNameTemplate, 
                                  uint32 NumberOfFields,
                                  CSSM_DATA_PTR *result_val);

CSSM_RETURN x509_cl_CopyAlgId   (CSSM_HANDLE AppHandle, 
                                  CSSM_UPCALLS_PTR MemoryFunctions, 
                                  CSSM_X509_ALGORITHM_IDENTIFIER_PTR AlgIdIn,
                                  CSSM_X509_ALGORITHM_IDENTIFIER_PTR AlgIdOut);

/* Write a Certificate to a flat file */
CSSM_RETURN x509_cl_WriteCertificateToFile(FILE *FilePtr, 
                                           CSSM_DATA_PTR Cert );

/* Read a Certificate from a flat file */
CSSM_RETURN x509_cl_ReadCertificateFromFile(CSSM_HANDLE AppHandle, 
                                            CSSM_UPCALLS_PTR MemoryFunctions, 
                                            FILE *FilePtr,
                                            CSSM_DATA_PTR *result_val);

CSSM_RETURN x509_cl_GetLeafCertCopy(CSSM_HANDLE AppHandle,
                              CSSM_UPCALLS_PTR MemoryFunctions,
                              CSSM_CERTGROUP_PTR CertGroup,
                              CSSM_DATA_PTR *result_val);


CSSM_BOOL cl_IsBadMemFuncsPtr (CSSM_UPCALLS_PTR MemoryFunctions);

void
x509_getToBeSignedCertificate(const CSSM_DATA *SubjectCert, CSSM_DATA *TBSCert);

#ifdef __cplusplus
}
#endif

#endif  /* ndef _X509HLPR_H */
