/*-----------------------------------------------------------------------
 * File: GETKACL.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "cspinc.h"

CSSM_RETURN CSSMAPI ADDIN_CSM_GetKeyAcl(
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_KEY *Key,
                        const CSSM_STRING *SelectionTag,
                        uint32 *NumberOfAclInfos,
                        CSSM_ACL_ENTRY_INFO_PTR *AclInfos )
{
    CSSM_RETURN rv = CSSM_OK;
    CSM_KM_KEY_HANDLE hKey = CSSM_INVALID_HANDLE;
    const ADDIN_LOAD_DATA *pLoadData = NULL;

    pLoadData = Addin_GetLoadData();
    assert( pLoadData );

    /* Make sure the key is a reference of some sort */
    if ( Key->KeyHeader.BlobType != CSSM_KEYBLOB_REFERENCE )
    {
        ERR( rv = CSSMERR_CSP_INVALID_KEY_FORMAT );
    }
    else
    {
        /* Find the key handle */
        if ( Key->KeyHeader.Format == CSSM_KEYBLOB_REF_FORMAT_INTEGER )
        { /* Integer reference */
            if ( Key->KeyData.Length != sizeof(CSM_KM_KEY_HANDLE) )
            {
                ERR( rv = CSSMERR_CSP_INVALID_KEY_REFERENCE );
            }
            else
            {
                hKey = *(uint32*)Key->KeyData.Data;
            }
        } /* Integer reference */
        else if ( Key->KeyHeader.Format == CSSM_KEYBLOB_REF_FORMAT_STRING )
        { /* Name reference */
            rv = CSM_KMFindKeyByName( pLoadData->hKeyFile,
                                      &Key->KeyData,
                                      NULL,
                                      NULL,
                                      NULL,
                                      &hKey );
        } /* Name reference */
        else
        { /* Unknown reference type */
            ERR( rv = CSSMERR_CSP_INVALID_KEY_FORMAT  );
        } /* Unknown reference type */
        if ( rv == CSSM_OK )
        {
            /* Have the key manager fetch the ACL information */
            rv = CSM_KMGetKeyAcl( CSPHandle,
                                  pLoadData->hKeyFile,
                                  hKey,
                                  SelectionTag,
                                  AclInfos,
                                  NumberOfAclInfos );
        } /* Find key handle */
    } /* Ensure key reference */

    return rv;
}

