/*-----------------------------------------------------------------------
 * File: MAF_CONFIG.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef MAF_CONFIG_H
#define MAF_CONFIG_H

#include "CSM/csm_sesn.h"
#include "guids.h"
#include "csm_keymgr.h"

#define MAF_CSM_INCLUDED /* Include MAF-CSM headers (CSP) */
#define MAF_NEED_KEYPARSE

/* GUID of the addin module */
#define ADDIN_GUID_INITIALIZER              intel_bsafecsp_guid_init

/* Module name of the service provider */
#define ADDIN_NAME                          "maf_bsafe.dll"

/* Manifest section name of the service provider */
#ifdef VMS
#define ADDIN_SELF_CHECK_SECTION            "cdsa$maf_bsafe_shr"
#else
#define ADDIN_SELF_CHECK_SECTION            "maf_bsafe"
#endif

/* Module name of the service provider */
#define ADDIN_PRINT_NAME                    "CDSA Adaptation Layer CSP for the BSafe Toolkit from RSA DSI"
#ifdef VMS
#define ADDIN_VENDOR_NAME                   "Compaq Computer Corporation"
#else
#define ADDIN_VENDOR_NAME                   "Intel Corporation"
#endif
#define ADDIN_STANDARD_DESCRIPTION          "CDSA Adaptation Layer CSP"
#define ADDIN_STANDARD_VERSION_MAJOR        3
#define ADDIN_STANDARD_VERSION_MINOR        1
#define ADDIN_PRODUCT_DESCRIPTION           ADDIN_STANDARD_DESCRIPTION
#define ADDIN_PRODUCT_VERSION_MAJOR         ADDIN_STANDARD_VERSION_MAJOR
#define ADDIN_PRODUCT_VERSION_MINOR         ADDIN_STANDARD_VERSION_MINOR

#define ADDIN_ENCAPSULATED_VENDOR_NAME      "RSA Data Security"
#define ADDIN_ENCAPSULATED_DESCRIPTION      "BSafe Toolkit"
#define ADDIN_ENCAPSULATED_VERSION_MAJOR    4
#define ADDIN_ENCAPSULATED_VERSION_MINOR    2

/* Macro used to define if a subservice pair (id, type) is valid for an addin */
#define ADDIN_CSP_SUBSERVICE_ID             0

/* Major version of the addin module */
#define ADDIN_VERSION_MAJOR                 CSSM_MAJOR

/* Minor version of the addin module */
#define ADDIN_VERSION_MINOR                 CSSM_MINOR

/* Is the addin module thread safe? */
#define ADDIN_THREAD_SAFE                   1

/* Does the addin module require the writer lock durring SPI functions on the load 
tracker node (is this node being modified?) */
#define ADDIN_NEED_LOAD_WRITER_LOCK         0

/* Does the addin module require the writer lock durring SPI functions on the attach 
tracker node (is this node being modified?) */
#define ADDIN_NEED_ATTACH_WRITER_LOCK       0

/* Does the addin module require the custom ADDIN_LOAD_DATA structure (see below) */
#define ADDIN_NEED_ADDIN_LOAD_STRUCT        1

/* Does the addin module require the custom ADDIN_ATTACH_DATA structure (see below) */
#define ADDIN_NEED_ADDIN_ATTACH_STRUCT      1

/* Macro used to define if a subservice pair (id, type) is valid for an addin */
#define ADDIN_IS_VALID_SUBSERVICE(SubServiceId, SubServiceType) \
    ((SubServiceId) == 0 && (SubServiceType) == CSSM_SERVICE_CSP)

#if (ADDIN_NEED_ADDIN_LOAD_STRUCT)
/*-----------------------------------------------------------------------------
 * Description:
 * Structure containing custom fields in the CSSM_MODULE_LOAD_TRACKER structure
 *---------------------------------------------------------------------------*/
struct addin_load_data
{
    /* put custom fields here */

    /* Contains the handle to the private key file for this load */
    CSM_KM_HANDLE hKeyFile;
};
typedef struct addin_load_data ADDIN_LOAD_DATA;

#endif /* if ADDIN_NEED_ADDIN_LOAD_STRUCT */


#if (ADDIN_NEED_ADDIN_ATTACH_STRUCT)

/*-----------------------------------------------------------------------------
 * Description:
 * Structure containing custom fields in the CSSM_MODULE_ATTACH_TRACKER structure
 *---------------------------------------------------------------------------*/
struct addin_attach_data
{
    /* Maintains a list of CSP "sessions" in this attach. It does not
     * require the attach record to be locked since it has internal
     * locking machanisms. If the record is not locked, then this
     * value will have to be casted to (CSM_SESSION_LIST*) to remove
     * the 'const' declaration of the value returned by 
     * Addin_GetAttachData when ADDIN_NEED_ATTACH_WRITER_LOCK==0.
     */
    CSM_SESSION_LIST slSessions;
};
typedef struct addin_attach_data ADDIN_ATTACH_DATA;

#endif /* if ADDIN_NEED_ADDIN_ATTACH_STRUCT */

#endif
