/*-----------------------------------------------------------------------
 * File: MDS.CPP
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */


/*
 * This file contains module management functions for the DLM
 */

#if defined(_DEBUG) && defined(WIN32)
#undef new
#include <crtdbg.h>
static char s_szFileName[] = __FILE__;
#define new new(_NORMAL_BLOCK, s_szFileName, __LINE__)
#endif

/* MDS includes */
#include "cssmtype.h"

#include "mds.h"

#include "mds_internal.h"

/* DL include files */
#define DEFINE_FAILURE_ERROR_CODES
#define DAL_SOURCE_FILE
#include "dal_classes.h"
#include "dal_src.h" /* include to define DL interface functions */
#include "dal_internal.h"

/* CSSM includes */
#include "cssmport.h"
#include <cssmMemTrack.h>
static CSSM_RETURN mds_CheckParamters( 
    const CSSM_MEMORY_FUNCS * pMemoryFunctions,
    MDS_FUNCS * pDlFunctions,
    MDS_HANDLE *hMds);

extern CSSM_BOOL g_initComplete;
#ifdef VMS
extern "C" {
    int _init();
    int _fini();
}
#endif

/*---------------------------------------------------------------
 *
 * Name: MDS_Initialize
 *
 * Description:
 *   This function is called by an application to initialize the
 *   directory service
 *
 * Parameters: 
 *   cssmCredentialPath (input) - path to credentials for CSSM
 *   cssmSection (input)        - registry location of CSSM info
 *
 * Returns:
 *   CSSM_FAIL                  - authentication failed
 *   CSSM_OK                    - authentication succeeded,
 *                                mutual identity checks 
 *                                completed successfully
 *-------------------------------------------------------------*/
CSSM_RETURN CSSMAPI MDS_Initialize(
    const CSSM_GUID *pCallerGuid,
    const CSSM_DATA *pCallerManifest,
    const CSSM_MEMORY_FUNCS *pMemoryFunctions,
    MDS_FUNCS_PTR pDlFunctions,
    MDS_HANDLE *hMds )
{
    CSSM_RETURN retValue = CSSM_OK;

    pCallerGuid;
    pCallerManifest;

#ifdef _DEBUG
    if ( port_InitMemoryTracking() != CSSM_OK )
    {     
        return CSSMERR_DL_FUNCTION_FAILED;
    }
#endif

    /* 
     * Ensure that we are between _init and _fini 
     */
    if (g_initComplete == CSSM_FALSE)
#ifdef VMS
        _init();
#else
        return CSSMERR_DL_FUNCTION_FAILED;
#endif
    /* 
     * check the paramaters to the function 
     */
    if ((retValue = mds_CheckParamters(pMemoryFunctions, pDlFunctions, hMds)) != CSSM_OK)
        return retValue;

    /*
     * Set meory function pointers.
     */
    if ((retValue = mds_SetFunctionPointers(pMemoryFunctions, hMds)) != CSSM_OK) 
        return retValue;
        
    /*
     * Set the function pointers 
     */
    pDlFunctions->DbOpen = DL_DbOpen;
    pDlFunctions->DbClose = DL_DbClose;
    pDlFunctions->GetDbNames = DL_GetDbNames;
    pDlFunctions->GetDbNameFromHandle = DL_GetDbNameFromHandle;
    pDlFunctions->FreeNameList = DL_FreeNameList;
    pDlFunctions->DataInsert = DL_DataInsert;
    pDlFunctions->DataDelete = DL_DataDelete;
    pDlFunctions->DataModify = DL_DataModify;  
    pDlFunctions->DataGetFirst = DL_DataGetFirst;
    pDlFunctions->DataGetNext = DL_DataGetNext;
    pDlFunctions->DataAbortQuery = DL_DataAbortQuery;
    pDlFunctions->DataGetFromUniqueRecordId = DL_DataGetFromUniqueRecordId;
    pDlFunctions->FreeUniqueRecord = DL_FreeUniqueRecord;
    pDlFunctions->CreateRelation = DL_CreateRelation;
    pDlFunctions->DestroyRelation = DL_DestroyRelation;

    return CSSM_OK;
}

CSSM_RETURN CSSMAPI MDS_Terminate(MDS_HANDLE Handle)
{
    return mds_Terminate (Handle);
}

static CSSM_RETURN mds_CheckParamters( 
    const CSSM_MEMORY_FUNCS * pMemoryFunctions, 
    MDS_FUNCS * pDlFunctions,
    MDS_HANDLE *hMds)
{
    if (IsBadReadPtr(pMemoryFunctions, sizeof(CSSM_MEMORY_FUNCS)) ||
        IsBadCodePtr((CSSM_PROC_ADDR)pMemoryFunctions->malloc_func) ||
        IsBadCodePtr((CSSM_PROC_ADDR)pMemoryFunctions->free_func) ||
        IsBadCodePtr((CSSM_PROC_ADDR)pMemoryFunctions->calloc_func) ||
        IsBadCodePtr((CSSM_PROC_ADDR)pMemoryFunctions->realloc_func)) 
    {
        return CSSMERR_DL_INVALID_POINTER;
    }


    if (IsBadWritePtr(pDlFunctions, sizeof(MDS_FUNCS_PTR))) 
    {
        return CSSMERR_DL_INVALID_POINTER;
    }

    if (IsBadWritePtr(hMds, sizeof(MDS_HANDLE))) 
    {
        return CSSMERR_DL_INVALID_POINTER;
    }
    
    return CSSM_OK;
}

