/*-----------------------------------------------------------------------
 * File: MDS_CREATE.CPP
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
 * This file contains module management functions for the DLM
 */

#ifdef VMS
#include <string.h>
#include <strings.h>
#endif

#if defined(_DEBUG) && defined(WIN32)
#undef new
#include <crtdbg.h>
static char s_szFileName[] = __FILE__;
#define new new(_NORMAL_BLOCK, s_szFileName, __LINE__)
#endif

#ifndef ASSERT
    #ifdef _DEBUG
        #include <assert.h>
        /* Passing this boolean expression is nice when checking 
         * pointers on 64-bit architectures. It prevents lots of
         * compiler warnings about "conversion from pointer to 
         * smaller integer"
         */
        #define ASSERT(cond) assert(0 != (cond))
    #else
        #define ASSERT(cond) 
    #endif
#endif

#include "cssmport.h"
#include "mds.h"
#include "cssmlock.h"
#include "cssmerr.h"
#ifdef ISL_INCLUDED
#include "eislapi.h"
#endif
#include "maf_integ.h"
#include "maf_collectn.h"
#include "maf_util.h"
#include "mds_internal.h"
#include "mds_schema.h"
#include "dal_common.h"
/*
#define CSSMDLI WINAPI
*/
#define CSSMDLI CSSMAPI

#include <cssmMemTrack.h>

/* prototype internal function from dal_src.cpp */


CSSM_RETURN dal_GetDbNames(CSSM_DL_HANDLE DLHandle, CSSM_NAME_LIST_PTR *NameList);

CSSM_RETURN dal_FreeNameList(CSSM_DL_HANDLE DLHandle, CSSM_NAME_LIST_PTR NameList);

CSSM_RETURN dal_DbCreate (
    CSSM_DL_HANDLE DLHandle,
    const char *DB_NAME_PARAMATER,
    const CSSM_NET_ADDRESS *DbLocation,   
    const CSSM_DBINFO *DbInfo,
    CSSM_DB_ACCESS_TYPE AccessRequest,
    const CSSM_RESOURCE_CONTROL_CONTEXT *CredAndAclEntry,
    const void *OpenParameters,
    CSSM_DB_HANDLE *DbHandle);

CSSM_RETURN dal_DbDelete (
    CSSM_DL_HANDLE DLHandle,
    const char * DB_NAME_PARAMATER,
    const CSSM_NET_ADDRESS * DbLocation,
    const CSSM_ACCESS_CREDENTIALS * AccessCred);

CSSM_RETURN dal_DbClose (CSSM_DL_DB_HANDLE DLDBHandle);


#define UINDEX(attribute) { \
    CSSM_DB_INDEX_UNIQUE, /* IndexType */ \
    CSSM_DB_INDEX_ON_ATTRIBUTE, /* IndexedDataLocation */ \
    attribute \
}

#define INDEX(attribute) { \
    CSSM_DB_INDEX_NONUNIQUE, /* IndexType */ \
    CSSM_DB_INDEX_ON_ATTRIBUTE, /* IndexedDataLocation */ \
    attribute \
}

/*********************************************************************************************************/
/* Object Directory Schema 
 */

const CSSM_DBINFO *mds_object_schema_GetDBInfo()
{
    static CSSM_DB_PARSING_MODULE_INFO rgDefaultParsingModule[MDS_OBJECT_NUM_RELATIONS] = {
        MDS_OBJECT_RECORDTYPE, /* RecordType */
        { /* ModuleSubserviceUid */
            { 0, 0, 0, {0,0,0,0,0,0,0,0} }, /* ModuleSubserviceUid.Guid */
            {0, 0}, /* ModuleSubserviceUid.Version */
            0, /* ModuleSubserviceUid.SubserviceId */
            0, /* ModuleSubserviceUid.SubserviceFlags */
        }
    };

    static CSSM_DB_ATTRIBUTE_INFO rgAttributeInfo[MDS_OBJECT_NUM_INDEXES] = {
        MDS_ATTRINFO_MODULEID, MDS_ATTRINFO_MANIFEST, MDS_ATTRINFO_MODULENAME, MDS_ATTRINFO_PATH,
        MDS_ATTRINFO_PRODUCTVERSION
    };

    static CSSM_DB_RECORD_ATTRIBUTE_INFO rgDefaultRecordAttributeNames[MDS_OBJECT_NUM_RELATIONS] = {
        MDS_OBJECT_RECORDTYPE, /* DataRecordType */
        MDS_OBJECT_NUM_INDEXES, /* NumberOfAttributes */
        rgAttributeInfo /* AttributeInfo */
    };

    static CSSM_DB_INDEX_INFO rgIndexInfo[MDS_OBJECT_NUM_INDEXES] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        INDEX(MDS_ATTRINFO_MANIFEST),
        INDEX(MDS_ATTRINFO_MODULENAME),
        INDEX(MDS_ATTRINFO_PATH),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION)
    };

    static CSSM_DB_RECORD_INDEX_INFO rgDefaultRecordIndexes[MDS_OBJECT_NUM_RELATIONS] = {
        MDS_OBJECT_RECORDTYPE, /* DataRecordType */
        MDS_OBJECT_NUM_INDEXES, /* NumberOfIndexes */
        rgIndexInfo/* IndexInfo */
    };

    static const CSSM_DBINFO DefaultDbInfo = {
        MDS_OBJECT_NUM_RELATIONS, /* NumberOfRecordTypes */
        rgDefaultParsingModule, /* DefaultParsingModules */
        rgDefaultRecordAttributeNames, /* RecordAttributeNames */
        rgDefaultRecordIndexes, /* RecordIndexes */     
        CSSM_TRUE, /* IsLocal */
        NULL, /* AccessPath */
        NULL /* Reserved */
    };

    return &DefaultDbInfo;
}

static void mds_CSSM_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_CDSAVERSION,
        MDS_ATTRINFO_VENDOR,
        MDS_ATTRINFO_DESC,
        MDS_ATTRINFO_NATIVESERVICES
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSSM_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_CSSM_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_CSSM_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[MDS_CDSADIR_CSSM_NUM_INDEXES] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        INDEX(MDS_ATTRINFO_CDSAVERSION),
        INDEX(MDS_ATTRINFO_VENDOR),
        INDEX(MDS_ATTRINFO_DESC),
        INDEX(MDS_ATTRINFO_NATIVESERVICES)      
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSSM_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_CSSM_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_EMM_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_MANIFEST,
        MDS_ATTRINFO_MODULENAME,
        MDS_ATTRINFO_PATH,
        MDS_ATTRINFO_CDSAVERSION,
        MDS_ATTRINFO_EMMSPECVERSION,
        MDS_ATTRINFO_DESC,
        MDS_ATTRINFO_POLICYSTMT,
        MDS_ATTRINFO_EMMVERSION,
        MDS_ATTRINFO_EMMVENDOR,
        MDS_ATTRINFO_EMMTYPE
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_EMM_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_EMM_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_EMM_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        INDEX(MDS_ATTRINFO_MANIFEST),
        INDEX(MDS_ATTRINFO_MODULENAME),
        INDEX(MDS_ATTRINFO_PATH),
        INDEX(MDS_ATTRINFO_CDSAVERSION),
        INDEX(MDS_ATTRINFO_EMMSPECVERSION),
        INDEX(MDS_ATTRINFO_DESC),
        INDEX(MDS_ATTRINFO_POLICYSTMT),
        INDEX(MDS_ATTRINFO_EMMVERSION),
        INDEX(MDS_ATTRINFO_EMMVENDOR),
        INDEX(MDS_ATTRINFO_EMMTYPE)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_EMM_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_EMM_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_EMM_PRIMARY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_SERVICETYPE,
        MDS_ATTRINFO_MANIFEST,
        MDS_ATTRINFO_MODULENAME,
        MDS_ATTRINFO_VERSION,
        MDS_ATTRINFO_VENDOR,
        MDS_ATTRINFO_SAMPLETYPES,
        MDS_ATTRINFO_ACLSUBJECTTYPES,
        MDS_ATTRINFO_AUTHTAGS,
        MDS_ATTRINFO_EMMSPECVERSION
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_EMM_PRIMARY_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_EMM_PRIMARY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_EMM_PRIMARY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),      
        INDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_SERVICETYPE),
        INDEX(MDS_ATTRINFO_MANIFEST),
        INDEX(MDS_ATTRINFO_MODULENAME),
        INDEX(MDS_ATTRINFO_VERSION),
        INDEX(MDS_ATTRINFO_VENDOR),
        INDEX(MDS_ATTRINFO_SAMPLETYPES),
        INDEX(MDS_ATTRINFO_ACLSUBJECTTYPES),
        INDEX(MDS_ATTRINFO_AUTHTAGS),
        INDEX(MDS_ATTRINFO_EMMSPECVERSION)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_EMM_PRIMARY_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_EMM_PRIMARY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_COMMON_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_MANIFEST,
        MDS_ATTRINFO_MODULENAME,
        MDS_ATTRINFO_PATH,
        MDS_ATTRINFO_CDSAVERSION,
        MDS_ATTRINFO_DESC,
        MDS_ATTRINFO_DYNAMICFLAG,
        MDS_ATTRINFO_MULTITHREADFLAG,
        MDS_ATTRINFO_SERVICEMASK
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_COMMON_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_COMMON_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_COMMON_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        INDEX(MDS_ATTRINFO_MANIFEST),
        INDEX(MDS_ATTRINFO_MODULENAME),
        INDEX(MDS_ATTRINFO_PATH),
        INDEX(MDS_ATTRINFO_CDSAVERSION),
        INDEX(MDS_ATTRINFO_DESC),
        INDEX(MDS_ATTRINFO_DYNAMICFLAG),        
        INDEX(MDS_ATTRINFO_MULTITHREADFLAG),
        INDEX(MDS_ATTRINFO_SERVICEMASK)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_COMMON_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_COMMON_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_CSP_PRIMARY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_MANIFEST,
        MDS_ATTRINFO_MODULENAME,
        MDS_ATTRINFO_PRODUCTVERSION,
        MDS_ATTRINFO_VENDOR,
        MDS_ATTRINFO_CSPTYPE,
        MDS_ATTRINFO_CSPFLAGS,
        MDS_ATTRINFO_CSPCUSTOMFLAGS,
        MDS_ATTRINFO_USEETAGS,
        MDS_ATTRINFO_SAMPLETYPES,
        MDS_ATTRINFO_ACLSUBJECTTYPES,
        MDS_ATTRINFO_AUTHTAGS
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSP_PRIMARY_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_CSP_PRIMARY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_CSP_PRIMARY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_MANIFEST),
        INDEX(MDS_ATTRINFO_MODULENAME),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION),
        INDEX(MDS_ATTRINFO_VENDOR),
        INDEX(MDS_ATTRINFO_CSPTYPE),
        INDEX(MDS_ATTRINFO_CSPFLAGS),
        INDEX(MDS_ATTRINFO_CSPCUSTOMFLAGS),
        INDEX(MDS_ATTRINFO_USEETAGS),
        INDEX(MDS_ATTRINFO_SAMPLETYPES),
        INDEX(MDS_ATTRINFO_ACLSUBJECTTYPES),
        INDEX(MDS_ATTRINFO_AUTHTAGS)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSP_PRIMARY_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_CSP_PRIMARY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_CSP_CAPABILITY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_USEETAG,
        MDS_ATTRINFO_CONTEXTTYPE,
        MDS_ATTRINFO_ALGTYPE,
        MDS_ATTRINFO_GROUPID,
        MDS_ATTRINFO_ATTRIBUTETYPE,
        MDS_ATTRINFO_ATTRIBUTEVALUE,
        MDS_ATTRINFO_DESCRIPTION
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSP_CAPABILITY_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_CSP_CAPABILITY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_CSP_CAPABILITY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        UINDEX(MDS_ATTRINFO_USEETAG),       
        UINDEX(MDS_ATTRINFO_CONTEXTTYPE),
        UINDEX(MDS_ATTRINFO_ALGTYPE),
        UINDEX(MDS_ATTRINFO_GROUPID),
        UINDEX(MDS_ATTRINFO_ATTRIBUTETYPE),
        INDEX(MDS_ATTRINFO_ATTRIBUTEVALUE),
        INDEX(MDS_ATTRINFO_DESCRIPTION)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSP_CAPABILITY_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_CSP_CAPABILITY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_CSP_ENCAPSULATED_PRODUCT_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_PRODUCTDESC,
        MDS_ATTRINFO_PRODUCTVENDOR,
        MDS_ATTRINFO_PRODUCTVERSION,
        MDS_ATTRINFO_PRODUCTFLAGS,
        MDS_ATTRINFO_CUSTOMFLAGS,
        MDS_ATTRINFO_STANDARDDESC,
        MDS_ATTRINFO_STANDARDVERSION,
        MDS_ATTRINFO_READERDESC,
        MDS_ATTRINFO_READERVENDOR,
        MDS_ATTRINFO_READERVERSION,     
        MDS_ATTRINFO_READERFIRMWAREVERSION,
        MDS_ATTRINFO_READERFLAGS,
        MDS_ATTRINFO_READERCUSTOMFLAGS,
        MDS_ATTRINFO_READERSERIALNUMBER
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_CSP_ENCAPSULATED_PRODUCT_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_PRODUCTDESC),
        INDEX(MDS_ATTRINFO_PRODUCTVENDOR),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION),
        INDEX(MDS_ATTRINFO_PRODUCTFLAGS),
        INDEX(MDS_ATTRINFO_CUSTOMFLAGS),
        INDEX(MDS_ATTRINFO_STANDARDDESC),
        INDEX(MDS_ATTRINFO_STANDARDVERSION),
        INDEX(MDS_ATTRINFO_READERDESC),
        INDEX(MDS_ATTRINFO_READERVENDOR),
        INDEX(MDS_ATTRINFO_READERVERSION),      
        INDEX(MDS_ATTRINFO_READERFIRMWAREVERSION),
        INDEX(MDS_ATTRINFO_READERFLAGS),
        INDEX(MDS_ATTRINFO_READERCUSTOMFLAGS),
        INDEX(MDS_ATTRINFO_READERSERIALNUMBER)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_CSP_SC_INFO_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_SCDESC,
        MDS_ATTRINFO_SCVENDOR,
        MDS_ATTRINFO_SCVERSION,
        MDS_ATTRINFO_SCFIRMWAREVERSION,
        MDS_ATTRINFO_SCFLAGS,
        MDS_ATTRINFO_SCCUSTOMFLAGS,
        MDS_ATTRINFO_SCSERIALNUMBER
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSP_SC_INFO_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_CSP_SC_INFO_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_CSP_SC_INFO_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_SCDESC),
        INDEX(MDS_ATTRINFO_SCVENDOR),
        INDEX(MDS_ATTRINFO_SCVERSION),
        INDEX(MDS_ATTRINFO_SCFIRMWAREVERSION),
        INDEX(MDS_ATTRINFO_SCFLAGS),
        INDEX(MDS_ATTRINFO_SCCUSTOMFLAGS),
        INDEX(MDS_ATTRINFO_SCSERIALNUMBER)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CSP_SC_INFO_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_CSP_SC_INFO_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_DL_PRIMARY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_MANIFEST,
        MDS_ATTRINFO_MODULENAME,
        MDS_ATTRINFO_PRODUCTVERSION,
        MDS_ATTRINFO_VENDOR,
        MDS_ATTRINFO_DLTYPE,
        MDS_ATTRINFO_QUERYLIMITSFLAG,
        MDS_ATTRINFO_SAMPLETYPES,
        MDS_ATTRINFO_ACLSUBJECTTYPES,
        MDS_ATTRINFO_AUTHTAGS,
        MDS_ATTRINFO_CONJUNCTIVEOPS,
        MDS_ATTRINFO_RELATIONALOPS
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_DL_PRIMARY_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_DL_PRIMARY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_DL_PRIMARY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_MANIFEST),
        INDEX(MDS_ATTRINFO_MODULENAME),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION),
        INDEX(MDS_ATTRINFO_VENDOR),
        INDEX(MDS_ATTRINFO_DLTYPE),
        INDEX(MDS_ATTRINFO_QUERYLIMITSFLAG),        
        INDEX(MDS_ATTRINFO_SAMPLETYPES),
        INDEX(MDS_ATTRINFO_ACLSUBJECTTYPES),
        INDEX(MDS_ATTRINFO_AUTHTAGS),
        INDEX(MDS_ATTRINFO_CONJUNCTIVEOPS),
        INDEX(MDS_ATTRINFO_RELATIONALOPS)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_DL_PRIMARY_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_DL_PRIMARY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_DL_ENCAPSULATED_PRODUCT_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_PRODUCTDESC,
        MDS_ATTRINFO_PRODUCTVERSION,
        MDS_ATTRINFO_PRODUCTVENDOR,
        MDS_ATTRINFO_PRODUCTFLAGS,
        MDS_ATTRINFO_STANDARDDESC,
        MDS_ATTRINFO_STANDARDVERSION,
        MDS_ATTRINFO_PROTOCOL,
        MDS_ATTRINFO_RETRIEVALMODE
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_DL_ENCAPSULATED_PRODUCT_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_PRODUCTDESC),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION),
        INDEX(MDS_ATTRINFO_PRODUCTVENDOR),
        INDEX(MDS_ATTRINFO_PRODUCTFLAGS),       
        INDEX(MDS_ATTRINFO_STANDARDDESC),
        INDEX(MDS_ATTRINFO_STANDARDVERSION),
        INDEX(MDS_ATTRINFO_PROTOCOL),
        INDEX(MDS_ATTRINFO_RETRIEVALMODE)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_CL_PRIMARY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_MANIFEST,
        MDS_ATTRINFO_MODULENAME,
        MDS_ATTRINFO_PRODUCTVERSION,
        MDS_ATTRINFO_VENDOR,
        MDS_ATTRINFO_CERTTYPEFORMAT,
        MDS_ATTRINFO_CRLTYPEFORMAT,
        MDS_ATTRINFO_CERTFIELDNAMES,
        MDS_ATTRINFO_BUNDLETYPEFORMAT,
        MDS_ATTRINFO_XLATIONTYPEFORMAT,
        MDS_ATTRINFO_DEFAULTTEMPLATETYPE,
        MDS_ATTRINFO_TEMPLATEFIELDNAMES
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CL_PRIMARY_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_CL_PRIMARY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_CL_PRIMARY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_MANIFEST),
        INDEX(MDS_ATTRINFO_MODULENAME),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION),
        INDEX(MDS_ATTRINFO_VENDOR),
        INDEX(MDS_ATTRINFO_CERTTYPEFORMAT),
        INDEX(MDS_ATTRINFO_CRLTYPEFORMAT),
        INDEX(MDS_ATTRINFO_CERTFIELDNAMES),
        INDEX(MDS_ATTRINFO_BUNDLETYPEFORMAT),
        INDEX(MDS_ATTRINFO_XLATIONTYPEFORMAT),
        INDEX(MDS_ATTRINFO_DEFAULTTEMPLATETYPE),
        INDEX(MDS_ATTRINFO_TEMPLATEFIELDNAMES)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CL_PRIMARY_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_CL_PRIMARY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_CL_ENCAPSULATED_PRODUCT_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_PRODUCTDESC,
        MDS_ATTRINFO_PRODUCTVENDOR,
        MDS_ATTRINFO_PRODUCTVERSION,
        MDS_ATTRINFO_PRODUCTFLAGS,
        MDS_ATTRINFO_STANDARDDESC,
        MDS_ATTRINFO_STANDARDVERSION        
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_CL_ENCAPSULATED_PRODUCT_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_PRODUCTDESC),
        INDEX(MDS_ATTRINFO_PRODUCTVENDOR),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION),
        INDEX(MDS_ATTRINFO_PRODUCTFLAGS),       
        INDEX(MDS_ATTRINFO_STANDARDDESC),
        INDEX(MDS_ATTRINFO_STANDARDVERSION)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_TP_PRIMARY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_MANIFEST,
        MDS_ATTRINFO_MODULENAME,
        MDS_ATTRINFO_PRODUCTVERSION,
        MDS_ATTRINFO_VENDOR,
        MDS_ATTRINFO_CERTTYPEFORMAT,
        MDS_ATTRINFO_SAMPLETYPES,
        MDS_ATTRINFO_ACLSUBJECTTYPES,
        MDS_ATTRINFO_AUTHTAGS
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_TP_PRIMARY_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_TP_PRIMARY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_TP_PRIMARY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_MANIFEST),
        INDEX(MDS_ATTRINFO_MODULENAME),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION),
        INDEX(MDS_ATTRINFO_VENDOR),
        INDEX(MDS_ATTRINFO_CERTTYPEFORMAT),
        INDEX(MDS_ATTRINFO_SAMPLETYPES),
        INDEX(MDS_ATTRINFO_ACLSUBJECTTYPES),
        INDEX(MDS_ATTRINFO_AUTHTAGS)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_TP_PRIMARY_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_TP_PRIMARY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_TP_OIDS_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_OID,
        MDS_ATTRINFO_VALUE
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_TP_OIDS_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_TP_OIDS_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_TP_OIDS_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        UINDEX(MDS_ATTRINFO_OID),
        INDEX(MDS_ATTRINFO_VALUE)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_TP_OIDS_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_TP_OIDS_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_TP_ENCAPSULATED_PRODUCT_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_PRODUCTDESC,
        MDS_ATTRINFO_PRODUCTVENDOR,
        MDS_ATTRINFO_PRODUCTVERSION,
        MDS_ATTRINFO_PRODUCTFLAGS,
        MDS_ATTRINFO_AUTHORITYREQUESTTYPE,
        MDS_ATTRINFO_STANDARDDESC,
        MDS_ATTRINFO_STANDARDVERSION,
        MDS_ATTRINFO_PROTOCOLDESC,
        MDS_ATTRINFO_PROTOCOLFLAGS,
        MDS_ATTRINFO_CERTCLASSNAME,
        MDS_ATTRINFO_ROOTCERTIFICATE,
        MDS_ATTRINFO_ROOTCERTTYPEFORMAT
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_TP_ENCAPSULATED_PRODUCT_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_PRODUCTDESC),
        INDEX(MDS_ATTRINFO_PRODUCTVENDOR),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION),
        INDEX(MDS_ATTRINFO_PRODUCTFLAGS),
        INDEX(MDS_ATTRINFO_AUTHORITYREQUESTTYPE),
        INDEX(MDS_ATTRINFO_STANDARDDESC),
        INDEX(MDS_ATTRINFO_STANDARDVERSION),
        INDEX(MDS_ATTRINFO_PROTOCOLDESC),
        INDEX(MDS_ATTRINFO_PROTOCOLFLAGS),
        INDEX(MDS_ATTRINFO_CERTCLASSNAME),
        INDEX(MDS_ATTRINFO_ROOTCERTIFICATE),
        INDEX(MDS_ATTRINFO_ROOTCERTTYPEFORMAT)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_AC_PRIMARY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_MODULEID,
        MDS_ATTRINFO_SSID,
        MDS_ATTRINFO_MANIFEST,
        MDS_ATTRINFO_MODULENAME,
        MDS_ATTRINFO_PRODUCTVERSION,
        MDS_ATTRINFO_VENDOR
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_AC_PRIMARY_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_AC_PRIMARY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_AC_PRIMARY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_MODULEID),
        UINDEX(MDS_ATTRINFO_SSID),
        INDEX(MDS_ATTRINFO_MANIFEST),
        INDEX(MDS_ATTRINFO_MODULENAME),
        INDEX(MDS_ATTRINFO_PRODUCTVERSION),
        INDEX(MDS_ATTRINFO_VENDOR)      
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_AC_PRIMARY_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_AC_PRIMARY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_SCHEMA_RELATIONS_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_RELATIONID,
        MDS_ATTRINFO_RELATIONNAME
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_SCHEMA_RELATIONS_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_SCHEMA_RELATIONS_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_SCHEMA_RELATIONS_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_RELATIONID),
        UINDEX(MDS_ATTRINFO_RELATIONNAME)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_SCHEMA_RELATIONS_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_SCHEMA_RELATIONS_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_SCHEMA_ATTRIBUTES_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_RELATIONID,        
        MDS_ATTRINFO_ATTRIBUTEID,
        MDS_ATTRINFO_ATTRIBUTENAMEFORMAT,
        MDS_ATTRINFO_ATTRIBUTENAME,
        MDS_ATTRINFO_ATTRIBUTENAMEID,
        MDS_ATTRINFO_ATTRIBUTEDATATYPE      
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_SCHEMA_ATTRIBUTES_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_SCHEMA_ATTRIBUTES_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_SCHEMA_ATTRIBUTES_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_RELATIONID),
        UINDEX(MDS_ATTRINFO_ATTRIBUTEID),
        INDEX(MDS_ATTRINFO_ATTRIBUTENAMEFORMAT),
        INDEX(MDS_ATTRINFO_ATTRIBUTENAME),
        INDEX(MDS_ATTRINFO_ATTRIBUTENAMEID),
        INDEX(MDS_ATTRINFO_ATTRIBUTEDATATYPE)           
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_SCHEMA_ATTRIBUTES_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_SCHEMA_ATTRIBUTES_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

static void mds_SCHEMA_INDEXES_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO * pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        MDS_ATTRINFO_RELATIONID,
        MDS_ATTRINFO_INDEXID,
        MDS_ATTRINFO_ATTRIBUTEID,
        MDS_ATTRINFO_INDEXTYPE,
        MDS_ATTRINFO_INDEXEDDATALOCATION    
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_SCHEMA_INDEXES_RECORDTYPE);
    pList->NumberOfAttributes = MDS_CDSADIR_SCHEMA_INDEXES_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}
static void mds_SCHEMA_INDEXES_GetIndexes(CSSM_DB_RECORD_INDEX_INFO * pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        UINDEX(MDS_ATTRINFO_RELATIONID),
        UINDEX(MDS_ATTRINFO_INDEXID),
        UINDEX(MDS_ATTRINFO_ATTRIBUTEID),
        INDEX(MDS_ATTRINFO_INDEXTYPE),
        INDEX(MDS_ATTRINFO_INDEXEDDATALOCATION)
    };

    ASSERT(pList->DataRecordType == MDS_CDSADIR_SCHEMA_INDEXES_RECORDTYPE);
    pList->NumberOfIndexes = MDS_CDSADIR_SCHEMA_INDEXES_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}



/*********************************************************************************************************/
/* CDSA Directory Schema 
 */

const CSSM_DBINFO *mds_cdsa_schema_GetDBInfo()
{
    const CSSM_DB_RECORDTYPE rgRecordTypes[MDS_CDSADIR_NUM_RELATIONS] = {
        MDS_CDSADIR_CSSM_RECORDTYPE, MDS_CDSADIR_EMM_RECORDTYPE, MDS_CDSADIR_EMM_PRIMARY_RECORDTYPE,
        MDS_CDSADIR_COMMON_RECORDTYPE,
        MDS_CDSADIR_CSP_PRIMARY_RECORDTYPE, MDS_CDSADIR_CSP_CAPABILITY_RECORDTYPE,
        MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_RECORDTYPE, MDS_CDSADIR_CSP_SC_INFO_RECORDTYPE,
        MDS_CDSADIR_DL_PRIMARY_RECORDTYPE,
        MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_RECORDTYPE, MDS_CDSADIR_CL_PRIMARY_RECORDTYPE,
        MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_RECORDTYPE, MDS_CDSADIR_TP_PRIMARY_RECORDTYPE,
        MDS_CDSADIR_TP_OIDS_RECORDTYPE, MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_RECORDTYPE,
        MDS_CDSADIR_AC_PRIMARY_RECORDTYPE, MDS_CDSADIR_SCHEMA_RELATIONS_RECORDTYPE,
        MDS_CDSADIR_SCHEMA_ATTRIBUTES_RECORDTYPE, MDS_CDSADIR_SCHEMA_INDEXES_RECORDTYPE
    };

    static CSSM_DB_PARSING_MODULE_INFO rgDefaultParsingModule[MDS_CDSADIR_NUM_RELATIONS];
    static CSSM_DB_RECORD_ATTRIBUTE_INFO rgDefaultRecordAttributeNames[MDS_CDSADIR_NUM_RELATIONS];
    static CSSM_DB_RECORD_INDEX_INFO rgDefaultRecordIndexes[MDS_CDSADIR_NUM_RELATIONS];

    static const CSSM_DBINFO DefaultDbInfo = {
        MDS_CDSADIR_NUM_RELATIONS, /* NumberOfRecordTypes */
        rgDefaultParsingModule, /* DefaultParsingModules */
        rgDefaultRecordAttributeNames, /* RecordAttributeNames */
        rgDefaultRecordIndexes, /* RecordIndexes */     
        CSSM_TRUE, /* IsLocal */
        NULL, /* AccessPath */
        NULL /* Reserved */
    };
    uint32 i;

    memset(rgDefaultParsingModule, 0, sizeof(rgDefaultParsingModule));
    memset(rgDefaultRecordAttributeNames, 0, sizeof(rgDefaultRecordAttributeNames));
    memset(rgDefaultRecordIndexes, 0, sizeof(rgDefaultRecordIndexes));

    for (i = 0; i < MDS_CDSADIR_NUM_RELATIONS; i++)
    {
        rgDefaultParsingModule[i].RecordType = rgRecordTypes[i];
        rgDefaultRecordAttributeNames[i].DataRecordType = rgRecordTypes[i];
        rgDefaultRecordIndexes[i].DataRecordType = rgRecordTypes[i];
    }

    CSSM_DB_RECORD_ATTRIBUTE_INFO * pAttributeList = rgDefaultRecordAttributeNames;
    CSSM_DB_RECORD_INDEX_INFO * pIndexList = rgDefaultRecordIndexes;

    mds_CSSM_GetAttributes(pAttributeList++);
    mds_CSSM_GetIndexes(pIndexList++);
    mds_EMM_GetAttributes(pAttributeList++);
    mds_EMM_GetIndexes(pIndexList++);
    mds_EMM_PRIMARY_GetAttributes(pAttributeList++);
    mds_EMM_PRIMARY_GetIndexes(pIndexList++);
    mds_COMMON_GetAttributes(pAttributeList++);
    mds_COMMON_GetIndexes(pIndexList++);
    mds_CSP_PRIMARY_GetAttributes(pAttributeList++);
    mds_CSP_PRIMARY_GetIndexes(pIndexList++);
    mds_CSP_CAPABILITY_GetAttributes(pAttributeList++);
    mds_CSP_CAPABILITY_GetIndexes(pIndexList++);
    mds_CSP_ENCAPSULATED_PRODUCT_GetIndexes(pIndexList++);
    mds_CSP_ENCAPSULATED_PRODUCT_GetAttributes(pAttributeList++);
    mds_CSP_SC_INFO_GetIndexes(pIndexList++);
    mds_CSP_SC_INFO_GetAttributes(pAttributeList++);
    mds_DL_PRIMARY_GetAttributes(pAttributeList++);
    mds_DL_PRIMARY_GetIndexes(pIndexList++);
    mds_DL_ENCAPSULATED_PRODUCT_GetAttributes(pAttributeList++);
    mds_DL_ENCAPSULATED_PRODUCT_GetIndexes(pIndexList++);
    mds_CL_PRIMARY_GetAttributes(pAttributeList++);
    mds_CL_PRIMARY_GetIndexes(pIndexList++);
    mds_CL_ENCAPSULATED_PRODUCT_GetAttributes(pAttributeList++);
    mds_CL_ENCAPSULATED_PRODUCT_GetIndexes(pIndexList++);
    mds_TP_PRIMARY_GetAttributes(pAttributeList++);
    mds_TP_PRIMARY_GetIndexes(pIndexList++);
    mds_TP_OIDS_GetAttributes(pAttributeList++);
    mds_TP_OIDS_GetIndexes(pIndexList++);
    mds_TP_ENCAPSULATED_PRODUCT_GetAttributes(pAttributeList++);
    mds_TP_ENCAPSULATED_PRODUCT_GetIndexes(pIndexList++);
    mds_AC_PRIMARY_GetAttributes(pAttributeList++);
    mds_AC_PRIMARY_GetIndexes(pIndexList++);
    mds_SCHEMA_RELATIONS_GetAttributes(pAttributeList++);
    mds_SCHEMA_RELATIONS_GetIndexes(pIndexList++);
    mds_SCHEMA_ATTRIBUTES_GetAttributes(pAttributeList++);
    mds_SCHEMA_ATTRIBUTES_GetIndexes(pIndexList++);
    mds_SCHEMA_INDEXES_GetAttributes(pAttributeList++);
    mds_SCHEMA_INDEXES_GetIndexes(pIndexList++);

    return &DefaultDbInfo;
}


/*******************************************************************/
static CSSM_RETURN mds_CreateDb(
    MDS_HANDLE MdsHandle,
    const char *szDbName,
    const CSSM_DBINFO *pDbInfo)
{
    uint32 i;
    CSSM_RETURN retValue;
    CSSM_DB_HANDLE DBHandle;
    CSSM_DB_ACCESS_TYPE access = CSSM_DB_ACCESS_READ | CSSM_DB_ACCESS_WRITE;
    CSSM_DL_DB_HANDLE DLDBHandle;
    CSSM_NAME_LIST_PTR dblist = NULL;

    ASSERT(pDbInfo);
    ASSERT(szDbName);
    ASSERT(MdsHandle);

    if (!MdsHandle)
    {
        return (CSSMERR_DL_INVALID_DL_HANDLE);
    }
    
    if ((retValue = dal_GetDbNames(MdsHandle, &dblist)) != CSSM_OK)
        return retValue;

    if (dblist) {
        for (i = 0; i < dblist->NumStrings; i++) {
            if (!stricmp (dblist->String[i], szDbName)) {
                dal_FreeNameList (MdsHandle, dblist);  
                return CSSM_OK;
            }
        }

        dal_FreeNameList (MdsHandle, dblist); 
    }

    if ((retValue = dal_DbCreate (MdsHandle, szDbName, NULL, (CSSM_DBINFO_PTR)pDbInfo, 
                                    access, NULL, NULL, &DBHandle)) != CSSM_OK)
        return retValue;

    if (!DBHandle)
        return CSSMERR_DL_INTERNAL_ERROR;

    DLDBHandle.DLHandle = MdsHandle;
    DLDBHandle.DBHandle = DBHandle;
    dal_DbClose (DLDBHandle);

    return CSSM_OK;
}

CSSM_RETURN CSSMAPI MDS_Install(MDS_HANDLE MdsHandle)
{
    void *LockRef;
    CSSM_RETURN retValue = CSSM_OK;

    ADDIN_SPI_BEGIN(retValue, MdsHandle, LockRef);
    if (CSSM_OK != retValue)
    {
        return retValue;
    }

    if ((retValue = mds_CreateDb(MdsHandle, MDS_OBJECT_DIRECTORY, mds_object_schema_GetDBInfo()))
                                            != CSSM_OK)
    {
        Addin_SPIEnd(LockRef);
        return retValue;
    }

    if ((retValue = mds_CreateDb(MdsHandle, MDS_CDSA_DIRECTORY, mds_cdsa_schema_GetDBInfo()))
                                                            != CSSM_OK)
    {
        Addin_SPIEnd(LockRef);
        return retValue;
    }

    Addin_SPIEnd(LockRef);

    return CSSM_OK;
}

CSSM_RETURN CSSMAPI MDS_Uninstall(MDS_HANDLE MdsHandle)
{
    void *LockRef;
    CSSM_RETURN retValue = CSSM_OK;

    ADDIN_SPI_BEGIN(retValue, MdsHandle, LockRef);
    if (CSSM_OK != retValue)
    {
        return retValue;
    }

    if ((retValue = dal_DbDelete(MdsHandle, MDS_CDSA_DIRECTORY, NULL, NULL)) != CSSM_OK)
    {
        Addin_SPIEnd(LockRef);
        return retValue;
    }

    if ((retValue = dal_DbDelete(MdsHandle, MDS_OBJECT_DIRECTORY, NULL, NULL)) != CSSM_OK)
    {
        Addin_SPIEnd(LockRef);
        return retValue;
    }

    Addin_SPIEnd(LockRef);

    return CSSM_OK;
}
