/*-----------------------------------------------------------------------
 * File: CERTREVK.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* CDSA includes */
#include "cssm.h"               /* CSSM types, errors, and APIs */

/* local includes */
#include "inx509tp.h"           /* func and data definitions 
                                   specific to this module */
#include "api_shim.h"


/*---------------------------------------------------------------
 *
 * Name: TP_CertRevoke
 *
 * Description:
 *   Revoke the cerificate given the inputs.  Maps directly to a 
 *   call to CSSM_CL_CrlAddCert() with the CL handle provided.
 *
 *  Returns:
 *   CSSM_OK - revocation succeeded.  new CRL returned in NewCrlTemplate
 *   non CSSM_OK - error detected
 *
 *-------------------------------------------------------------*/
CSSM_RETURN CSSMAPI 
TP_CertRevoke( CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_DATA *OldCrlTemplate,
                        const CSSM_CERTGROUP *CertGroupToBeRevoked,
                        const CSSM_CERTGROUP *RevokerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *RevokerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult,
                        CSSM_TP_CERTCHANGE_REASON Reason,
                        CSSM_DATA_PTR NewCrlTemplate )
{
    CSSM_DATA_PTR pCert = NULL;
    CSSM_RETURN Rtn = CSSM_OK;

    /* unused parameters */
    TPHandle;
    CSPHandle;
    RevokerCertGroup;
    RevokerVerifyContext;
    RevokerVerifyResult;
    Reason;

    /* --CHECK INPUTS-- */
    if (CertGroupToBeRevoked == NULL)
    {
        return(CSSMERR_TP_INVALID_CERTGROUP_POINTER);
    }

    /* --GET CERT TO BE REVOKED-- */
    if ((pCert = CertGroupToBeRevoked->GroupList.CertList) == NULL)
	{	
		return(CSSMERR_TP_INVALID_CERTGROUP_POINTER);
	}


    Rtn = addin_cl_CrlAddCert(CLHandle, 
                                0 /* CCHandle */, 
                                pCert, 
                                0, 
                                NULL, 
                                OldCrlTemplate, 
                                NewCrlTemplate);

    return (Rtn);
}
