/*-----------------------------------------------------------------------
 * File: CHN_UTIL.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 


#ifndef _CHN_UTIL_H
#define _CHN_UTIL_H

#include "x509defs.h"


#ifdef VMS
#define tp_CertGroupConstruct tp_int_CertGroupConstruct
#endif


/**************************************************************************** 
 * Internal helper functions for working with X.509 certificates and 
 *  certificate chains.
 ****************************************************************************/


/* RETRIEVAL FUNCTIONS */
CSSM_RETURN
tp_CertGetIssuerX509Name( CSSM_CL_HANDLE hCL, 
                          const CSSM_DATA *pCert, 
                          CSSM_DATA_PTR *ppName);

CSSM_RETURN 
tp_CertGetSubjectX509Name( CSSM_CL_HANDLE hCL, 
                           const CSSM_DATA *pCert,
                           CSSM_DATA_PTR *ppName);

CSSM_RETURN 
tp_GetCertFromDBList( CSSM_TP_HANDLE hTP,
                      CSSM_CL_HANDLE hCL, 
                      CSSM_SELECTION_PREDICATE_PTR pPredicate,
                      uint32 NumberOfPredicates,
                      const CSSM_DL_DB_LIST *pDBList,
                      CSSM_DATA_PTR *ppCert);

CSSM_RETURN
tp_GetCertFromDBListBySName( CSSM_TP_HANDLE hTP,
                             CSSM_CL_HANDLE hCL, 
                             const CSSM_DATA *pSubjectName,
                             const CSSM_DL_DB_LIST *pDBList,
                             CSSM_DATA_PTR *ppCert);


/* CHECK FUNCTIONS */
CSSM_BOOL 
tp_IsSameX509Name( const CSSM_DATA *pName1,  
                   const CSSM_DATA *pName2);

CSSM_BOOL 
tp_IsSelfSignedCert( CSSM_TP_HANDLE hTP,
                     CSSM_CL_HANDLE hCL, 
                     const CSSM_DATA *pCert);


CSSM_RETURN
tp_CheckCertValidityPeriodValid( CSSM_CL_HANDLE hCL, 
                              const CSSM_DATA *pCert );

CSSM_BOOL
tp_IsCertRevoked( CSSM_TP_HANDLE TPHandle, 
                  CSSM_CL_HANDLE CLHandle, 
                  const CSSM_DL_DB_LIST *pDBList, 
                  CSSM_DATA_PTR pCert );

CSSM_BOOL
tp_IsValidCrlType( CSSM_CRL_TYPE crlType );

CSSM_BOOL
tp_IsValidCrlEncoding( CSSM_CRL_ENCODING crlEncoding );

void 
free_certgroup(CSSM_CERTGROUP_PTR pCertGroup);

CSSM_RETURN
CSSMAPI tp_CertGroupConstruct( CSSM_TP_HANDLE TPHandle,
                               CSSM_CL_HANDLE CLHandle,
                               CSSM_CSP_HANDLE CSPHandle,
                               const CSSM_DL_DB_LIST *DBList,
                               const CSSM_CERTGROUP *CertGroupFrag,
                               CSSM_BOOL IsAppCall,
                               CSSM_CERTGROUP_PTR *ppCertGroup); 

/* CrlVerify is defined in crlvrfy.c */
CSSM_RETURN CSSMAPI 
CrlVerify ( CSSM_TP_HANDLE TPHandle,
                       CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_ENCODED_CRL *CrlToBeVerified,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *VerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult );

#endif /* _CHN_UTIL_H */


