/*-----------------------------------------------------------------------
 * File: INX509TP.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 


#define NOCOMM
#define BUFSIZE 180

// suppress warnings in the runtime headers
#ifndef VMS
#pragma warning (disable:4201 4514 4214 4115)
#endif
#ifdef WIN32
#include <windows.h>
#endif
#include <stdio.h>              
#ifndef VMS
#pragma warning (default:4201 4514 4214 4115)
/* note that error 4514 is "unreferenced inline function has been removed */
#endif
#include <assert.h>

/* CDSA includes */
#include "cssm.h"               /* CSSM types, errors, and APIs */
#include "cssmspi.h"            /* CSSM service provider interface */
#include "cssmtpi.h"            /* CSSM trust policy interface */
#ifdef VMS
/* include this file asap so the local version will be picked up */
#include "maf_config.h"
#endif
/* Intel includes */
#define CSSM_INITGUID
#if defined (_GUIDS_H) && defined (VMS)
#undef _GUIDS_H	                /* Pick up the correct version of the */
#undef _CSSM_DEFINE_GUID        /* macro in guids.h for this module. */
#endif
#include "guids.h"              /* guid definitions */

/* local includes */
#include "inx509tp.h"           /* func and data definitions specific 
                                   to this module */
#include "api_shim.h"
#include "tpm_cspi.h"
#include "maf_include.h"
#include "maf_callouts.h"
#include <cssmMemTrack.h>

static CSSM_MODULE_FUNCS_PTR s_pTPModuleFuncs = NULL;
static CSSM_SPI_TP_FUNCS_PTR s_pTPSPIFuncs = NULL;

/*---------------------------------------------------------------
 *
 * Name: Initialize
 *
 * Description:
 *   Required function that CSSM calls during the attach process for
 *   checking the compatibilty of the version of the TP requested.   
 *
 * Parameters: 
 *   VerMajor (input)       - Major version of the TP requested
 *   VerMinor (input)       - Minor version of the TP requested
 *
 * Returns:
 *   CSSM_FAIL              - versions are not compatible
 *   CSSM_OK                - versions are compatible
 *
 *-------------------------------------------------------------*/
CSSM_RETURN
Addin_callout_Initialize(void)
{
    return( CSSM_OK );
}

CSSM_RETURN
Addin_callout_Terminate(void)
{
    free( s_pTPModuleFuncs );
    return( CSSM_OK );
}


void
Addin_callout_LoadDataDestroy( ADDIN_LOAD_DATA * pAddinLoadData )
{
    assert( pAddinLoadData != NULL );

    /* Clear the structure */
    memset( pAddinLoadData, 0, sizeof(*pAddinLoadData) );
}

CSSM_RETURN
Addin_callout_LoadDataConstruct(ADDIN_MODULE_HANDLE CssmHandle, 
                                ADDIN_LOAD_DATA * pAddinLoadData)
{

    assert( pAddinLoadData != NULL );

    /* Clear the structure */
    memset( pAddinLoadData, 0, sizeof(*pAddinLoadData) );

    /* obtain needed CSSM entry points */
    /* CSSM API */
    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_GetAPIMemoryFunctions", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->GetAPIMemoryFunctions ) 
        != CSSM_OK )
        goto FAIL;

    /* CSP API */
    if( Addin_GetProcAddress( CssmHandle,
                              "CSSM_CSP_CreateSignatureContext", 
                              (CSSM_PROC_ADDR*)&pAddinLoadData->csp_CreateSignatureContext )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_DeleteContext", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->DeleteContext )
        != CSSM_OK )
        goto FAIL;
   
    /* CL API */
    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CertAbortQuery", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CertAbortQuery )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CertSign", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CertSign )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CertGetFirstFieldValue", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CertGetFirstFieldValue )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CertGetNextFieldValue", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CertGetNextFieldValue )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CertGetKeyInfo", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CertGetKeyInfo )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CertVerify", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CertVerify ) 
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CrlVerify", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CrlVerify )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CrlSign", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CrlSign )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CrlAddCert", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CrlAddCert )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_CrlGetFirstFieldValue", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_CrlGetFirstFieldValue )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_CL_IsCertInCrl", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->cl_IsCertInCrl )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_DL_DataGetFirst", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->dl_DataGetFirst )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_DL_DataGetNext", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->dl_DataGetNext )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_DL_DataAbortQuery", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->dl_AbortQuery )
        != CSSM_OK )
        goto FAIL;

    if( Addin_GetProcAddress( CssmHandle,
                                "CSSM_DL_FreeUniqueRecord", 
                                (CSSM_PROC_ADDR*)&pAddinLoadData->dl_FreeUniqueRecord )
        != CSSM_OK )
    goto FAIL;


    return CSSM_OK;

FAIL:
    /* Clear the structure, with a known bad value (for debugging) */
    memset( pAddinLoadData, 0x33, sizeof(*pAddinLoadData) );

    return CSSMERR_TP_INTERNAL_ERROR;
}

#define TP_FUNCTBL_LEN (sizeof(CSSM_SPI_TP_FUNCS)+sizeof(CSSM_MODULE_FUNCS)-sizeof(CSSM_PROC_ADDR))

CSSM_RETURN Addin_callout_ModuleAttach(
    MAF_MODULE_LOAD_TRACKER_PTR pLoadTracker, 
    MAF_MODULE_ATTACH_TRACKER_PTR pAttachTracker,
    CSSM_MODULE_FUNCS_PTR *FuncTbl)
{
    /* The size of the buffer is the size of the header + the size needed to store the csp functions */
/*  static uint8 TpFuncTblBuffer[TP_FUNCTBL_LEN]; MHS REMOVED IN R3*/
    static CSSM_MODULE_FUNCS    FunctionList;
    static CSSM_SPI_TP_FUNCS    TpFunctions;

    /* Reference unused parameters to quiet the compiler */
    pLoadTracker;
    pAttachTracker;

/*  memset(TpFuncTblBuffer, 0, TP_FUNCTBL_LEN); MHS REMOVED IN R3*/

    /* Assign the pointers */
/*  pFunctionList = (CSSM_MODULE_FUNCS_PTR)TpFuncTblBuffer; MHS REMOVED IN R3 */
    *FuncTbl = &FunctionList;
/*  pTpFunctions = (CSSM_SPI_TP_FUNCS_PTR)pFunctionList->ServiceFuncs; MHS REMOVED IN R3 */

    /* Fill out the header */
/*  pFunctionList->TblSize = sizeof(TpFuncTblBuffer); MHS REMOVED IN R3 */
    FunctionList.ServiceType = CSSM_SERVICE_TP;
    FunctionList.NumberOfServiceFuncs = sizeof(CSSM_SPI_TP_FUNCS) / sizeof(CSSM_PROC_ADDR);
    FunctionList.ServiceFuncs = (CSSM_PROC_ADDR *) &TpFunctions;

    /* Fill in the TP SPI functions */
    TpFunctions.SubmitCredRequest =    TPM_SubmitCredRequest; 
    TpFunctions.RetrieveCredResult =   TPM_RetrieveCredResult;
    TpFunctions.ConfirmCredResult =    TPM_ConfirmCredResult;
    TpFunctions.ReceiveConfirmation =  TPM_ReceiveConfirmation;
    TpFunctions.CertReclaimKey =       TPM_CertReclaimKey;
    TpFunctions.CertReclaimAbort =     TPM_CertReclaimAbort;
    TpFunctions.FormRequest =          TPM_FormRequest;
    TpFunctions.FormSubmit =           TPM_FormSubmit;

    TpFunctions.CertGroupVerify =      TPM_CertGroupVerify; 
    TpFunctions.CertCreateTemplate =   TPM_CertCreateTemplate;
    TpFunctions.CertGetAllTemplateFields = TPM_CertGetAllTemplateFields; 
    TpFunctions.CertSign =             TPM_CertSign;
    TpFunctions.CrlVerify =            TPM_CrlVerify;
    TpFunctions.CrlCreateTemplate =    TPM_CrlCreateTemplate;
    TpFunctions.CertRevoke =           TPM_CertRevoke;
    TpFunctions.CertRemoveFromCrlTemplate = TPM_CertRemoveFromCrlTemplate;
    TpFunctions.CrlSign =              TPM_CrlSign;
    TpFunctions.ApplyCrlToDb =         TPM_ApplyCrlToDb;
    TpFunctions.CertGroupConstruct =   TPM_CertGroupConstruct;
    TpFunctions.CertGroupPrune =       TPM_CertGroupPrune;
    TpFunctions.CertGroupToTupleGroup = TPM_CertGroupToTupleGroup;
    TpFunctions.TupleGroupToCertGroup = TPM_TupleGroupToCertGroup;
    TpFunctions.PassThrough =          TPM_PassThrough;

    return CSSM_OK;
}

CSSM_RETURN Addin_callout_ModuleLoad(
    MAF_MODULE_LOAD_TRACKER_PTR pLoadTracker,
    CSSM_SPI_ModuleEventHandler CssmNotifyCallback,
    const void* CssmNotifyCallbackCtx)
{
    /* Reference unused parameters to quiet the compiler */
    pLoadTracker;
    
    /* Trigger a callback into the CSSM to simulate an insert */
    if ( CssmNotifyCallback != NULL )
    {
        CssmNotifyCallback( &intel_tpm_guid,
                            (void *)CssmNotifyCallbackCtx,
                            0,
                            CSSM_SERVICE_TP,
                            CSSM_NOTIFY_INSERT );
        return CSSM_OK;
    }

    return CSSMERR_TP_INTERNAL_ERROR;
}

CSSM_RETURN Addin_callout_ModuleUnload(
    MAF_MODULE_LOAD_TRACKER_PTR pLoadTracker,
    CSSM_SPI_ModuleEventHandler CssmNotifyCallback,
    const void* CssmNotifyCallbackCtx)
{
    /* Reference unused parameters to quiet the compiler */
    pLoadTracker;
    CssmNotifyCallback;
    CssmNotifyCallbackCtx;

    return CSSM_OK;
}




/* List of non-implemented functions.  These definitions are included so we
   can use the TPM in MAF.
*/
CSSM_RETURN CSSMAPI TP_SubmitCredRequest(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_TP_AUTHORITY_ID *PreferredAuthority,
                        CSSM_TP_AUTHORITY_REQUEST_TYPE RequestType,
                        const CSSM_TP_REQUEST_SET *RequestInput,
                        const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthContext,
                        sint32 *EstimatedTime,
                        CSSM_DATA_PTR ReferenceIdentifier )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_RetrieveCredResult(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_DATA *ReferenceIdentifier,
                        const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthCredentials,
                        sint32 *EstimatedTime,
                        CSSM_BOOL *ConfirmationRequired,
                        CSSM_TP_RESULT_SET_PTR *RetrieveOutput )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_ConfirmCredResult(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_DATA *ReferenceIdentifier,
                        const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthCredentials,
                        const CSSM_TP_CONFIRM_RESPONSE *Responses,
                        const CSSM_TP_AUTHORITY_ID *PreferredAuthority )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_ReceiveConfirmation(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_DATA *ReferenceIdentifier,
                        CSSM_TP_CONFIRM_RESPONSE_PTR *Responses,
                        sint32 *ElapsedTime )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_CertReclaimKey(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_CERTGROUP *CertGroup,
                        uint32 CertIndex,
                        CSSM_LONG_HANDLE KeyCacheHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_RESOURCE_CONTROL_CONTEXT *CredAndAclEntry )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_CertReclaimAbort(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_LONG_HANDLE KeyCacheHandle )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_FormRequest(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_TP_AUTHORITY_ID *PreferredAuthority,
                        CSSM_TP_FORM_TYPE FormType,
                        CSSM_DATA_PTR BlankForm )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_FormSubmit(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_TP_FORM_TYPE FormType,
                        const CSSM_DATA *Form,
                        const CSSM_TP_AUTHORITY_ID *ClearanceAuthority,
                        const CSSM_TP_AUTHORITY_ID *RepresentedAuthority,
                        CSSM_ACCESS_CREDENTIALS_PTR Credentials )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_CertCreateTemplate(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        uint32 NumberOfFields,
                        const CSSM_FIELD *CertFields,
                        CSSM_DATA_PTR CertTemplate )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_CertGetAllTemplateFields(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_DATA *CertTemplate,
                        uint32 *NumberOfFields,
                        CSSM_FIELD_PTR *CertFields )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_CrlCreateTemplate(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        uint32 NumberOfFields,
                        const CSSM_FIELD *CrlFields,
                        CSSM_DATA_PTR NewCrlTemplate )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_CertRemoveFromCrlTemplate(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_DATA *OldCrlTemplate,
                        const CSSM_CERTGROUP *CertGroupToBeRemoved,
                        const CSSM_CERTGROUP *RevokerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *RevokerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult,
                        CSSM_DATA_PTR NewCrlTemplate )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_CertGroupToTupleGroup(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_CERTGROUP *CertGroup,
                        CSSM_TUPLEGROUP_PTR *TupleGroup )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMAPI TP_TupleGroupToCertGroup(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_TUPLEGROUP *TupleGroup,
                        CSSM_CERTGROUP_PTR *CertTemplates )
{
	return CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED;
}
