/*-----------------------------------------------------------------------
 * File: MEF_MAIN.C
 *
 * Copyright (c) 2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
 * This file contains DllMain for Win32 and equivalent functions for other
 * operating systems.
 */

#include "precomp.h"


#ifdef WIN32
/*---------------------------------------------------------------
 *
 *Name: DllMain
 *
 *Description:
 *   Entry point for the DLL.
 *
 *Parameters: 
 *   hInstance (input/ignored) - Instance handle of calling process
 *   dwReason (input) - Reasons for receiving this call
 *   lpReserved (input/ignored) - Reserved field
 *
 *Returns:
 *   FALSE - unable to initialize DLL.  This will cause the LoadLibrary
 *           call to fail.
 *   TRUE - DLL is initialized and ready to be called
 *
 *-------------------------------------------------------------*/

BOOL WINAPI
DllMain(HANDLE hInstance,
        DWORD dwReason,
        LPVOID lpReserved)
{
    BOOL Success = TRUE;

    switch(dwReason)
    {
        case DLL_PROCESS_ATTACH:
        {

#ifdef _DEBUG
            /* Initialize memory tracking */
            if ( port_InitMemoryTracking() != CSSM_OK )
            {
                Success = FALSE;
            }

            OutputDebugString("In HRS EMM Process Attach");
#endif  /* _DEBUG */

            /* initialize the various locks */
            if(CSSM_OK == ERR(mef_InitMDSShim()))
            {
                if(CSSM_OK == ERR(mef_InitManage()))
                {
                    if(CSSM_OK == ERR(mef_InitIntegrity()))
                    {
                        if(CSSM_OK == ERR(mef_InitInterface()))
                        {
                            Success = TRUE;
                        }
                        else
                        {
                            Success = FALSE;
                        }

                        if(FALSE == Success)
                        {
                            ERR(mef_FiniIntegrity());
                        }
                    }

                    if(FALSE == Success)
                    {
                        ERR(mef_FiniManage());
                    }
                }

                if(FALSE == Success)
                {
                    ERR(mef_FiniMDSShim());
                }
            }
        }
        break;

        case DLL_THREAD_ATTACH:
        {
            /* we don't do anything for thread attach */
        }
        break;

        case DLL_THREAD_DETACH:
        {
            /* we don't do anything for thread detach */
        }
        break;

        case DLL_PROCESS_DETACH:
        {

            if(CSSM_OK != ERR(mef_FiniInterface()))
            {
                Success = FALSE;
            }

            if(CSSM_OK != ERR(mef_FiniIntegrity()))
            {
                Success = FALSE;
            }

            if(CSSM_OK != ERR(mef_FiniManage()))
            {
                Success = FALSE;
            }

            if(CSSM_OK != ERR(mef_FiniMDSShim()))
            {
                Success = FALSE;
            }

#ifdef _DEBUG
            /* Dump heap leaks when we are unloaded */
            port_DumpMemoryLeaks( EMM_NAME );

            OutputDebugString("In HRS EMM Process Detach");
#endif  /* _DEBUG */

        }
        break;
    }

    return Success;
}

#else /* !WIN32 */

#if defined (LINUX) || defined (VMS)
#ifdef VMS
static int _init();
static int _fini();
#else
static int _init() __attribute__ ((constructor));
static int _fini() __attribute__ ((destructor));
#endif
int _init()
{
    CSSM_BOOL Success = CSSM_TRUE;

#ifdef _DEBUG
    /* Initialize memory tracking */
    if ( port_InitMemoryTracking() != CSSM_OK )
    {
        Success = CSSM_FALSE;
    }

#endif  /* _DEBUG */

    /* initialize the various locks */
    if(CSSM_OK == ERR(mef_InitMDSShim()))
    {
        if(CSSM_OK == ERR(mef_InitManage()))
        {
            if(CSSM_OK == ERR(mef_InitIntegrity()))
            {
                if(CSSM_OK == ERR(mef_InitInterface()))
                {
                    Success = CSSM_TRUE;
                }
                else
                {
                    Success = CSSM_FALSE;
                }

                if(CSSM_FALSE == Success)
                {
                    ERR(mef_FiniIntegrity());
                }
            }

            if(CSSM_FALSE == Success)
            {
                ERR(mef_FiniManage());
            }
        }

        if(CSSM_FALSE == Success)
        {
            ERR(mef_FiniMDSShim());
        }
    }

    return Success;
}



int _fini()
{
    CSSM_BOOL Success = CSSM_TRUE;

    if(CSSM_OK != ERR(mef_FiniInterface()))
    {
        Success = CSSM_FALSE;
    }

    if(CSSM_OK != ERR(mef_FiniIntegrity()))
    {
        Success = CSSM_FALSE;
    }

    if(CSSM_OK != ERR(mef_FiniManage()))
    {
        Success = CSSM_FALSE;
    }

    if(CSSM_OK != ERR(mef_FiniMDSShim()))
    {
        Success = CSSM_FALSE;
    }

#ifdef _DEBUG

    /* Dump heap leaks when we are unloaded */
    port_DumpMemoryLeaks( EMM_NAME );

#endif  /* _DEBUG */

    return Success;
}

#endif /* LINUX || VMS */

#endif /* WIN32 */
