/*-----------------------------------------------------------------------
 * File: HRSPWBSP_INSTALL.C
 *
 * Copyright (c) 1999-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
 * INTEL CONFIDENTIAL 
 * This file, software, or program is supplied under the terms of a 
 * license agreement or nondisclosure agreement with Intel Corporation 
 * and may not be copied or disclosed except in accordance with the 
 * terms of that agreement. This file, software, or program contains 
 * copyrighted material and/or trade secret information of Intel 
 * Corporation, and must be treated as such. Intel reserves all rights 
 * in this material, except as the license agreement or nondisclosure 
 * agreement specifically indicate. 
 */


#include "precomp.h"
#include "hrsaddin_schema.h"


extern IfMdsuSchemaManipulator IfiMdsuCdsadirHrsBspCapabilitySchema;
extern IfMdsuSchemaManipulator IfiMdsuCdsadirHrsBspDynamicSchema;
 /* Number of MDS update operations that will be remembered by mds_util for
 * error clean-up during installation.
 */
#define HRS_DUMMY_ADDIN_MDS_COMMIT_SIZE 20

static CSSM_RETURN
__CleanMDSHrsDynamic(MDSU_CONTEXT *pContext)
{
    CSSM_RETURN err = CSSM_OK;
    CDSA_HRS_BSP_DYNAMIC_SCHEMA DynamicTemplate;
    CDSA_HRS_BSP_DYNAMIC_SCHEMA DynamicData;


    /* Execute a search on every record for this module to get a handle of all
     * records and then delete them.
     */

    /* Dynamic Schema */
    DynamicTemplate.ModuleID = ADDIN_GUID;
    ERR(err = MDSU_FindFirst(pContext,
                             &IfiMdsuCdsadirHrsBspDynamicSchema,
                             &DynamicTemplate,
                             MDSU_HRS_DYN_MODULE_ID,
                             &DynamicData,
                             NULL));
    while(CSSM_OK == err)
    {

        ERR(err = MDSU_FindNext(pContext,
                                &DynamicData,
                                NULL));
    }

    if(CSSMERR_DL_ENDOFDATA == err)
    {
       /* Flush everything from the MDS */
       ERR(err = MDSU_Revert(pContext));
    } 


    /* Make sure nothing gets removed if there is an error */
    if((CSSMERR_DL_ENDOFDATA != err) &&
       (CSSM_OK != err)) 
    {
        MDSU_Commit( pContext );
    }

    return err;
}


static CSSM_RETURN
__CleanMDS(MDSU_CONTEXT *pContext)
{
    CSSM_RETURN err = CSSM_OK;
    MDSU_CDSA_COMMON_SCHEMA CommonTemplate;
    MDSU_CDSA_COMMON_SCHEMA CommonData;
    MDSU_CDSA_EMM_PRIMARY_SCHEMA EMMPrimaryTemplate;
    MDSU_CDSA_EMM_PRIMARY_SCHEMA EMMPrimaryData;

    CDSA_HRS_BSP_CAPABILITIES_SCHEMA HrsBspCapTemplate;
    CDSA_HRS_BSP_CAPABILITIES_SCHEMA HrsBspCapData;

    /* Execute a search on every record for this module to get a handle of all
     * records and then delete them.
     */

    /* Common Schema */
    CommonTemplate.ModuleID = ADDIN_GUID;
    ERR(err = MDSU_FindFirst(pContext,
                             &IfiMdsuCdsadirCommonSchema,
                             &CommonTemplate,
                             MDSU_COM_MODULE_ID,
                             &CommonData,
                             NULL));
    while(CSSM_OK == err)
    {
        if(NULL != CommonData.Manifest.Data)
        {
            MDSU_free(pContext,
                      CommonData.Manifest.Data);
        }

        ERR(err = MDSU_FindNext(pContext,
                                &CommonData,
                                NULL));
    }

    if(CSSMERR_DL_ENDOFDATA == err)
    {
        /* DL Primary Schema */
        EMMPrimaryTemplate.ModuleID = ADDIN_GUID;
        ERR(err = MDSU_FindFirst(pContext,
                                 &IfiMdsuCdsadirEMMPrimarySchema,
                                 &EMMPrimaryTemplate,
                                 MDSU_EMMP_MODULE_ID,
                                 &EMMPrimaryData,
                                 NULL));
        while(CSSM_OK == err)
        {
            if(NULL != EMMPrimaryData.Manifest.Data)
            {
                MDSU_free(pContext,
                          EMMPrimaryData.Manifest.Data);
            }

            ERR(err = MDSU_FindNext(pContext,
                                    &EMMPrimaryData,
                                    NULL));
        }


        if(CSSMERR_DL_ENDOFDATA == err)
        {
            /* DL Primary Schema */
            HrsBspCapTemplate.ModuleID = ADDIN_GUID;
            ERR(err = MDSU_FindFirst(pContext,
                                     &IfiMdsuCdsadirHrsBspCapabilitySchema,
                                     &HrsBspCapTemplate,
                                     MDSU_HRS_CAP_MODULE_ID,
                                     &HrsBspCapData,
                                     NULL));
            while(CSSM_OK == err)
            {
                ERR(err = MDSU_FindNext(pContext,
                                        &HrsBspCapData,
                                        NULL));
            }

            if(CSSMERR_DL_ENDOFDATA == err)
            {
                /* Flush everything from the MDS */
                ERR(err = MDSU_Revert(pContext));

            } /* HrsBspCapability removed */
        } 
    }
    /* Make sure nothing gets removed if there is an error */
    if((CSSMERR_DL_ENDOFDATA != err) &&
       (CSSM_OK != err)) 
    {
        MDSU_Commit( pContext );
    }
    err = __CleanMDSHrsDynamic(pContext);

    return err;
}


static void hrs_Get_Bsp_Capability_Schema_Attributes(CSSM_DB_SCHEMA_ATTRIBUTE_INFO **pList)
{
    static CSSM_DB_SCHEMA_ATTRIBUTE_INFO rgAttributes[MDS_CDSADIR_HRS_BSP_CAPABILITY_NUM_ATTRIBUTES] = {
        MDS_HRS_DB_SCHEMA_MODULEID,
        MDS_HRS_DB_SCHEMA_SSID,
        MDS_HRS_DB_SCHEMA_BSPNAME,
        MDS_HRS_DB_SCHEMA_SPECVERSION,
        MDS_HRS_DB_SCHEMA_PRODVERSION,
		MDS_HRS_DB_SCHEMA_VENDOR,
		MDS_HRS_DB_SCHEMA_SUPPORTEDFORMATS,
		MDS_HRS_DB_SCHEMA_AUTHFACTORS,
		MDS_HRS_DB_SCHEMA_OPERATIONS,
		MDS_HRS_DB_SCHEMA_OPTIONS,
		MDS_HRS_DB_SCHEMA_PAYLOADPOLICY,
		MDS_HRS_DB_SCHEMA_MAXPAYLOADSIZE,
		MDS_HRS_DB_SCHEMA_DEFAULTVERIFYTIMEOUT,
		MDS_HRS_DB_SCHEMA_DEFAULTIDENTIFYTIMEOUT,
		MDS_HRS_DB_SCHEMA_DEFAULTCAPTURETIMEOUT,
		MDS_HRS_DB_SCHEMA_DEFAULTENROLLTIMEOUT,
		MDS_HRS_DB_SCHEMA_MAXBSPDBSIZE,
		MDS_HRS_DB_SCHEMA_MAXIDENTIFY
    };

    *pList = rgAttributes;
}

static void hrs_Get_Bsp_Capability_Indexes(CSSM_DB_SCHEMA_INDEX_INFO ** pList)
{
    static CSSM_DB_SCHEMA_INDEX_INFO rgIndexes[MDS_CDSADIR_HRS_BSP_CAPABILITY_NUM_ATTRIBUTES] = {
        MDS_HRS_DB_INDEX_SCHEMA_MODULEID,
        MDS_HRS_DB_INDEX_SCHEMA_SSID,
        MDS_HRS_DB_INDEX_SCHEMA_BSPNAME,
        MDS_HRS_DB_INDEX_SCHEMA_SPECVERSION,
        MDS_HRS_DB_INDEX_SCHEMA_PRODVERSION,
		MDS_HRS_DB_INDEX_SCHEMA_VENDOR,
		MDS_HRS_DB_INDEX_SCHEMA_SUPPORTEDFORMATS,
		MDS_HRS_DB_INDEX_SCHEMA_AUTHFACTORS,
		MDS_HRS_DB_INDEX_SCHEMA_OPERATIONS,
		MDS_HRS_DB_INDEX_SCHEMA_OPTIONS,
		MDS_HRS_DB_INDEX_SCHEMA_PAYLOADPOLICY,
		MDS_HRS_DB_INDEX_SCHEMA_MAXPAYLOADSIZE,
		MDS_HRS_DB_INDEX_SCHEMA_DEFAULTVERIFYTIMEOUT,
		MDS_HRS_DB_INDEX_SCHEMA_DEFAULTIDENTIFYTIMEOUT,
		MDS_HRS_DB_INDEX_SCHEMA_DEFAULTCAPTURETIMEOUT,
		MDS_HRS_DB_INDEX_SCHEMA_DEFAULTENROLLTIMEOUT,
		MDS_HRS_DB_INDEX_SCHEMA_MAXBSPDBSIZE,
		MDS_HRS_DB_INDEX_SCHEMA_MAXIDENTIFY
    };

    *pList = rgIndexes;
}

static void hrs_Get_Bsp_Dynamic_Schema_Attributes(CSSM_DB_SCHEMA_ATTRIBUTE_INFO **pList)
{
    static CSSM_DB_SCHEMA_ATTRIBUTE_INFO rgDynamicAttributes[MDS_CDSADIR_HRS_BSP_DYNAMIC_NUM_ATTRIBUTES] = {
        MDS_HRS_DB_SCHEMA_MODULEID,
        MDS_HRS_DB_SCHEMA_SSID,
        MDS_HRS_DB_SCHEMA_BSPNAME,
		MDS_HRS_DB_SCHEMA_SUPPORTEDFORMATS,
		MDS_HRS_DB_SCHEMA_SUPPORTEDEVENTS,
		MDS_HRS_DB_SCHEMA_DEVICEVENDOR,
		MDS_HRS_DB_SCHEMA_DEVICEDESCRIPTION,
		MDS_HRS_DB_SCHEMA_DEVICESERIALNUMBER,
        MDS_HRS_DB_SCHEMA_DEVICEHWVERSION,
        MDS_HRS_DB_SCHEMA_DEVICEFIRMWAREVERSION,
        MDS_HRS_DB_SCHEMA_AUTHENTICATEDDEVICE
    };

    *pList = rgDynamicAttributes;
}

static void hrs_Get_Bsp_Dynamic_Schema_Indexes(CSSM_DB_SCHEMA_INDEX_INFO ** pList)
{
    static CSSM_DB_SCHEMA_INDEX_INFO rgDynamicIndexes[MDS_CDSADIR_HRS_BSP_DYNAMIC_NUM_ATTRIBUTES] = {
        MDS_HRS_DB_INDEX_SCHEMA_MODULEID,
        MDS_HRS_DB_INDEX_SCHEMA_SSID,
        MDS_HRS_DB_INDEX_SCHEMA_BSPNAME,
		MDS_HRS_DB_INDEX_SCHEMA_SUPPORTEDFORMATS,
		MDS_HRS_DB_INDEX_SCHEMA_SUPPORTEDEVENTS,
		MDS_HRS_DB_INDEX_SCHEMA_DEVICEVENDOR,
		MDS_HRS_DB_INDEX_SCHEMA_DEVICEDESCRIPTION,
		MDS_HRS_DB_INDEX_SCHEMA_DEVICESERIALNUMBER,
        MDS_HRS_DB_INDEX_SCHEMA_DEVICEHWVERSION,
        MDS_HRS_DB_INDEX_SCHEMA_DEVICEFIRMWAREVERSION,
        MDS_HRS_DB_INDEX_SCHEMA_AUTHENTICATEDDEVICE
    };

    *pList = rgDynamicIndexes;
}

/* Creates the new relation*/
CSSM_RETURN CSSMAPI hrs_CreateMDSRelation (const char *szDirectoryName,
                                           CSSM_DB_RECORDTYPE RelationID,
                                           const char *RelationName,
                                           uint32 NumberOfAttributes,
                                           const CSSM_DB_SCHEMA_ATTRIBUTE_INFO *pAttributeInfo,
                                           uint32 NumberOfIndexes,
                                           const CSSM_DB_SCHEMA_INDEX_INFO *pIndexInfo)
{

    static MDS_FUNCS s_hrsMDSFuncs;
    MDS_DB_HANDLE hMds;
    CSSM_RETURN ret;
    if ( ( ret = MDS_Initialize( &ADDIN_GUID,
                                NULL,
                                &Addin_APIMemFuncs,
                                &s_hrsMDSFuncs,
                                &(hMds.DLHandle) ) ) == CSSM_OK )
    {
        /* Open the requested MDS directory */
        if ( ( ret = s_hrsMDSFuncs.DbOpen( hMds.DLHandle,
                                         szDirectoryName,
                                         NULL,
                                         CSSM_DB_ACCESS_READ | CSSM_DB_ACCESS_WRITE | CSSM_DB_ACCESS_PRIVILEGED,
                                         NULL,
                                         NULL,
                                         &hMds.DBHandle ) ) == CSSM_OK )
        {
            ret = s_hrsMDSFuncs.CreateRelation(
                                            hMds,
                                            RelationID,
                                            RelationName,
                                            NumberOfAttributes,
                                            pAttributeInfo,
                                            NumberOfIndexes,
                                            pIndexInfo);

        }
		 
		s_hrsMDSFuncs.DbClose ( hMds );
		 
        MDS_Terminate( hMds.DLHandle );

    } /* MDS_Initialize */
    return (ret);
}

CSSM_RETURN CSSMAPI hrs_UpdateDynamicInfo( CDSA_HRS_BSP_DYNAMIC_SCHEMA* HrsBspDynamic, 
                                          CSSM_BOOL bInstall)
{
    CSSM_RETURN err = CSSM_OK;
    MDSU_CONTEXT MDSUContext;

    /* Initialize MDS */
    if (ERR(err = MDSU_Init(&MDSUContext, 
						    &Addin_APIMemFuncs,
						    &ADDIN_GUID,
						    MDS_CDSA_DIRECTORY,
						    (CSSM_DB_ACCESS_READ | CSSM_DB_ACCESS_WRITE),
						    HRS_DUMMY_ADDIN_MDS_COMMIT_SIZE)) == CSSM_OK)
    {
        if( ( bInstall == CSSM_TRUE)  && (HrsBspDynamic != NULL) )
        {
	        /* Insert the common schema record into the MDS */
		    ERR(err = MDSU_UpdateSchema(&MDSUContext,
		    					        &IfiMdsuCdsadirHrsBspDynamicSchema,
								        HrsBspDynamic,
								        NULL));
	        if(CSSM_OK == err)
	        {
                /* commit the records */
			    MDSU_Commit(&MDSUContext);

                /* reset error to CSSM_OK */
                err = CSSM_OK;
		    }
	        else
		    {
		        /* there's been an error, so we back out all changes */
			    MDSU_Revert( &MDSUContext );
		    }
        }
        else
        {
			err = __CleanMDSHrsDynamic(&MDSUContext);

        }
			        /* Shut down the MDS connection */
        MDSU_Term( &MDSUContext );
    }
    return err;
}

CSSM_RETURN CSSMAPI
INSTALL_ENTRY_NAME(const char *szModuleName,
                   const char *szSearchPath,
                   CSSM_DATA *pCredential,
                   INSTALL_ACTION Action,
                   INSTALL_ERROR *pError)
{
    CSSM_RETURN err = CSSM_OK;
    MDSU_CONTEXT MDSUContext;
    MDSU_CDSA_COMMON_SCHEMA Common;
    MDSU_CDSA_EMM_PRIMARY_SCHEMA EMMPrimary;
    CSSM_DB_SCHEMA_ATTRIBUTE_INFO_PTR CapAttrInfo = NULL;
    CSSM_DB_SCHEMA_INDEX_INFO_PTR CapIndexInfo = NULL;
    CDSA_HRS_BSP_CAPABILITIES_SCHEMA HrsBspCapability;
    CSSM_DB_SCHEMA_ATTRIBUTE_INFO_PTR DynamicAttrInfo = NULL;
    CSSM_DB_SCHEMA_INDEX_INFO_PTR DynamicIndexInfo = NULL;
    CDSA_HRS_BSP_DYNAMIC_SCHEMA HrsBspDynamic;

    if (INSTALL_ACTION_UNINSTALL != Action)
    {
	    hrs_Get_Bsp_Capability_Schema_Attributes(&CapAttrInfo);
	    hrs_Get_Bsp_Capability_Indexes(&CapIndexInfo);
	    err = hrs_CreateMDSRelation (MDS_CDSA_DIRECTORY,
                               MDS_CDSADIR_HRS_BSP_CAPABILITY_RECORDTYPE,
                               HRS_BSP_CAPABILITIES,
                               MDS_CDSADIR_HRS_BSP_CAPABILITY_NUM_ATTRIBUTES,
                               CapAttrInfo,
                               MDS_CDSADIR_HRS_BSP_CAPABILITY_NUM_ATTRIBUTES,
                               CapIndexInfo);
        if( err == CSSM_OK || err == CSSMERR_DL_INVALID_RECORDTYPE )
        {
	        hrs_Get_Bsp_Dynamic_Schema_Attributes(&DynamicAttrInfo);
	        hrs_Get_Bsp_Dynamic_Schema_Indexes(&DynamicIndexInfo);
	        err = hrs_CreateMDSRelation (MDS_CDSA_DIRECTORY,
                                   MDS_CDSADIR_HRS_BSP_DYNAMIC_RECORDTYPE,
                                   HRS_BSP_DYNAMIC,
                                   MDS_CDSADIR_HRS_BSP_DYNAMIC_NUM_ATTRIBUTES,
                                   DynamicAttrInfo,
                                   MDS_CDSADIR_HRS_BSP_DYNAMIC_NUM_ATTRIBUTES,
                                   DynamicIndexInfo);
        }
    }
	if( err == CSSM_OK || err == CSSMERR_DL_INVALID_RECORDTYPE )
	{
	    /* Initialize MDS */
	    if (ERR(err = MDSU_Init(&MDSUContext, 
							    &Addin_APIMemFuncs,
							    &ADDIN_GUID,
							    MDS_CDSA_DIRECTORY,
							    (CSSM_DB_ACCESS_READ | CSSM_DB_ACCESS_WRITE),
							    HRS_DUMMY_ADDIN_MDS_COMMIT_SIZE)) == CSSM_OK)
	    {
			err = __CleanMDS(&MDSUContext);

			if((CSSM_OK == err) &&
			   (INSTALL_ACTION_UNINSTALL != Action))
			{
				/* Fill in the "easy" common schema structure */
				memcpy( &Common.ModuleID, &ADDIN_GUID, sizeof(ADDIN_GUID) );
				Common.Manifest = *pCredential;
				strcpy( Common.ModuleName, szModuleName );
				strcpy( Common.Path, szSearchPath );
				Common.CDSAVersion.Major = CSSM_MAJOR;
				Common.CDSAVersion.Minor = CSSM_MINOR;
				strcpy( Common.Desc, ADDIN_NAME );
				Common.DynamicFlag = CSSM_FALSE;
				Common.MultiThreadFlag = CSSM_FALSE;
				Common.ServiceMask = CSSM_SERVICE_HRS;

				/* Insert the common schema record into the MDS */
				ERR(err = MDSU_UpdateSchema(&MDSUContext,
											&IfiMdsuCdsadirCommonSchema,
											&Common,
											NULL));

				/* if there hasn't been an error, we'll move to updating the
				   primary schema
				*/
				if(CSSM_OK == err)
				{
                    /* reset error to CSSM_OK */
                    err = CSSM_OK;


					/* Fill in the EMM primary schema data */
					memcpy(&EMMPrimary.ModuleID,
						   &ADDIN_GUID,
						   sizeof(ADDIN_GUID));

					EMMPrimary.SSID = ADDIN_HRS_SUBSERVICE_ID;
					EMMPrimary.ServiceType = CSSM_SERVICE_HRS;
					EMMPrimary.Manifest.Data = NULL;
					EMMPrimary.Manifest.Length = 0;
					strcpy( EMMPrimary.ModuleName, ADDIN_NAME );
					EMMPrimary.Version.Major = ADDIN_PRODUCT_VERSION_MAJOR;
					EMMPrimary.Version.Minor = ADDIN_PRODUCT_VERSION_MINOR;

					strcpy(EMMPrimary.Vendor,
						   "Intel Corporation");

					EMMPrimary.NumSampleTypes = 0;
					memset(EMMPrimary.SampleTypes,
						   0,
						   sizeof(uint32) * MDSU_MAX_VALUE_COUNT);

					EMMPrimary.NumAclSubjectTypes = 0;
					memset(EMMPrimary.AclSubjectTypes,
						   0,
						   sizeof(uint32) * MDSU_MAX_VALUE_COUNT);

					EMMPrimary.NumAuthTags = 0;
					memset(EMMPrimary.AuthTags,
						   0,
						   sizeof(uint32) * MDSU_MAX_VALUE_COUNT);

					EMMPrimary.EmmSpecVersion.Major = HRS_VERSION_MAJOR;
					EMMPrimary.EmmSpecVersion.Minor = HRS_VERSION_MINOR;

					/* Write the data to MDS */
					ERR(err = MDSU_UpdateSchema(&MDSUContext,
												&IfiMdsuCdsadirEMMPrimarySchema,
												&EMMPrimary,
												NULL));
					if(CSSM_OK == err)
					{
                        /* reset error to CSSM_OK */
                        err = CSSM_OK;

						memcpy( &HrsBspCapability.ModuleID, &ADDIN_GUID, sizeof(ADDIN_GUID) );						
						
						
						HrsBspCapability.SSID = 0; 	                        
						
						strcpy( HrsBspCapability.BSPName, "HRS Password BSP" );
						strcpy( HrsBspCapability.Vendor,  "Intel Corporation");
                        HrsBspCapability.SpecVersion.Major = HRS_VERSION_MAJOR;
						HrsBspCapability.SpecVersion.Minor = HRS_VERSION_MINOR;
						HrsBspCapability.ProdVersion.Major = ADDIN_PRODUCT_VERSION_MAJOR;
						HrsBspCapability.ProdVersion.Minor = ADDIN_PRODUCT_VERSION_MINOR;
						
						HrsBspCapability.NumSupportedformats    = 1;

						HrsBspCapability.AuthFactors            = CSSM_HRS_FACTOR_PASSWORD;
						HrsBspCapability.Operations             = CSSM_HRS_CAPTURE  | CSSM_HRS_CREATETEMPLATE |
																  CSSM_HRS_PROCESS  | CSSM_HRS_VERIFYMATCH    |
																  CSSM_HRS_ENROLL   | CSSM_HRS_VERIFY;
						
						HrsBspCapability.Options                = CSSM_HRS_BIR_ENCRYPT | 
						                                          CSSM_HRS_PAYLOAD     |
						                                          CSSM_HRS_LOCAL_BSP;
						
						/* False Accept Rate is zero - only the right password will be         */
						/* accepted.  Collisions in the password hash are about as likely as   */
						/* a tornado blowing through a junkyard and putting together a new car */
						HrsBspCapability.PayloadPolicy          = 0;

						HrsBspCapability.MaxPayloadSize         = 0xFFFFFFFF;
						
						/* Are there values for these that mean indefinite?  This BSP uses a 
						 * modal dialog to capture the password; it can be canceled at any 
						 * time, but will remain up until the user acts on it one way or 
						 * another. */
						HrsBspCapability.DefaultVerifyTimeout   = DEFAULT_TIMEOUT;
						HrsBspCapability.DefaultCaptureTimeout  = DEFAULT_TIMEOUT;
						HrsBspCapability.DefaultEnrollTimeout   = DEFAULT_TIMEOUT;

						/* BSP does not support database functions */
						HrsBspCapability.MaxBspDbSize           = 0;

						/* BSP does not support Identify function  */
						HrsBspCapability.DefaultIdentifyTimeout = 0;
						HrsBspCapability.MaxIdentify            = 0;

						/* Write the data to MDS */
						ERR(err = MDSU_UpdateSchema(&MDSUContext,
													&IfiMdsuCdsadirHrsBspCapabilitySchema,
													&HrsBspCapability,
													NULL));
						
						if(CSSM_OK == err)
						{
							/* commit the records */
							MDSU_Commit(&MDSUContext);

                            err = CSSM_OK;
						}
						else
						{
							/* there's been an error, so we back out all changes */
							MDSU_Revert( &MDSUContext );
							sprintf(pError->ErrorString,
									"MDS Error (DL Primary): %X\n",
									err);
							pError->ErrorCode = err;
						}
					}
				}
				else
				{
					sprintf(pError->ErrorString,
							"MDS Error (common-DL): %X\n",
							err);
					pError->ErrorCode = err;
				}
			}

			/* Shut down the MDS connection */
			MDSU_Term( &MDSUContext );
			if((CSSM_OK == err) &&
			   (INSTALL_ACTION_UNINSTALL != Action))
            {
				/* Updating of Dynamic schema. This should be */
				/* called after the device is inserted        */

				/* The Intel HRS Password BSP is a static software module. */
				/* Most of the data below is not pertinent.                */
                memcpy( &HrsBspDynamic.ModuleID, &ADDIN_GUID, sizeof(ADDIN_GUID) );

				HrsBspDynamic.SSID = 0;
				
                strcpy( HrsBspDynamic.BSPName, "HRS Password BSP");
			    
				HrsBspDynamic.NumSupportedformats = 1;

			    HrsBspDynamic.SupportedEvents = 0;
                strcpy( HrsBspDynamic.DeviceVendor,       "Intel Corporation");
                strcpy( HrsBspDynamic.DeviceDescription,  "HRS Password BSP");
                strcpy( HrsBspDynamic.DeviceSerialNumber, "HRS Password BSP");
                HrsBspDynamic.DeviceHWVersion.Major       = ADDIN_PRODUCT_VERSION_MAJOR;
			    HrsBspDynamic.DeviceHWVersion.Minor       = ADDIN_PRODUCT_VERSION_MINOR;
                HrsBspDynamic.DeviceFirmwareVersion.Major = ADDIN_PRODUCT_VERSION_MAJOR;
			    HrsBspDynamic.DeviceFirmwareVersion.Minor = ADDIN_PRODUCT_VERSION_MINOR;
                
				
				/* By virtue of the module being attached to the CSSM    */
				/* under the EISL, the "device" should be authenticated. */
				HrsBspDynamic.AuthenticatedDevice         = CSSM_TRUE;
                hrs_UpdateDynamicInfo( &HrsBspDynamic, CSSM_TRUE);
            }
		}
		else
		{
			sprintf(pError->ErrorString,
					"MDS Error (Init): %X\n",
					err);
			pError->ErrorCode = err;
		}
	}
    return err;
}

