/*-----------------------------------------------------------------------
 * File: MAF_INCLUDE.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
 * Main include file for MAF. All files should include this file first
 * before any custom files. This will insure proper inclusion of all MAF
 * files. The file "maf_setup.h" should be created by the SP developer to
 * limit the included files to those used by a specific SP type.
 */

#ifndef __MAF_INCLUDE_H__
#define __MAF_INCLUDE_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Standard headers */
#include <assert.h>

/* CSSM headers */
#define NO_CSSM_EMM_API /* Only include API and SPI definitions */
#include "cssm.h"

/* Porting library headers */
#include "port/cssmport.h"
#include "port/cssmlock.h"

/* MAF headers required by all SPs */
#include "maf_collectn.h"
#include "maf_integ.h"
#include "maf_util.h"
#include "maf_mds.h"

/* MAF utility headers containing definitions often used in load and/or
 * attach tracking structures.
 */
#ifdef VMS
#include <maf_api.h>
#else
#include "util/maf_api.h"
#endif
/* Project specific configuration */
#ifndef MAF_NO_CONFIG_HEADER
#include "maf_config.h"
#else
  /* Make the other headers think maf_config.h was included */
#define MAF_CONFIG_H
#endif

/* MAF headers required by all SPs; cont. */
#include "maf_interface.h"
#include "maf_callouts.h"

/* MAF Utility headers usable by all SPs */
#ifdef VMS
#include <maf_dbg.h>
#include <maf_misc.h>
#include <maf_validate.h>
#include <maf_list.h>
#include <maf_acl.h>
#else
#include "util/maf_dbg.h"
#include "util/maf_misc.h"
#include "util/maf_validate.h"
#include "util/acl/maf_list.h"
#include "util/acl/maf_acl.h"
#endif

#if defined(MAF_CSM_INCLUDED) || defined(MAF_CLM_INCLUDED) || defined(MAF_NEED_BERDER)
/* BER/DER headers required by utility functions */
#include "ber_der.h"
#endif

/* MAF-CSM headers; CSP specific */
#ifdef MAF_CSM_INCLUDED

#include "CSM/csm_cspi.h"
#include "CSM/csm_sesn.h"
#include "CSM/csm_cntx.h"
#include "CSM/csm_vect.h"

#endif /* MAF_CSM_INCLUDED */

#if defined(MAF_CSM_INCLUDED) || defined(MAF_NEED_KEYPARSE)
#include "CSM/csm_keyparse.h"
#endif

/* MAF-DLM headers; DL specific */
#ifdef MAF_DLM_INCLUDED


#endif /* MAF_DLM_INCLUDED */

/* EISL services library */
#ifdef ISL_INCLUDED
#include "eislapi.h"
#endif

#ifdef __cplusplus
}
#endif

#endif /* ifndef __MAF_INCLUDE_H__ */
