/*-----------------------------------------------------------------------
 * File: SAMPLEGROUP.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "maf_include.h"

/* Deallocates the sample contents, but not the sample structure */
CSSM_RETURN MAF_FreeSample(
                        CSSM_HANDLE hModule,
                        CSSM_SAMPLE *pSample )
{
    CSSM_RETURN rv = CSSM_OK;

    if ( pSample )
    {
        /* Free the verifier is present */
        if ( pSample->Verifier != NULL )
        {
            MAF_FREE( hModule, (CSSM_SUBSERVICE_UID*)pSample->Verifier );
        }

        /* Free the list structure */
        MAF_DeallocateCSSMList( hModule, &pSample->TypedSample );
    }

    return rv;
}

/* Deallocates all samples, but does not deallocate the group structure */
CSSM_RETURN CSSMAPI MAF_FreeSampleGroup(
                        CSSM_HANDLE hModule,
                        CSSM_SAMPLEGROUP *pSampleGroup )
{
    CSSM_RETURN rv = CSSM_OK;
    uint32 i;

    if ( pSampleGroup != NULL )
    {
        /* Free the samples */
        for ( i = 0; i < pSampleGroup->NumberOfSamples; i++ )
        {
            MAF_FreeSample( hModule, 
                            (CSSM_SAMPLE*)&pSampleGroup->Samples[i] );
        }

        /* Free the sample structure array */
        MAF_FREE( hModule, pSampleGroup );
    }

    return rv;
}

