/*-----------------------------------------------------------------------
 * File: SUBJECT.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "maf_include.h"

CSSM_RETURN CSSMAPI MAF_GetAclSubjectType(
                        const CSSM_LIST *pTypedSubject,
                        CSSM_ACL_SUBJECT_TYPE *pSubjectType )
{
    CSSM_RETURN rv = CSSM_OK;
    const CSSM_LIST_ELEMENT *pElement;

    assert( pTypedSubject && pSubjectType );

    pElement = pTypedSubject->Head;
    if ( pElement == NULL )
    {
        ERR( rv = CSSM_ERRCODE_INVALID_NEW_ACL_ENTRY );
    }
    else if ( ( pTypedSubject->ListType != CSSM_LIST_TYPE_SEXPR ) ||
              ( pElement->ElementType != CSSM_LIST_ELEMENT_WORDID ) )
    {
        ERR( rv = CSSM_ERRCODE_INVALID_NEW_ACL_ENTRY );
    }
    else
    {
        *pSubjectType = pElement->WordID;
    }

    return rv;
}

CSSM_BOOL CSSMAPI MAF_SubjectIsResolvableByChallenge(
                        const CSSM_ACL_ENTRY_INPUT *pSubject )
{
    CSSM_BOOL rv = CSSM_TRUE;
    const CSSM_LIST_ELEMENT *pElement = NULL;

    assert( !port_IsBadReadPtr( pSubject, sizeof(CSSM_ACL_ENTRY_INPUT) ) );

    if ( ( port_IsBadCodePtr( (CSSM_PROC_ADDR)pSubject->Callback ) ) ||
         ( pSubject->Prototype.TypedSubject.ListType != CSSM_LIST_TYPE_SEXPR ) )
    {
        rv = CSSM_FALSE;
    }
    else
    {
        pElement = pSubject->Prototype.TypedSubject.Head;
        if ( ( pElement == NULL ) ||
             ( pElement->ElementType != CSSM_LIST_ELEMENT_WORDID ) )
        {
            rv = CSSM_FALSE;
        }
        else
        {
            switch ( pElement->WordID )
            {
            case CSSM_ACL_SUBJECT_TYPE_THRESHOLD:
                {
                    /* Unsupported ACL subject, so return an error */
                    rv = CSSM_FALSE;
                    break;
                }
            default:
                {
                    /* All subject challenge specifications only have a
                     * single list element with a WORDID type. If there are
                     * any additional elements, this is an invalid challenge
                     * specification.
                     */
                    if ( pElement->NextElement != NULL )
                    {
                        rv = CSSM_FALSE;
                    }
                }
            }
        }
    }

    return rv;
}

