/*-----------------------------------------------------------------------
 * File: MAF_VALIDATE.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef __MAF_VALIDATE_H__
#define __MAF_VALIDATE_H__

CSSM_RETURN CSM_ValidateCryptoData( const CSSM_CRYPTO_DATA *pCryptoData,
                                   CSSM_BOOL bAllowEmpty,
                                   CSSM_BOOL bAllowNULL );

CSSM_RETURN CSM_ValidateInputBuffers( 
                        const CSSM_DATA *pInBufs,
                        uint32 uNumInBufs,
                        CSSM_BOOL IsVector );

CSSM_RETURN CSM_ValidateOutputBuffers(
                        CSSM_DATA *pOutBufs,
                        uint32 uNumOutBufs,
                        CSSM_BOOL IsVector );

CSSM_RETURN CSM_ValidateInputKey( const CSSM_KEY *pKey );

CSSM_RETURN CSM_ValidateInputKeyFormat(const CSSM_KEY *pKey);

CSSM_RETURN CSM_ValidateOutputKey( const CSSM_KEY *pKey );

CSSM_BOOL CSM_IsValidKeyReturnType( uint32 KeyAttr );

#define CSM_VALID_PUBLIC_KEY_USES       ( CSSM_KEYUSE_ANY | \
                                          CSSM_KEYUSE_ENCRYPT | \
                                          CSSM_KEYUSE_DECRYPT | \
                                          CSSM_KEYUSE_VERIFY | \
                                          CSSM_KEYUSE_VERIFY_RECOVER | \
                                          CSSM_KEYUSE_WRAP | \
                                          CSSM_KEYUSE_DERIVE )
#define CSM_VALID_PRIVATE_KEY_USES      ( CSSM_KEYUSE_ANY | \
                                          CSSM_KEYUSE_ENCRYPT | \
                                          CSSM_KEYUSE_DECRYPT | \
                                          CSSM_KEYUSE_SIGN | \
                                          CSSM_KEYUSE_SIGN_RECOVER | \
                                          CSSM_KEYUSE_UNWRAP | \
                                          CSSM_KEYUSE_DERIVE )
#define CSM_VALID_SYMMETRIC_KEY_USES    ( CSSM_KEYUSE_ANY | \
                                          CSSM_KEYUSE_ENCRYPT | \
                                          CSSM_KEYUSE_DECRYPT | \
                                          CSSM_KEYUSE_SIGN | \
                                          CSSM_KEYUSE_VERIFY | \
                                          CSSM_KEYUSE_WRAP | \
                                          CSSM_KEYUSE_UNWRAP | \
                                          CSSM_KEYUSE_DERIVE )

CSSM_BOOL CSM_IsValidKeyAttributes( uint32 KeyAttr );

CSSM_BOOL CSM_IsValidKeyUsage( uint32 KeyUsage,
                               uint32 ValidUsage );

CSSM_RETURN MAF_ValidateNetAddress( const CSSM_NET_ADDRESS *pAddress );

CSSM_RETURN MAF_ValidateData( const CSSM_DATA *pData,
                             CSSM_BOOL bAllowEmpty,
                             CSSM_BOOL bWriteable );

CSSM_RETURN MAF_ValidateDbAttributeInfo( const CSSM_DB_ATTRIBUTE_INFO *pInfo );

CSSM_RETURN MAF_ValidateDbRecordAttribute(
                            const CSSM_DB_ATTRIBUTE_DATA *pAttribute,
                            CSSM_BOOL bRequireEmptyBufs );

CSSM_RETURN MAF_ValidateSelectionPredicate(
                                const CSSM_SELECTION_PREDICATE *pPredicate );

CSSM_RETURN MAF_ValidateQuery( const CSSM_QUERY *pQuery );

CSSM_RETURN MAF_ValidateInputDbRecordAttributeData(
                        const CSSM_DB_RECORD_ATTRIBUTE_DATA *pAttributes );

CSSM_RETURN MAF_ValidateOutputDbRecordAttributeData(
                        const CSSM_DB_RECORD_ATTRIBUTE_DATA *pAttributes );

#endif /* ifndef __MAF_VALIDATE_H__ */
