#include "spkiutil.h"


/* Removes leading 0 bytes in a SPKI integer and returns the number of bytes
   deleted */
static uint32 RemoveLeadingZeros(CSSM_DATA *Data)
{
    uint32 deleted=0;
    if (!Data || !Data->Length || !Data->Data)
    {
        return deleted;
    }

    while (!Data->Data[0] )
    {
        Data->Length--;
        Data->Data++;
        deleted++;
    }

    return deleted;
}


CSSM_RETURN HandleLegacyIntegers(const CSSM_LIST *SigList, 
                                        const CSSM_LIST *PKList)
{
    CSSM_LIST_ELEMENT_PTR w, y;
    CSSM_LIST SigValList;


    if (!PKList || !SigList)
    {
        return CSSMERR_CL_INTERNAL_ERROR;
    }

    /* Lets walk the SigList and arrive at the SigVal List */
    w = SigList->Head;
    if (!w)
    {
        
        return CSSMERR_CL_UNKNOWN_FORMAT;
    }
    w = w->NextElement; /* w points to elt containing hash-sub-list */
    if (!w)
    {
        
        return CSSMERR_CL_UNKNOWN_FORMAT;
    }
    w = w->NextElement; /* w points to elt containing pk-sub-list */
    if (!w)
    {    
        return CSSMERR_CL_UNKNOWN_FORMAT;
    }
    w = w->NextElement; /* w points to elt containing sig-val-sub-list */
    if (!w || w->ElementType != CSSM_LIST_ELEMENT_SUBLIST)
    {
        return CSSMERR_CL_UNKNOWN_FORMAT;
    }
    SigValList = w->Element.Sublist;
    
    /* Now lets walk the PK and massage integers if we find any legacy integers */
    w = PKList->Head;
    if (!w)
    {
        return CSSMERR_CL_UNKNOWN_FORMAT;
    }
    w = w->NextElement;
    if (!w || w->ElementType != CSSM_LIST_ELEMENT_SUBLIST)
    {
        return CSSMERR_CL_UNKNOWN_FORMAT;
    }
    

    w = w->Element.Sublist.Head;
    if (!w)
    {
        return CSSMERR_CL_UNKNOWN_FORMAT;
    }
    switch(w->WordID)
    {
        case CSSM_WORDID_RSA_PKCS1_MD5:
            
        case CSSM_WORDID_RSA_PKCS1_SHA1:
            
        case CSSM_WORDID_RSA_PKCS1:
            w = w->NextElement;
            /* e CSM_RSA_PUBLIC_EXPONENT */
            if (!w)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
                w = w->NextElement;
            /* n CSM_RSA_MODULUS */
            if (!w)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = w->Element.Sublist.Head;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = y->NextElement;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            
            RemoveLeadingZeros(&(y->Element.Word));
            /* walk to sigval and remove deleted bytes from beginning */
            w = SigValList.Head; /* w now points to the algid node */
            w = w->NextElement;
            if (!w)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            /* w now points to the sigval node */
            RemoveLeadingZeros(&(w->Element.Word));
            break;

        case CSSM_WORDID_DSA_SHA1:

            w = w->NextElement;
            /* p  CSM_DSA_PRIME */
            if (!w)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = w->Element.Sublist.Head;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = y->NextElement;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            RemoveLeadingZeros(&(y->Element.Word));

            /* g CSM_DSA_BASE */
            w = w->NextElement;
            if (!w)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = w->Element.Sublist.Head;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = y->NextElement;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            RemoveLeadingZeros(&(y->Element.Word));

            /* q CSM_DSA_SUB_PRIME */
            w = w->NextElement;
            if (!w)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = w->Element.Sublist.Head;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = y->NextElement;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            RemoveLeadingZeros(&(y->Element.Word));

            /* pub_key y CSM_DSA_PUBLIC_VALUE */
            w = w->NextElement;
            if (!w)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = w->Element.Sublist.Head;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = y->NextElement;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            RemoveLeadingZeros(&(y->Element.Word));
            /* walk to sigval and remove deleted bytes from beginning */
            w = SigValList.Head; /* w now points to the algid node */
            if (!w)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            
            w = w->NextElement; /* w points to the elt having the r sublist */
            if (!w || w->ElementType != CSSM_LIST_ELEMENT_SUBLIST)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = w->Element.Sublist.Head;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = y->NextElement;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            RemoveLeadingZeros(&(y->Element.Word));

            w = w->NextElement; /* w points to the elt having the s sublist */
            if (!w || w->ElementType != CSSM_LIST_ELEMENT_SUBLIST)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = w->Element.Sublist.Head;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            y = y->NextElement;
            if (!y)
            {
                return CSSMERR_CL_UNKNOWN_FORMAT;
            }
            RemoveLeadingZeros(&(y->Element.Word));

            break;
    }
    return CSSM_OK;

}

