/*-----------------------------------------------------------------------
 * File: BASE64.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/* 
 * This library base64 encodes a byte string
 */

#include "precomp.h"

uint8 Base64_DecodeChar(char EncodedChar) 
{
    uint8 value = 0;

    if ((EncodedChar >= 'A') && (EncodedChar  <= 'Z'))
        value = (uint8)(EncodedChar - 'A');
    else if ((EncodedChar >= 'a') && (EncodedChar  <= 'z'))
        value = (uint8)(EncodedChar - 'a' + 26);
    else if ((EncodedChar >= '0') && (EncodedChar  <= '9'))
        value = (uint8)(EncodedChar - '0' + 52);
    else if (EncodedChar == '+')
        value = 62;
    else if (EncodedChar == '/')
        value = 63;

    return value;
}

/* This functions returns the (right adjusted) n-bit field of x that
   begins at position p.  Assume that bit position 0 is at the right end */ 
uint8 Base64_GetBits(uint32 x, int p, int n) 
{

    return(uint8)((x>>(p+1-n)) & ~(~0 <<n));
}

/* This function performs base64 decoding - if no handle is provided,
   the memory will be allocated off the local heap
*/
CSSM_DATA_PTR CSSMAPI
Base64_Decode(CSSM_HANDLE AppHandle, 
              const CSSM_UPCALLS *MemoryFunctions, 
              const CSSM_DATA *pEncodedData)
{
    uint32 i;
    uint32 NumberOfChars = 0;
    uint32 j, l;
    int pos;
    CSSM_DATA_PTR pDecodedData = NULL;

    /* calculate how many valid charcaters are there in the encoded data */
    i = 0;
    while (i < pEncodedData->Length && pEncodedData->Data[i] != '=') {
        if ((pEncodedData->Data[i] >= 'A' && pEncodedData->Data[i] <= 'Z') ||
             (pEncodedData->Data[i] >= 'a' && pEncodedData->Data[i] <= 'z') ||
             (pEncodedData->Data[i] >= '0' && pEncodedData->Data[i] <= '9') ||
             (pEncodedData->Data[i] == '+') || (pEncodedData->Data[i] == '/')) {
                NumberOfChars++;
                i++;
        }
        else if ((pEncodedData->Data[i] == '\n') ||(pEncodedData->Data[i] == '\r'))
            i++;
        else 
            return NULL; /* Encoded data contains invalid character */

    }

    if(0 != AppHandle)
    {
        pDecodedData = (CSSM_DATA_PTR) CSSM_MALLOC_WRAPPER(MemoryFunctions->malloc_func,
                                                           AppHandle,
                                                           sizeof(CSSM_DATA));
        if (pDecodedData == NULL)
            return NULL;

        /* allocate the output buffer, each character will need 6-bits output
           buffer
        */
        pDecodedData->Length = NumberOfChars * 6 / 8; 

        pDecodedData->Data = (uint8 *) CSSM_MALLOC_WRAPPER(MemoryFunctions->malloc_func,
                                                           AppHandle,
                                                           pDecodedData->Length);
        if (pDecodedData->Data == NULL)
        {
            CSSM_FREE_WRAPPER(MemoryFunctions->free_func,
                              AppHandle,
                              pDecodedData);
            return NULL;
        }
    }
    else
    {
        pDecodedData = (CSSM_DATA_PTR)local_malloc(sizeof(CSSM_DATA),
                                                   NULL);
        if(NULL != pDecodedData)
        {
            pDecodedData->Length = NumberOfChars * 6 / 8;

            pDecodedData->Data = (uint8*)local_malloc(pDecodedData->Length,
                                                      NULL);

            if (pDecodedData->Data == NULL)
            {
                local_free(pDecodedData,
                           NULL);
                return NULL;
            }
        }
        else
        {
            return NULL;
        }

    }

    j= 0; 
    i= 0;
    /* decode input data */
    while ((i < pEncodedData->Length) && (pEncodedData->Data[i] != '=')) {
        uint32 temp = 0;
        uint32 value = 0;
        uint32 k = 0;
        uint32 NumberOfBytesUsed = 0;
        uint32 BitsToShift = 0;

        /* decode 4 characters (each character will be decoded to
           a 6-bits group) and store the 24-bits in the right most bits of
           uint32 */
        while (k < 4 && i < pEncodedData->Length) {
            if ((pEncodedData->Data[i] != '\n') && (pEncodedData->Data[i] != '\r')) {
                value <<= 6;
                temp = Base64_DecodeChar(pEncodedData->Data[i]);
                value |= temp;
                k++;
            }
            i++;
            if (pEncodedData->Data[i] == '=')
                break;
        
        }
    
        
        /* if less than 4 characters were decoded, shift the value to the
           left up to the 2nd byte of uint32 and pad the rest bits with zero's */
        BitsToShift = 24 - k * 6 ;
        value <<= BitsToShift; 

        /* calculate the number of bytes in the decoded value that
           we need to copy to the output buffer */
        NumberOfBytesUsed = k * 6 / 8;

        /* copy decoded value to the output buffer starting
           from the 2nd bytes of decoded value */
        pos = 23;
        for (l=0; l<NumberOfBytesUsed; l++) {
            if(j >= pDecodedData->Length)  /* don't permit a write past end of allocated buffer */
                break;  /* see bug #435 */
            pDecodedData->Data[j] = Base64_GetBits(value, pos, 8);
            pos -= 8;
            j++;
        }
    
    }

    return pDecodedData;
}
