/*-----------------------------------------------------------------------
 * File: CALLOUTS.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/*
 * This file contains the code used for self-check and bilateral.
 */

#include "precomp.h"

#ifdef ISL_INCLUDED

#define MAX_CERTS 5

int EISL_RetrieveRootIssuerName(ISL_CALLOUT_DATA *Name)
{
Name; /*to remove level4 warning*/
return 0;
}
#if defined (_IBM)
#define SECTION "libincssm300"
#elif defined (VMS)
#define SECTION "cdsa$incssm300_shr"
#else
#define SECTION "incssm300"
#endif


/*---------------------------------------------------------------
 *
 *Name: EISL_RetrieveSelfCheckSectionName
 *
 *Description:
 *   sets the slefcheck section name in the manifest
 *   This name has to be set by all modules by defining a global string
 *
 *Parameters: 
 *   ISL_CALLOUT_DATA*
 *
 *Returns:
 * None
 *
 *----------------------------------------------------------------*/

void EISL_RetrieveSelfCheckSectionName(
    ISL_CALLOUT_DATA *Name)
{

    sint32 len = (uint32)strlen(SECTION);

    TRACE(cssm_trace_info, FNCODE_EISL_RetrieveRootIssuerName);

    if(len < Name->length)
    {
        Name->length = len;
        port_memcpy(Name->value,
                    SECTION,
                    Name->length+1);
    }
    else
    {
        Name->length = 0;
    }
}

/*---------------------------------------------------------------
 *
 *Name: EISL_RetrieveSelfCheckCredentials
 *
 *Description:
 *   Retrieves credential name for the  module
 *
 *Parameters: 
 *   ISL_CALLOUT_DATA*
 *
 *Returns:
 * None
 *
 *----------------------------------------------------------------*/
void EISL_RetrieveSelfCheckCredentials(
        ISL_CALLOUT_DATA *Name, ISL_CALLOUT_DATA *Path)
{
    CSSM_DATA Credential;
    CSSM_DATA ModulePath;

    if (mdsutil_GetModuleCredentialInfo (&cssm30_GUID,
                                 &Credential, 
                                 NULL, 
                                 &ModulePath) == CSSM_OK) {
        port_memcpy (Name->value, Credential.Data, Credential.Length);
        Name->length = Credential.Length;
        port_memcpy (Path->value, ModulePath.Data, ModulePath.Length);
        Path->length = ModulePath.Length;
        local_free (Credential.Data, NULL);
        local_free (ModulePath.Data, NULL);
    }
}

void EISL_RetrieveSelfCheckCredentialsSize(
                            uint32 *Size)
{
    CSSM_DATA Credential;
    CSSM_DATA ModulePath;

    if (mdsutil_GetModuleCredentialInfo (&cssm30_GUID,
                                 &Credential, 
                                 NULL, 
                                 &ModulePath) == CSSM_OK) {
        if (Size)
            *Size = Credential.Length;
        local_free (Credential.Data, NULL);
        local_free (ModulePath.Data, NULL);
    }
}

#endif
