/*-----------------------------------------------------------------------
 * File: MDS_UTIL_ATTR.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/*
 * This file contains the mds functions.
 */
#include "cssm.h"
#include "cssmtype.h"
#include "emmtype.h"
#include "mds_app_util.h"
#include "mds_util_intrn.h"
#include "mds_schema.h"
//#include <cssmMemTrack.h>


/*-----------------------------------------------------------------------------
                            Internal Functions
-----------------------------------------------------------------------------*/
CSSM_RETURN set_attrib_at_pos(CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes,
                              uint32 pos,
                              const CSSM_DB_ATTRIBUTE_INFO *AttributeInfo,
                              uint8 *Data,
                              uint32 Length);

/*---------------------------------------------------------------------------*/


CSSM_RETURN CSSMAPI 
object_schema_GetAttributes(cssm_MDS_OBJECT_RECORD_PTR Record,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_OBJECT_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_OBJECT_NUM_ATTRIBUTES;

    /* Guid */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId, 
                                      Record->guid.Data, 
                                      Record->guid.Length)) != CSSM_OK)
        return (retValue);

    /* Manifest */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_Manifest, 
                                      Record->Manifest.Data, 
                                      Record->Manifest.Length)) != CSSM_OK)
        return (retValue);

    /* ModuleName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_ModuleName, 
                                      Record->ModuleName.Data, 
                                      Record->ModuleName.Length)) != CSSM_OK)
        return (retValue);

    /* Filepath */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_Path, 
                                      Record->Path.Data, 
                                      Record->Path.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes,
                                      4,
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->ProductVersion.Data,
                                      Record->ProductVersion.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
cssm_schema_GetAttributes(cssm_MDS_CDSADIR_CSSM_PTR Record,
                          CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_CSSM_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_CSSM_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId, 
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* Version */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_CDSAVersion,
                                      Record->version.Data, 
                                      Record->version.Length)) != CSSM_OK)
        return (retValue);

    /* Vendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_Vendor, 
                                      Record->vendor.Data, 
                                      Record->vendor.Length)) != CSSM_OK)
        return (retValue);

    /* Desc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_Desc,
                                      Record->desc.Data, 
                                      Record->desc.Length)) != CSSM_OK)
        return (retValue);

    /* NativeServices */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_NativeServices,
                                      Record->nativeservices.Data, 
                                      Record->nativeservices.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
emm_schema_GetAttributes(cssm_MDS_CDSADIR_EMM_RECORD_PTR Record,
                         CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_EMM_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_EMM_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* Manifest */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_Manifest,
                                      Record->manifest.Data, 
                                      Record->manifest.Length)) != CSSM_OK)
        return (retValue);

    /* ModuleName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_ModuleName,
                                      Record->modulename.Data, 
                                      Record->modulename.Length)) != CSSM_OK)
        return (retValue);

    /* Filepath */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_Path,
                                      Record->path.Data, 
                                      Record->path.Length)) != CSSM_OK)
        return (retValue);

    /* CDSAVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_CDSAVersion,
                                      Record->CDSAVersion.Data, 
                                      Record->CDSAVersion.Length)) != CSSM_OK)
        return (retValue);

    /* EMMSpecVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_EMMSpecVersion,
                                      Record->EMMSpecVersion.Data, 
                                      Record->EMMSpecVersion.Length)) != CSSM_OK)
        return (retValue);

    /* Desc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_Desc,
                                      Record->desc.Data, 
                                      Record->desc.Length)) != CSSM_OK)
        return (retValue);

    /* PolicyStmt */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7,
                                      &s_MdsAttrInfo_PolicyStmt,
                                      Record->policystmt.Data, 
                                      Record->policystmt.Length)) != CSSM_OK)
        return (retValue);

    /* EmmVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_EmmVersion,
                                      Record->emmversion.Data, 
                                      Record->emmversion.Length)) != CSSM_OK)
        return (retValue);

    /* EmmVendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      9, 
                                      &s_MdsAttrInfo_EmmVendor,
                                      Record->emmvendor.Data, 
                                      Record->emmvendor.Length)) != CSSM_OK)
        return (retValue);

    /* EmmType */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      10, 
                                      &s_MdsAttrInfo_EmmType,
                                      Record->emmtype.Data, 
                                      Record->emmtype.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
common_schema_GetAttributes(cssm_MDS_CDSADIR_COMMON_PTR Record,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_COMMON_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_COMMON_NUM_ATTRIBUTES;

    /* Guid */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->guid.Data, 
                                      Record->guid.Length)) != CSSM_OK)
        return (retValue);

    /* Manifest */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_Manifest,
                                      Record->Manifest.Data, 
                                      Record->Manifest.Length)) != CSSM_OK)
        return (retValue);

    /* ModuleName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_ModuleName,
                                      Record->ModuleName.Data, 
                                      Record->ModuleName.Length)) != CSSM_OK)
        return (retValue);

    /* Path */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_Path,
                                      Record->Path.Data, 
                                      Record->Path.Length)) != CSSM_OK)
        return (retValue);

    /* CDSAVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_CDSAVersion,
                                      Record->Version.Data, 
                                      Record->Version.Length)) != CSSM_OK)
        return (retValue);

    /* Desc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_Desc,
                                      Record->Desc.Data, 
                                      Record->Desc.Length)) != CSSM_OK)
        return (retValue);

    /* Dynamic */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_DynamicFlag,
                                      Record->Dynamic.Data, 
                                      Record->Dynamic.Length)) != CSSM_OK)
        return (retValue);

    /* MultiThreadFlag */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_MultiThreadFlag,
                                      Record->MultiThreadFlag.Data, 
                                      Record->MultiThreadFlag.Length)) != CSSM_OK)
        return (retValue);

    /* ServiceMask */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_ServiceMaskFlag,
                                      Record->ServiceMask.Data, 
                                      Record->ServiceMask.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
emm_primary_schema_GetAttributes(cssm_MDS_CDSADIR_EMM_PRIMARY_PTR Record, 
                                 CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_EMM_PRIMARY_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_EMM_PRIMARY_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* ServiceType */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_ServiceType,
                                      Record->servicetype.Data, 
                                      Record->servicetype.Length)) != CSSM_OK)
        return (retValue);

    /* Manifest */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_Manifest,
                                      Record->manifest.Data, 
                                      Record->manifest.Length)) != CSSM_OK)
        return (retValue);

    /* ModuleName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ModuleName,
                                      Record->modulename.Data, 
                                      Record->modulename.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->addinversion.Data, 
                                      Record->addinversion.Length)) != CSSM_OK)
        return (retValue);

    /* Vendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_Vendor,
                                      Record->addinvendor.Data, 
                                      Record->addinvendor.Length)) != CSSM_OK)
        return (retValue);


    /* SampleTypes */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_SampleTypes,
                                      Record->sampletypes.Data, 
                                      Record->sampletypes.Length)) != CSSM_OK)
        return (retValue);

    /* AclSubjectTypes */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_AclSubjectTypes,
                                      Record->aclsubjecttypes.Data, 
                                      Record->aclsubjecttypes.Length)) != CSSM_OK)
        return (retValue);

    /* AuthTags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      9, 
                                      &s_MdsAttrInfo_AuthTags,
                                      Record->authtags.Data, 
                                      Record->authtags.Length)) != CSSM_OK)
        return (retValue);
    /* EMMSpecVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      10, 
                                      &s_MdsAttrInfo_EMMSpecVersion,
                                      Record->emmspecversion.Data, 
                                      Record->emmspecversion.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
csp_primary_schema_GetAttributes(cssm_MDS_CDSADIR_CSP_PRIMARY_PTR Record, 
                                 CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_CSP_PRIMARY_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_CSP_PRIMARY_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* Manifest */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_Manifest,
                                      Record->manifest.Data, 
                                      Record->manifest.Length)) != CSSM_OK)
        return (retValue);

    /* ModuleName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ModuleName,
                                      Record->modulename.Data, 
                                      Record->modulename.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->addinversion.Data, 
                                      Record->addinversion.Length)) != CSSM_OK)
        return (retValue);

    /* Vendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_Vendor,
                                      Record->addinvendor.Data, 
                                      Record->addinvendor.Length)) != CSSM_OK)
        return (retValue);

    /* CspType */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_CspType,
                                      Record->csptype.Data, 
                                      Record->csptype.Length)) != CSSM_OK)
        return (retValue);

    /* CspFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_CspFlags,
                                      Record->cspflags.Data, 
                                      Record->cspflags.Length)) != CSSM_OK)
        return (retValue);

    /* CspCustomFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_CspCustomFlags,
                                      Record->cspcustomflags.Data, 
                                      Record->cspcustomflags.Length)) != CSSM_OK)
        return (retValue);

    /* UseeTags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      9, 
                                      &s_MdsAttrInfo_UseeTags,
                                      Record->useetags.Data, 
                                      Record->useetags.Length)) != CSSM_OK)
        return (retValue);

    /* SampleTypes */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      10, 
                                      &s_MdsAttrInfo_SampleTypes,
                                      Record->sampletypes.Data, 
                                      Record->sampletypes.Length)) != CSSM_OK)
        return (retValue);

    /* AclSubjectTypes */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      11, 
                                      &s_MdsAttrInfo_AclSubjectTypes,
                                      Record->aclsubjecttypes.Data, 
                                      Record->aclsubjecttypes.Length)) != CSSM_OK)
        return (retValue);

    /* AuthTags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      12, 
                                      &s_MdsAttrInfo_AuthTags,
                                      Record->authtags.Data, 
                                      Record->authtags.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
csp_capability_schema_GetAttributes(cssm_MDS_CDSADIR_CSP_CAPABILITY_PTR Record,
                                    CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_CSP_CAPABILITY_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_CSP_CAPABILITY_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* UsseTag */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_UseeTag,
                                      Record->useetag.Data, 
                                      Record->useetag.Length)) != CSSM_OK)
        return (retValue);

    /* ContextType */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ContextType,
                                      Record->contexttype.Data, 
                                      Record->contexttype.Length)) != CSSM_OK)
        return (retValue);

    /* AlgType */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_AlgType,
                                      Record->algtype.Data, 
                                      Record->algtype.Length)) != CSSM_OK)
        return (retValue);

    /* GroupId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_GroupId,
                                      Record->groupid.Data, 
                                      Record->groupid.Length)) != CSSM_OK)
        return (retValue);

    /* AttributeType */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_AttributeType,
                                      Record->attributetype.Data, 
                                      Record->attributetype.Length)) != CSSM_OK)
        return (retValue);

    /* AttributeValue */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_AttributeValue,
                                      Record->attributevalue.Data, 
                                      Record->attributevalue.Length)) != CSSM_OK)
        return (retValue);

    /* Description */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_Description,
                                      Record->description.Data, 
                                      Record->description.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
csp_encapsulated_schema_GetAttributes(cssm_MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_PTR Record,
                                      CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES;

    /* Guid */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* ProductDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_ProductDesc,
                                      Record->productdesc.Data, 
                                      Record->productdesc.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ProductVendor,
                                      Record->productvendor.Data, 
                                      Record->productvendor.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->productversion.Data, 
                                      Record->productversion.Length)) != CSSM_OK)
        return (retValue);

    /* ProductFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_ProductFlags,
                                      Record->productflags.Data, 
                                      Record->productflags.Length)) != CSSM_OK)
        return (retValue);

    /* CustomFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_CustomFlags,
                                      Record->customflags.Data, 
                                      Record->customflags.Length)) != CSSM_OK)
        return (retValue);

    /* StandardDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_StandardDesc,
                                      Record->standarddesc.Data, 
                                      Record->standarddesc.Length)) != CSSM_OK)
        return (retValue);

    /* StandardVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_StandardVersion,
                                      Record->standardversion.Data, 
                                      Record->standardversion.Length)) != CSSM_OK)
        return (retValue);

    /* ReaderDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      9, 
                                      &s_MdsAttrInfo_ReaderDesc,
                                      Record->readerdescription.Data, 
                                      Record->readerdescription.Length)) != CSSM_OK)
        return (retValue);

    /* ReaderVendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      10, 
                                      &s_MdsAttrInfo_ReaderVendor,
                                      Record->readervendor.Data, 
                                      Record->readervendor.Length)) != CSSM_OK)
        return (retValue);

    /* ReaderVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      11, 
                                      &s_MdsAttrInfo_ReaderVersion,
                                      Record->readerversion.Data, 
                                      Record->readerversion.Length)) != CSSM_OK)
        return (retValue);

    /* ReaderFirmwareVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      12, 
                                      &s_MdsAttrInfo_ReaderFirmwareVersion,
                                      Record->readerfirmwareversion.Data, 
                                      Record->readerfirmwareversion.Length)) != CSSM_OK)
        return (retValue);

    /* ReaderFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      13, 
                                      &s_MdsAttrInfo_ReaderFlags,
                                      Record->readerflags.Data, 
                                      Record->readerflags.Length)) != CSSM_OK)
        return (retValue);

    /* ReaderCustomFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      14, 
                                      &s_MdsAttrInfo_ReaderCustomFlags,
                                      Record->readercustomflags.Data, 
                                      Record->readercustomflags.Length)) != CSSM_OK)
        return (retValue);

    /* ReaderSerialNumber */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      15, 
                                      &s_MdsAttrInfo_ReaderSerialNumber,
                                      Record->readerserialnumber.Data, 
                                      Record->readerserialnumber.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
csp_SC_schema_GetAttributes(cssm_MDS_CDSADIR_CSP_SC_INFO_PTR Record, 
                            CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_CSP_SC_INFO_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_CSP_SC_INFO_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* SCDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_SCDesc,
                                      Record->scdescription.Data, 
                                      Record->scdescription.Length)) != CSSM_OK)
        return (retValue);

    /* SCVendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_SCVendor,
                                      Record->scvendor.Data, 
                                      Record->scvendor.Length)) != CSSM_OK)
        return (retValue);

    /* SCVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_SCVersion,
                                      Record->scversion.Data, 
                                      Record->scversion.Length)) != CSSM_OK)
        return (retValue);

    /* SCFirmwareVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_SCFirmwareVersion,
                                      Record->scfirmwareversion.Data, 
                                      Record->scfirmwareversion.Length)) != CSSM_OK)
        return (retValue);

    /* SCFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_SCFlags,
                                      Record->scflags.Data, 
                                      Record->scflags.Length)) != CSSM_OK)
        return (retValue);

    /* SCCustomFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_SCCustomFlags,
                                      Record->sccustomflags.Data, 
                                      Record->sccustomflags.Length)) != CSSM_OK)
        return (retValue);

    /* SCSerialNumber */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_SCSerialNumber,
                                      Record->scserialnumber.Data, 
                                      Record->scserialnumber.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
dl_primary_schema_GetAttributes(cssm_MDS_CDSADIR_DL_PRIMARY_PTR Record, 
                                CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_DL_PRIMARY_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_DL_PRIMARY_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* Manifest */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_Manifest,
                                      Record->manifest.Data, 
                                      Record->manifest.Length)) != CSSM_OK)
        return (retValue);

    /* ModuleName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ModuleName,
                                      Record->modulename.Data, 
                                      Record->modulename.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->addinversion.Data, 
                                      Record->addinversion.Length)) != CSSM_OK)
        return (retValue);

    /* Vendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_Vendor,
                                      Record->addinvendor.Data, 
                                      Record->addinvendor.Length)) != CSSM_OK)
        return (retValue);

    /* DlType */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_DlType,
                                      Record->dltype.Data, 
                                      Record->dltype.Length)) != CSSM_OK)
        return (retValue);

    /* QueryLimitsFlag */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_QueryLimitsFlag,
                                      Record->querylimitsflag.Data, 
                                      Record->querylimitsflag.Length)) != CSSM_OK)
        return (retValue);

    /* SampleTypes */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_SampleTypes,
                                      Record->sampletypes.Data, 
                                      Record->sampletypes.Length)) != CSSM_OK)
        return (retValue);

    /* AclSubjectTypes */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      9, 
                                      &s_MdsAttrInfo_AclSubjectTypes,
                                      Record->aclsubjecttypes.Data, 
                                      Record->aclsubjecttypes.Length)) != CSSM_OK)
        return (retValue);

    /* AuthTags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      10, 
                                      &s_MdsAttrInfo_AuthTags,
                                      Record->authtags.Data, 
                                      Record->authtags.Length)) != CSSM_OK)
        return (retValue);

    /* ConjuntiveOps */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      11, 
                                      &s_MdsAttrInfo_ConjunctiveOps,
                                      Record->conjunctiveops.Data, 
                                      Record->conjunctiveops.Length)) != CSSM_OK)
        return (retValue);

    /* RelationalOps */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      12, 
                                      &s_MdsAttrInfo_RelationalOps,
                                      Record->relationalops.Data, 
                                      Record->relationalops.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
dl_encapsulated_schema_GetAttributes(cssm_MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_PTR Record,
                                     CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* ProductDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_ProductDesc,
                                      Record->productdesc.Data, 
                                      Record->productdesc.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->productversion.Data, 
                                      Record->productversion.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ProductVendor,
                                      Record->productvendor.Data, 
                                      Record->productvendor.Length)) != CSSM_OK)
        return (retValue);

    /* ProductFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_ProductFlags,
                                      Record->productflags.Data, 
                                      Record->productflags.Length)) != CSSM_OK)
        return (retValue);

    /* StandardDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_StandardDesc,
                                      Record->standarddesc.Data, 
                                      Record->standarddesc.Length)) != CSSM_OK)
        return (retValue);

    /* StandardVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_StandardVersion,
                                      Record->standardversion.Data, 
                                      Record->standardversion.Length)) != CSSM_OK)
        return (retValue);

    /* Protocol */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_Protocol,
                                      Record->protocol.Data, 
                                      Record->protocol.Length)) != CSSM_OK)
        return (retValue);

    /* RetrievalMode */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      9, 
                                      &s_MdsAttrInfo_RetrievalMode,
                                      Record->retrievalmode.Data, 
                                      Record->retrievalmode.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
cl_primary_schema_GetAttributes(cssm_MDS_CDSADIR_CL_PRIMARY_PTR Record, 
                                CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_CL_PRIMARY_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_CL_PRIMARY_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* Manifest */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_Manifest,
                                      Record->manifest.Data, 
                                      Record->manifest.Length)) != CSSM_OK)
        return (retValue);

    /* ModuleName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ModuleName,
                                      Record->modulename.Data, 
                                      Record->modulename.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->addinversion.Data, 
                                      Record->addinversion.Length)) != CSSM_OK)
        return (retValue);

    /* Vendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_Vendor,
                                      Record->addinvendor.Data, 
                                      Record->addinvendor.Length)) != CSSM_OK)
        return (retValue);

    /* CertTypeFormat */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_CertTypeFormat,
                                      Record->certtypeformat.Data, 
                                      Record->certtypeformat.Length)) != CSSM_OK)
        return (retValue);

    /* CrlTypeFormat */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_CrlTypeFormat,
                                      Record->crltypeformat.Data, 
                                      Record->crltypeformat.Length)) != CSSM_OK)
        return (retValue);

    /* CertFieldNames */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_CertFieldNames,
                                      Record->certfieldnames.Data, 
                                      Record->certfieldnames.Length)) != CSSM_OK)
        return (retValue);

    /* BundleTypeFormat */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      9, 
                                      &s_MdsAttrInfo_BundleTypeFormat,
                                      Record->bundletypeformat.Data, 
                                      Record->bundletypeformat.Length)) != CSSM_OK)
        return (retValue);

    /* XlationTypeFormat */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      10, 
                                      &s_MdsAttrInfo_XlationTypeFormat,
                                      Record->xlationtypeformat.Data, 
                                      Record->xlationtypeformat.Length)) != CSSM_OK)
        return (retValue);

    /* DefaultTemplateType */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      11, 
                                      &s_MdsAttrInfo_DefaultTemplateType,
                                      Record->defaulttemplatetype.Data, 
                                      Record->defaulttemplatetype.Length)) != CSSM_OK)
        return (retValue);

    /* TemplateFieldNames */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      12, 
                                      &s_MdsAttrInfo_TemplateFieldNames,
                                      Record->templatefieldnames.Data, 
                                      Record->templatefieldnames.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
cl_encapsulated_schema_GetAttributes(cssm_MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_PTR Record,
                                     CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* ProductDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_ProductDesc,
                                      Record->productdesc.Data, 
                                      Record->productdesc.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ProductVendor,
                                      Record->productvendor.Data, 
                                      Record->productvendor.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->productversion.Data, 
                                      Record->productversion.Length)) != CSSM_OK)
        return (retValue);

    /* ProductFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_ProductFlags,
                                      Record->productflags.Data, 
                                      Record->productflags.Length)) != CSSM_OK)
        return (retValue);

    /* StandardDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_StandardDesc,
                                      Record->standarddesc.Data, 
                                      Record->standarddesc.Length)) != CSSM_OK)
        return (retValue);

    /* StandardVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_StandardVersion,
                                      Record->standardversion.Data, 
                                      Record->standardversion.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
tp_primary_schema_GetAttributes(cssm_MDS_CDSADIR_TP_PRIMARY_PTR Record,
                                CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_TP_PRIMARY_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_TP_PRIMARY_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* Manifest */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_Manifest,
                                      Record->manifest.Data, 
                                      Record->manifest.Length)) != CSSM_OK)
        return (retValue);

    /* ModuleName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ModuleName,
                                      Record->modulename.Data, 
                                      Record->modulename.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->addinversion.Data, 
                                      Record->addinversion.Length)) != CSSM_OK)
        return (retValue);

    /* Vendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_Vendor,
                                      Record->addinvendor.Data, 
                                      Record->addinvendor.Length)) != CSSM_OK)
        return (retValue);

    /* CertTypeFormat */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_CertTypeFormat,
                                      Record->certtypeformat.Data, 
                                      Record->certtypeformat.Length)) != CSSM_OK)
        return (retValue);

    /* SampleTypes */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_SampleTypes,
                                      Record->sampletypes.Data, 
                                      Record->sampletypes.Length)) != CSSM_OK)
        return (retValue);

    /* AclSubjectTypes */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_AclSubjectTypes,
                                      Record->aclsubjecttypes.Data, 
                                      Record->aclsubjecttypes.Length)) != CSSM_OK)
        return (retValue);

    /* AuthTags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      9, 
                                      &s_MdsAttrInfo_AuthTags,
                                      Record->authtags.Data, 
                                      Record->authtags.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
tp_policy_schema_GetAttributes(cssm_MDS_CDSADIR_TP_OIDS_PTR Record,
                               CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_TP_OIDS_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_TP_OIDS_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* OID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_OID,
                                      Record->oid.Data, 
                                      Record->oid.Length)) != CSSM_OK)
        return (retValue);

    /* Value */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_Value,
                                      Record->value.Data, 
                                      Record->value.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI 
tp_encapsulated_schema_GetAttributes(cssm_MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_PTR Record,
                                     CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* ProductDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_ProductDesc,
                                      Record->productdesc.Data, 
                                      Record->productdesc.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ProductVendor,
                                      Record->productvendor.Data, 
                                      Record->productvendor.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->productversion.Data, 
                                      Record->productversion.Length)) != CSSM_OK)
        return (retValue);

    /* ProductFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_ProductFlags,
                                      Record->productflags.Data, 
                                      Record->productflags.Length)) != CSSM_OK)
        return (retValue);

    /* AuthorityRequestType */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      6, 
                                      &s_MdsAttrInfo_AuthorityRequestType,
                                      Record->authorityrequesttype.Data, 
                                      Record->authorityrequesttype.Length)) != CSSM_OK)
        return (retValue);

    /* StandardDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      7, 
                                      &s_MdsAttrInfo_StandardDesc,
                                      Record->standarddesc.Data, 
                                      Record->standarddesc.Length)) != CSSM_OK)
        return (retValue);

    /* StandardVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      8, 
                                      &s_MdsAttrInfo_StandardVersion,
                                      Record->standardversion.Data, 
                                      Record->standardversion.Length)) != CSSM_OK)
        return (retValue);

    /* ProtocolDesc */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      9, 
                                      &s_MdsAttrInfo_ProtocolDesc,
                                      Record->protocoldesc.Data, 
                                      Record->protocoldesc.Length)) != CSSM_OK)
        return (retValue);

    /* ProtocolFlags */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      10, 
                                      &s_MdsAttrInfo_ProtocolFlags,
                                      Record->protocolflags.Data, 
                                      Record->protocolflags.Length)) != CSSM_OK)
        return (retValue);

    /* CertClassName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      11, 
                                      &s_MdsAttrInfo_CertClassName,
                                      Record->certclassname.Data, 
                                      Record->certclassname.Length)) != CSSM_OK)
        return (retValue);

    /* RootCertificate */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      12, 
                                      &s_MdsAttrInfo_RootCertificate,
                                      Record->rootcertificate.Data, 
                                      Record->rootcertificate.Length)) != CSSM_OK)
        return (retValue);

    /* RootCertTypeFormat */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      13, 
                                      &s_MdsAttrInfo_RootCertTypeFormat,
                                      Record->rootcerttypeformat.Data, 
                                      Record->rootcerttypeformat.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}

CSSM_RETURN CSSMAPI
ac_primary_schema_GetAttributes(cssm_MDS_CDSADIR_AC_PRIMARY_PTR Record,
                                CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes)
{
    CSSM_RETURN retValue = CSSM_OK;

    Attributes->DataRecordType = MDS_CDSADIR_AC_PRIMARY_RECORDTYPE;
    Attributes->SemanticInformation = 0;
    Attributes->NumberOfAttributes = MDS_CDSADIR_AC_PRIMARY_NUM_ATTRIBUTES;

    /* ModuleId */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      0, 
                                      &s_MdsAttrInfo_ModuleId,
                                      Record->moduleid.Data, 
                                      Record->moduleid.Length)) != CSSM_OK)
        return (retValue);

    /* SSID */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      1, 
                                      &s_MdsAttrInfo_SSID,
                                      Record->ssid.Data, 
                                      Record->ssid.Length)) != CSSM_OK)
        return (retValue);

    /* Manifest */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      2, 
                                      &s_MdsAttrInfo_Manifest,
                                      Record->manifest.Data, 
                                      Record->manifest.Length)) != CSSM_OK)
        return (retValue);

    /* ModuleName */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      3, 
                                      &s_MdsAttrInfo_ModuleName,
                                      Record->modulename.Data, 
                                      Record->modulename.Length)) != CSSM_OK)
        return (retValue);

    /* ProductVersion */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      4, 
                                      &s_MdsAttrInfo_ProductVersion,
                                      Record->addinversion.Data, 
                                      Record->addinversion.Length)) != CSSM_OK)
        return (retValue);

    /* Vendor */
    if ((retValue = set_attrib_at_pos(Attributes, 
                                      5, 
                                      &s_MdsAttrInfo_Vendor,
                                      Record->addinvendor.Data, 
                                      Record->addinvendor.Length)) != CSSM_OK)
        return (retValue);

    return (retValue);
}


/*
 *  SET ATTRIBute AT POSition in array of attributes
 */
CSSM_RETURN set_attrib_at_pos(CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes,
                              uint32 pos,
                              const CSSM_DB_ATTRIBUTE_INFO *AttributeInfo,
                              uint8 *Data,
                              uint32 Length)
{   
    CSSM_DB_ATTRIBUTE_DATA_PTR attrib = &Attributes->AttributeData[pos];

    attrib->Info = *AttributeInfo;
    attrib->NumberOfValues = 1;

    if ( (attrib->Value != NULL) && (!IsBadWritePtr( attrib->Value, sizeof(CSSM_DATA))) )
    {
        attrib->Value->Data = Data;
        attrib->Value->Length = Length;
    }

    return (CSSM_OK);
}

/* 
 *  CLEAN ATTRIBUTES (upto given position) 
 */
void clean_attributes(CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes,
                      uint32 pos)
{
    uint32 index = 0;
    CSSM_DB_ATTRIBUTE_DATA_PTR attrib = &Attributes->AttributeData[pos];

    for (index = 0; index < pos; index++)
    {
        attrib = &Attributes->AttributeData[index];
        if (attrib->Value != NULL)
        {
            MDSUTIL_free(attrib->Value, NULL);
            attrib->Value = NULL;
        }
    }
}

/* 
 *  ALLOCATE VALUE in an array of attributes for given NUMber of ATTRIBUTES 
 *  if failure this function cleans-up allocations
 */
CSSM_RETURN allocate_value(CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes, uint32 numAttributes)
{
    uint32 index = 0;

    for(index = 0; index < numAttributes; index++)
    {
        Attributes->AttributeData[index].NumberOfValues = 1;

        Attributes->AttributeData[index].Value = (CSSM_DATA *) MDSUTIL_calloc (1, Attributes->AttributeData[index].NumberOfValues * sizeof (CSSM_DATA), NULL);
        if (Attributes->AttributeData[index].Value == NULL)
        {
            /* clean up; clean all the prior allocations */
            clean_attributes(Attributes, Attributes->NumberOfAttributes);

            /* allocation error */
            return (CSSM_ERRCODE_MEMORY_ERROR);
        }
    }

    return (CSSM_OK);
}
