/*-----------------------------------------------------------------------
 * File: CSSMCOREF.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Function declarations for the Cryptographic Service Provider module. The
 * CSSM_DEFINE_FUNC macro must be defined by the file that includes this header.
 */

/* Check to make sure that all macros were defined */
#ifndef CSSM_DEFINE_FUNCTION
#error "The CSSM_DEFINE_FUNCTION(name,params) macro must be defined!"
#endif

/*************************************************************************/
/*** Function declarations ***********************************************/
/*************************************************************************/

CSSM_DEFINE_FUNCTION( Init, (
						const CSSM_VERSION *Version,
						CSSM_PRIVILEGE_SCOPE Scope, 
						const CSSM_GUID *CallerGuid, 
						CSSM_KEY_HIERARCHY KeyHierarchy,
						CSSM_PVC_MODE *PvcPolicy, 
						const void *Reserved ) );

CSSM_DEFINE_FUNCTION( Terminate, (
						void ) );

CSSM_DEFINE_FUNCTION( ModuleLoad, (
						const CSSM_GUID *ModuleGuid,
						CSSM_KEY_HIERARCHY KeyHierarchy,
						CSSM_API_ModuleEventHandler AppNotifyCallback,
						void *AppNotifyCallbackCtx ) );

CSSM_DEFINE_FUNCTION( ModuleUnload, (
						const CSSM_GUID *ModuleGuid,
						CSSM_API_ModuleEventHandler AppNotifyCallback,
						void *AppNotifyCallbackCtx ) );

CSSM_DEFINE_FUNCTION( Introduce, ( 
						const CSSM_GUID *ModuleID,
						CSSM_KEY_HIERARCHY KeyHierarchy ) );

CSSM_DEFINE_FUNCTION( Unintroduce, (
						const CSSM_GUID *ModuleID ) );


CSSM_DEFINE_FUNCTION( ModuleAttach, (
						const CSSM_GUID *ModuleGuid,
						const CSSM_VERSION *Version,
						const CSSM_API_MEMORY_FUNCS *MemoryFuncs,
						uint32 SubserviceID,
						CSSM_SERVICE_TYPE SubServiceType,
						CSSM_ATTACH_FLAGS AttachFlags,
						CSSM_KEY_HIERARCHY KeyHierarchy,
						CSSM_FUNC_NAME_ADDR *FunctionTable,
						uint32 NumFunctionTable,
						const void *Reserved,
						CSSM_MODULE_HANDLE_PTR NewModuleHandle ) );

CSSM_DEFINE_FUNCTION( ModuleDetach, (
						CSSM_MODULE_HANDLE ModuleHandle ) );

CSSM_DEFINE_FUNCTION( SetPrivilege, ( 
						CSSM_PRIVILEGE Privilege ) );

CSSM_DEFINE_FUNCTION( GetPrivilege, ( 
						CSSM_PRIVILEGE *Privilege ) );

CSSM_DEFINE_FUNCTION( GetModuleGUIDFromHandle, (
						CSSM_MODULE_HANDLE ModuleHandle,
						CSSM_GUID_PTR ModuleGUID ) );

CSSM_DEFINE_FUNCTION( GetSubserviceUIDFromHandle, (
						CSSM_MODULE_HANDLE ModuleHandle,
						CSSM_SUBSERVICE_UID_PTR SubserviceUID ) );

CSSM_DEFINE_FUNCTION( ListAttachedModuleManagers, (
						uint32 *NumberOfModuleManagers,
						CSSM_GUID_PTR ModuleManagerGuids ) );

CSSM_DEFINE_FUNCTION( GetAPIMemoryFunctions, (
						CSSM_MODULE_HANDLE AddInHandle,
						CSSM_API_MEMORY_FUNCS_PTR AppMemoryFuncs ) );
