/*-----------------------------------------------------------------------
 * File: CSSMTPF.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Function declarations for the Cryptographic Service Provider module. The
 * CSSM_DEFINE_FUNC, __CSSM_PASTE2 and __CSSM_PASTE3 macros must be defined
 * by the file that includes this header.
 */

/* Check to make sure that all macros were defined */
#ifndef CSSM_DEFINE_FUNCTION
#error "The CSSM_DEFINE_FUNCTION(name,params) macro must be defined!"
#endif

/*************************************************************************/
/*** Function declarations ***********************************************/
/*************************************************************************/

CSSM_DEFINE_FUNCTION( SubmitCredRequest, (
						CSSM_TP_HANDLE TPHandle,
						const CSSM_TP_AUTHORITY_ID *PreferredAuthority,
						CSSM_TP_AUTHORITY_REQUEST_TYPE RequestType, 
						const CSSM_TP_REQUEST_SET *RequestInput,
						const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthContext,
						sint32 *EstimatedTime,
						CSSM_DATA_PTR ReferenceIdentifier ) ); 


CSSM_DEFINE_FUNCTION( RetrieveCredResult, (
						CSSM_TP_HANDLE TPHandle,
						const CSSM_DATA *ReferenceIdentifier, 
						const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthCredentials,
						sint32 *EstimatedTime,
						CSSM_BOOL *ConfirmationRequired,
						CSSM_TP_RESULT_SET_PTR *RetrieveOutput ) );


CSSM_DEFINE_FUNCTION( ConfirmCredResult, (
						CSSM_TP_HANDLE TPHandle, 
						const CSSM_DATA *ReferenceIdentifier,
						const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthCredentials,
						const CSSM_TP_CONFIRM_RESPONSE *Responses, 
						const CSSM_TP_AUTHORITY_ID *PreferredAuthority ) );


CSSM_DEFINE_FUNCTION( ReceiveConfirmation, (
						CSSM_TP_HANDLE TPHandle, 
						const CSSM_DATA *ReferenceIdentifier,
						CSSM_TP_CONFIRM_RESPONSE_PTR *Responses,
						sint32 *ElapsedTime ) );


CSSM_DEFINE_FUNCTION( CertReclaimKey, (
						CSSM_TP_HANDLE TPHandle, 
						const CSSM_CERTGROUP *CertGroup,
						uint32 CertIndex, 
						CSSM_LONG_HANDLE KeyCacheHandle,
						CSSM_CSP_HANDLE CSPHandle,
						const CSSM_RESOURCE_CONTROL_CONTEXT *CredAndAclEntry ) );


CSSM_DEFINE_FUNCTION( CertReclaimAbort, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_LONG_HANDLE KeyCacheHandle ) );


CSSM_DEFINE_FUNCTION( FormRequest, (
						CSSM_TP_HANDLE TPHandle,
						const CSSM_TP_AUTHORITY_ID *PreferredAuthority,
						CSSM_TP_FORM_TYPE FormType,
						CSSM_DATA_PTR BlankForm ) );


CSSM_DEFINE_FUNCTION( FormSubmit, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_TP_FORM_TYPE FormType,
						const CSSM_DATA *Form,
						const CSSM_TP_AUTHORITY_ID *ClearanceAuthority,
						const CSSM_TP_AUTHORITY_ID *RepresentedAuthority,
						CSSM_ACCESS_CREDENTIALS_PTR Credentials ) );


CSSM_DEFINE_FUNCTION( CertGroupVerify, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						CSSM_CSP_HANDLE CSPHandle,
						const CSSM_CERTGROUP *CertGroupToBeVerified,
						const CSSM_TP_VERIFY_CONTEXT *VerifyContext,
						CSSM_TP_VERIFY_CONTEXT_RESULT_PTR VerifyContextResult ) );


CSSM_DEFINE_FUNCTION( CertCreateTemplate, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						uint32 NumberOfFields,
						const CSSM_FIELD *CertFields,
						CSSM_DATA_PTR CertTemplate ) );


CSSM_DEFINE_FUNCTION( CertGetAllTemplateFields, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						const CSSM_DATA *CertTemplate,
						uint32 *NumberOfFields,
						CSSM_FIELD_PTR *CertFields ) );


CSSM_DEFINE_FUNCTION( CertSign, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						CSSM_CC_HANDLE CCHandle,
						const CSSM_DATA *CertTemplateToBeSigned,
						const CSSM_CERTGROUP *SignerCertGroup,
						const CSSM_TP_VERIFY_CONTEXT *SignerVerifyContext,
						CSSM_TP_VERIFY_CONTEXT_RESULT_PTR SignerVerifyResult,
						CSSM_DATA_PTR SignedCert ) ); 


CSSM_DEFINE_FUNCTION( CrlVerify, ( 
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						CSSM_CSP_HANDLE CSPHandle,
						const CSSM_ENCODED_CRL *CrlToBeVerified,
						const CSSM_CERTGROUP *SignerCertGroup,
						const CSSM_TP_VERIFY_CONTEXT *VerifyContext,
						CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult ) );


CSSM_DEFINE_FUNCTION( CrlCreateTemplate, ( 
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						uint32 NumberOfFields,
						const CSSM_FIELD *CrlFields,
						CSSM_DATA_PTR NewCrlTemplate ) );


CSSM_DEFINE_FUNCTION( CertRevoke, ( 
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						CSSM_CSP_HANDLE CSPHandle,
						const CSSM_DATA *OldCrlTemplate,
						const CSSM_CERTGROUP *CertGroupToBeRevoked,
						const CSSM_CERTGROUP *RevokerCertGroup,
						const CSSM_TP_VERIFY_CONTEXT *RevokerVerifyContext,
						CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult,
						CSSM_TP_CERTCHANGE_REASON Reason,
						CSSM_DATA_PTR NewCrlTemplate ) );


CSSM_DEFINE_FUNCTION( CertRemoveFromCrlTemplate, ( 
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						CSSM_CSP_HANDLE CSPHandle,
						const CSSM_DATA *OldCrlTemplate, 
						const CSSM_CERTGROUP *CertGroupToBeRemoved,
						const CSSM_CERTGROUP *RevokerCertGroup,
						const CSSM_TP_VERIFY_CONTEXT *RevokerVerifyContext,
						CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult,
						CSSM_DATA_PTR NewCrlTemplate ) );


CSSM_DEFINE_FUNCTION( CrlSign, ( 
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						CSSM_CC_HANDLE CCHandle,
						const CSSM_ENCODED_CRL *CrlToBeSigned,
						const CSSM_CERTGROUP *SignerCertGroup,
						const CSSM_TP_VERIFY_CONTEXT *SignerVerifyContext,
						CSSM_TP_VERIFY_CONTEXT_RESULT_PTR SignerVerifyResult,
						CSSM_DATA_PTR SignedCrl ) ); 
                                 

CSSM_DEFINE_FUNCTION( ApplyCrlToDb, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						CSSM_CSP_HANDLE CSPHandle,
						const CSSM_ENCODED_CRL *CrlToBeApplied,
						const CSSM_CERTGROUP *SignerCertGroup,
						const CSSM_TP_VERIFY_CONTEXT *ApplyCrlVerifyContext,
						CSSM_TP_VERIFY_CONTEXT_RESULT_PTR ApplyCrlVerifyResult ) );


/* Cert Group Functions */
/* Returns the constructed cert group */
CSSM_DEFINE_FUNCTION( CertGroupConstruct, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						CSSM_CSP_HANDLE CSPHandle,
						const CSSM_DL_DB_LIST *DBList,
						const void *ConstructParams,
						const CSSM_CERTGROUP *CertGroupFrag,
						CSSM_CERTGROUP_PTR *CertGroup ) );


/* Returns the pruned cert group */
CSSM_DEFINE_FUNCTION( CertGroupPrune, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle,
						const CSSM_DL_DB_LIST *DBList,
						const CSSM_CERTGROUP *OrderedCertGroup,
						CSSM_CERTGROUP_PTR *PrunedCertGroup ) );


CSSM_DEFINE_FUNCTION( CertGroupToTupleGroup, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle, 
						const CSSM_CERTGROUP *CertGroup,
						CSSM_TUPLEGROUP_PTR *TupleGroup ) );


CSSM_DEFINE_FUNCTION( TupleGroupToCertGroup, (
						CSSM_TP_HANDLE TPHandle,
						CSSM_CL_HANDLE CLHandle, 
						const CSSM_TUPLEGROUP *TupleGroup, 
						CSSM_CERTGROUP_PTR *CertTemplates ) );


/*
 * Extensibility Functions
 */
CSSM_DEFINE_FUNCTION( PassThrough, (
						CSSM_TP_HANDLE TPHandle, 
						CSSM_CL_HANDLE CLHandle, 
						CSSM_CC_HANDLE CCHandle, 
						const CSSM_DL_DB_LIST *DBList,
						uint32 PassThroughId,
						const void *InputParams,
						void **OutputParams ) );
