/*-----------------------------------------------------------------------
 * File: MDS_SHIMUTIL.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "cssmtype.h"
#include "cssmMemTrack.h"
#include "cssmport.h"
#include "mds.h"
#include "cssmerr.h"
#include "mds_shim.h"

/* Memory Funcs  */

void* CSSMAPI
mds_local_malloc(uint32 size,
                 void *ref
#ifdef CSSM_MEMTRACK_ON
                 , const char *szFilename,
                 uint32 u32LineNumber
#endif
                 )
{
    ref;
#if defined(_DEBUG) && defined(CSSM_MEMTRACK_ON) && defined(WIN32)
        return _malloc_dbg( size, _NORMAL_BLOCK, szFilename, u32LineNumber );
#else
        return malloc(size);
#endif
}

void CSSMAPI
mds_local_free(void *block,
               void *ref
#ifdef CSSM_MEMTRACK_ON
               , const char *szFilename,
               uint32 u32LineNumber
#endif
               )
{
    ref;
#if defined(_DEBUG) && defined(CSSM_MEMTRACK_ON) && defined(WIN32)
    _free_dbg( block, _NORMAL_BLOCK );
#else
    free(block);
#endif
}

void* CSSMAPI
mds_local_realloc(void *block,
                  uint32 size,
                  void *ref
#ifdef CSSM_MEMTRACK_ON
                  , const char *szFilename,
                  uint32 u32LineNumber
#endif
                  )
{
    ref;
#if defined(_DEBUG) && defined(CSSM_MEMTRACK_ON) && defined(WIN32)
    return _realloc_dbg( block, size, _NORMAL_BLOCK, szFilename, u32LineNumber );
#else
    return realloc(block, size);
#endif
}

void* CSSMAPI
mds_local_calloc(uint32 count,
                 uint32 size,
                 void *ref
#ifdef CSSM_MEMTRACK_ON
                 , const char *szFilename,
                 uint32 u32LineNumber
#endif
                 )
{
    ref;
#if defined(_DEBUG) && defined(CSSM_MEMTRACK_ON) && defined(WIN32)
    return _calloc_dbg( count, size, _NORMAL_BLOCK, szFilename, u32LineNumber );
#else
    return calloc(count, size );
#endif
}

CSSM_RETURN
mds_LoadFile (const char *FileName, CSSM_DATA_PTR FileData)
{
    FILE *fi;
    struct _stat st;

    if (NULL == FileData)
        return CSSMERR_CSSM_INTERNAL_ERROR;

    if (NULL == FileName) {
        FileData->Data = NULL;
        FileData->Length = 0;
        return CSSM_OK;
    }

    /* 
     * open file for reading 
     */  
    if ((fi = fopen(FileName, "rb")) == NULL)
        return CSSMERR_CSSM_INTERNAL_ERROR;

    /* 
     * stat the file and find out how much memory we need 
     */
    if ((_stat(FileName, &st)) != 0)
        return CSSMERR_CSSM_INTERNAL_ERROR;

    FileData->Length = st.st_size;
    
    /* 
     * allocate enough memory to hold the image 
     */
    FileData->Data = (unsigned char *)malloc(FileData->Length);
    if (NULL == FileData->Data)
        return CSSMERR_CSSM_INTERNAL_ERROR;

    /* 
     * Read the file 
     */
    if ((fread((void*)FileData->Data, 1, FileData->Length, fi)) != FileData->Length) {
        free(FileData->Data);
        fclose(fi);
        return CSSMERR_CSSM_INTERNAL_ERROR;
    }

    fclose(fi);

    return CSSM_OK;
}
