/*-----------------------------------------------------------------------
 * File: VERIFY.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Module name: verify.c
  Signature verification function of DSS
*/

#include "eicl.h"
#include "dss.h"
#include "dsskg.h"
#include "eiclproc.h"
#include "misc.h"

#ifdef BIG_ENDIAN
#undef bswap
#define bswap(y) (y)
#endif


/*  Verification of DSS signatures
	The function returns
	0		for verified signatures
	-1		for rejected signatures
*/
int EICL_DSSVerify
        (EICLData            *Message,
        EICLData             *Signature_r,
        EICLData             *Signature_s,
        EICLPublicKey        *PublicKey)
{
    EICLWord	sval[MODULUS], rval[MODULUS], qval[MODULUS],
			pval[MODULUS], wval[MODULUS], gval[MODULUS],
			u1val[MODULUS], u2val[MODULUS], yval[MODULUS],
            vval[MODULUS];
    EICLByte wbval[MODULUSBYTES];
	DSAData	s, r, p, q, w, g, u1, u2, y, v;
    EICLData wb;
    EICLSHAState     SHAState;
    EICLSHADigest    SHADigest;

/* array value assignments */
	s.value = sval;
	r.value = rval;
	p.value = pval;
	q.value = qval;
	w.value = wval;
	g.value = gval;
	u1.value = u1val;
	u2.value = u2val;
	y.value = yval;
	v.value = vval;

    wb.value = wbval;

	EICL_MoveByte2Word (Signature_s, &s);
	EICL_MoveByte2Word (Signature_r, &r);
	EICL_MoveByte2Word (&PublicKey->PrimeDivisor, &q);
	EICL_MoveByte2Word (&PublicKey->PrimeModulus, &p);
	EICL_MoveByte2Word (&PublicKey->OrderQ, &g);
	EICL_MoveByte2Word (&PublicKey->PublicKey, &y);

/* compare for lower bound 0 */
	if ( (s.length==1) && (s.value[0]==0) )		/* if s==0 */
		return -1;
	if ( (r.length==1) && (r.value[0]==0) )		/* if r==0 */
		return -1;
/* then compare for upper bound q */
	if (EICL_Compare (&s, &q)>=0)	/* if s>=q */
		return -1;
	if (EICL_Compare (&r, &q)>=0)    /* if r>=q */
		return -1;

/* OK, 0<r<q and 0<s<q, do the computations ... */

/* w = s^{-1} mod q */
	EICL_ModularInverse (Signature_s, &PublicKey->PrimeDivisor,
		&wb);
    EICL_MoveByte2Word(&wb, &w);


/* pad the high order bytes of q with zeros for dword alignment */
	
/* u1 = SHA(M)*w mod q */
    EICL_SHABegin (&SHAState);
    EICL_SHAProcess (Message, &SHAState);
    EICL_SHAEnd (&SHAState, SHADigest);

    u1.value[4] = bswap ( ((EICLWord *)SHADigest)[0] );
    u1.value[3] = bswap ( ((EICLWord *)SHADigest)[1] );
    u1.value[2] = bswap ( ((EICLWord *)SHADigest)[2] );
    u1.value[1] = bswap ( ((EICLWord *)SHADigest)[3] );
    u1.value[0] = bswap ( ((EICLWord *)SHADigest)[4] );
	u1.length = 5;

    EICL_Rem (&u1, &q, &u1);

	EICL_ModMul (&u1,&w, &q, &u1);

/* u2 = r*w mod q */
	EICL_ModMul (&r, &w, &q, &u2);

/* u1 = g^u1 mod p */
	EICL_ModExp (&g, &u1, &p, &u1);

/* u2 = y^u2 mod p */
	EICL_ModExp (&y, &u2, &p, &u2);

/* u1 = u1*u2 mod p */
	EICL_ModMul (&u1, &u2, &p, &u1);

/* v = (g^u1 * y^u2 mod p) mod q */
	
	EICL_Rem (&u1, &q, &v);
/* the signature comparison */
	if (EICL_Compare (&v, &r)!=0)
		return -1;

	return 0;
}

