/*-----------------------------------------------------------------------
 * File: MODEXP.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Module name: modexp.c
   Modular Exponentiation using Montgomery product.

   The higher words from DSSData.length to MODULUS are assumed
   to be filled with 0's before this function is called.
*/

#include "eicl.h"
#include "dss.h"

void __stdcall EICL_ModExp (DSAData *a, DSAData *e, DSAData *n, DSAData *t)
{
  DSAInt	a_bar;
  DSAData	tmp, t_bar;
  EICLWord	tmparray[2*MODULUS+2], tbararray[MODULUS+3];
  long		i, j;
  EICLWord	mask, eval, *aval, *evalue;
  DSAData	*Montgomery_Input, *Montgomery_Output, *swapptr;
  /* reserve space for former global variables */
  EICLWord	ModExpN0prime;
  DSAData	ModExpModulus;


  ModExpN0prime = N0Prime (n->value[0]);
  ModExpModulus.length = n->length;
  ModExpModulus.value = n->value;

  tmp.value = tmparray;
  t_bar.value = tbararray;

/* the initial t_bar value:  t_bar = R mod n */
  tmp.length=ModExpModulus.length + 1;
  for (j=ModExpModulus.length-1; j>=0; --j)
    tmparray[j] = 0;
  tmparray[ModExpModulus.length] = 1;
  EICL_Rem (&tmp, n, &t_bar);

/* compute a*R mod n */
  for (j=0; j<ModExpModulus.length; j++)        /* tmp = a*R */
    tmparray[j]=0;
  aval = a->value;
  for (i=0, j=ModExpModulus.length; i<a->length; i++, j++)
    tmparray[j]=aval[i];
  tmp.length=a->length + ModExpModulus.length;
  DSA_Rem (&tmp, n, &a_bar);           /* a_bar = a*R mod n */

/* * * * * *  The Exponentiation Loop  * * * * * */
  mask = 0x80000000UL;
  evalue = e->value;
/* by-pass any leading zeros in the exponent, start from MSB of exponent */
  eval = evalue[e->length-1];
  while ((mask!=0) && (eval & mask)==0)
    mask>>=1;
/* well done, start looping on each bit of exponent */
  Montgomery_Input = &t_bar;
  Montgomery_Output = &tmp;
  evalue  = e->value;

  for (i=e->length-1; i>=0; --i) {
    eval = evalue[i];
      for (; mask!=0; mask>>=1) {
		MontSquare (ModExpN0prime, ModExpModulus, Montgomery_Input, Montgomery_Output);
      if ((eval & mask)!=0) {
		MontProduct (ModExpN0prime, ModExpModulus, 
			Montgomery_Output, &a_bar, Montgomery_Input);
		continue;
      }
	  swapptr = Montgomery_Input;
	  Montgomery_Input = Montgomery_Output;
	  Montgomery_Output = swapptr;
    }
    mask=0x80000000;
  }

/* Obtain natural representation of the Montgomery residue result */
	for (j=MODULUS-1; j>=1; --j)
		a_bar.value[j]=0;
    a_bar.value[0] = 1;
	a_bar.length = 1;
	MontProduct (ModExpN0prime, ModExpModulus, Montgomery_Input, &a_bar, Montgomery_Output);

/* discard any leading zero words */
    i = MODULUS;
    aval = Montgomery_Output->value;
    while (i>0 && aval[i-1]==0)
        --i;
    Montgomery_Output->length = i;

/* copy the result to t */
	for (i=Montgomery_Output->length-1; i>=0; --i)
		t->value[i] = Montgomery_Output->value[i];
	t->length = Montgomery_Output->length;
}
