/*-----------------------------------------------------------------------
 * File: REM.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Module name: rem.c
   Computes the remainder of division a by b
*/

#include "eicl.h"
#include "dss.h"

#define ShiftRight(m) { posb=m.length-1;\
                        for (posa=0; posa<posb; ++posa)\
                          m.value[posa]=\
                            (m.value[posa]>>1)|(m.value[posa+1]<<31);\
                        m.value[posb]>>=1;\
                        if (m.value[posb]==0 && posb>=1)\
                          m.length=posb;}

void __stdcall EICL_Rem (DSAData *a, DSAData *b, DSAData *r)
{
  register int	posa, posb;
  int		worddiff, bdiff;
  DSAData	sb, rem;
  EICLWord	sbarray[2*MODULUS+2], remarray[2*MODULUS+2];
  EICLWord	msw, *bval;

  if (EICL_Compare (a, b)<0) {    /* a = a mod b   already */
    bval = a->value;
    for (posa=a->length-1; posa>=0; --posa)
      r->value[posa] = bval[posa];
    r->length = a->length;
    for (posa=a->length; posa<MODULUS; ++posa)  /* fill upper words with 0 */
        r->value[posa] = 0;
    return;
  }
  rem.value = remarray;
  sb.value = sbarray;

/* sbval will contain the shifted b-value thru iterations
   rem contains the computed partial remainder
*/
  bval=a->value;
  for (posa=a->length-1; posa>=0; --posa)
    remarray[posa]=bval[posa];

/* find the difference btw a and b in bits */
  msw = remarray[(rem.length=a->length)-1];
  for (posa=0; msw!=0; ++posa)
    msw>>=1;
  msw = b->value[b->length-1];
  for (posb=0; msw!=0; ++posb)
    msw>>=1;
/* posa and posb indicate the actual bit indices of MSWs */
  worddiff = rem.length - b->length - (posa<posb);
  bdiff = (posa>=posb) ? posa-posb : (8*sizeof(EICLWord))+posa-posb;

/* OK, the difference is 32*worddiff + bdiff.
Now shift b and copy to sb */
  bval = b->value;
  if (bdiff==0) {
    for (posa=worddiff+(posb=b->length-1); posb>=0; --posb, --posa)
      sbarray[posa]=bval[posb];
    sb.length = rem.length;
  }
  else {
    for (posa=worddiff+(posb=b->length-1); posb>0; --posb, --posa)
      sbarray[posa]=(bval[posb] << bdiff) | (bval[posb-1] >> (32-bdiff));
    sbarray[worddiff] = bval[0] << bdiff;
    sbarray[rem.length]=0;
    sbarray[b->length+worddiff] = bval[b->length-1] >> (32-bdiff);
    sb.length = rem.length + (sbarray[rem.length]!=0);
  }

/* fill the lower words with 0 */
  for (posa=worddiff-1; posa>=0; --posa)
    sbarray[posa]=0;

/* OK, now start the reduction loop */
  worddiff=(worddiff << 5) + bdiff;     /* number of bits to scan */
  while (worddiff-->=0) {
    if (EICL_Compare (&rem, &sb)>=0)
      EICL_Subtract (&rem, &sb);
    ShiftRight (sb);
  }

/* copy the remainder in rem to r */
  bval = r->value;
  for (posa=0; posa<rem.length; ++posa)
    bval[posa] = remarray[posa];
  r->length = rem.length;

/* fill the upper words with 0's */
  while (posa<MODULUS)
    bval[posa++]=0;
}

void __stdcall DSA_Rem (DSAData *a, DSAData *b, DSAInt *r)
{
  register int	posa, posb;
  int		worddiff, bdiff;
  DSAData	sb, rem;
  EICLWord	sbarray[2*MODULUS+2], remarray[2*MODULUS+2];
  EICLWord	msw, *bval;

  if (EICL_Compare (a, b)<0) {    /* a = a mod b   already */
    bval = a->value;
    for (posa=a->length-1; posa>=0; --posa)
      r->value[posa] = bval[posa];
    r->length = a->length;
    for (posa=a->length; posa<MODULUS; ++posa)  /* fill upper words with 0 */
        r->value[posa] = 0;
    return;
  }
  rem.value = remarray;
  sb.value = sbarray;

/* sbval will contain the shifted b-value thru iterations */
  bval=a->value;
  for (posa=a->length-1; posa>=0; --posa)
    remarray[posa]=bval[posa];
  rem.length=a->length;

/* find the difference btw a and b in bits */
  msw = remarray[rem.length-1];
  for (posa=0; msw!=0; ++posa)
    msw>>=1;
  msw = b->value[b->length-1];
  for (posb=0; msw!=0; ++posb)
    msw>>=1;
  worddiff = rem.length - b->length - (posa<posb);
  bdiff = (posa>=posb) ? posa-posb : 32+posa-posb;

/* OK, the difference is 32*worddiff + bdiff. Now shift b and copy to sb */
  bval = b->value;
  if (bdiff==0) {
    for (posa=worddiff+(posb=b->length-1); posb>=0; --posb, --posa)
      sbarray[posa]=bval[posb];
    sb.length = rem.length;
  }
  else {
    for (posa=worddiff+(posb=b->length-1); posb>0; --posb, --posa)
      sbarray[posa]=(bval[posb] << bdiff) | (bval[posb-1] >> (32-bdiff));
    sbarray[worddiff] = bval[0] << bdiff;
    sbarray[rem.length]=0;
    sbarray[b->length+worddiff] = bval[b->length-1] >> (32-bdiff);
    sb.length = rem.length + (sbarray[rem.length]!=0);
  }

/* fill the lower words with 0 */
  for (posa=worddiff-1; posa>=0; --posa)
    sbarray[posa]=0;

/* OK, now start the reduction loop */
  worddiff=(worddiff << 5) + bdiff;     /* number of bits to scan */
  while (worddiff-->=0) {
    if (EICL_Compare (&rem, &sb)>=0)
      EICL_Subtract (&rem, &sb);
    ShiftRight (sb);
  }

/* copy the remainder in rem to r */
  bval = r->value;
  for (posa=0; posa<rem.length; ++posa)
    bval[posa] = remarray[posa];
  r->length = rem.length;

/* fill the upper words with 0's */
  while (posa<MODULUS)
    bval[posa++]=0;
}
