/*-----------------------------------------------------------------------
 * File: ITERATOR.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

#include "isl_internal.h"
#include <isl_memtrack.h>

ISL_ITERATOR_PTR 
isl_CreateIterator(
	ISL_ARCHIVE_CONTEXT_PTR	pArchive,
	ISL_DATA *List)
{
	ISL_ITERATOR_PTR Iterator;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_ITERATOR_TAG,0x00);
	if (List == NULL || pArchive == NULL)
		return NULL;

	Iterator = (ISL_ITERATOR_PTR) isl_AllocateMemory(pArchive,sizeof(ISL_ITERATOR));
	if (Iterator == NULL) {
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive,sizeof(ISL_ITERATOR));
		return NULL;
	}
	Iterator->pArchive = pArchive;

	Iterator->Node = (ISL_NODE_PTR) isl_AllocateMemory(pArchive, List->Length);
	if (Iterator->Node == NULL) {
		isl_FreeMemory(pArchive, Iterator);
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive, List->Length);
		return NULL;
	}

	/* Here we assume the ISL_DATA is an array of pointers to objects */
	Iterator->NumNodes = List->Length / sizeof(uint8*);
	Iterator->NodeCount = 0;
	memcpy(Iterator->Node, List->Data, List->Length);

	return (Iterator);
}

void *
isl_GetNextNode(ISL_ITERATOR_PTR Iterator)
{
	uint32 NodeCount;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_ITERATOR_TAG,0x01);
	if (Iterator == NULL)
		return NULL;
	
	NodeCount = Iterator->NodeCount++;
	if (NodeCount < Iterator->NumNodes)
		return (Iterator->Node[NodeCount]);
	else
		return NULL;
}

ISL_STATUS
isl_RecycleIterator(ISL_ITERATOR_PTR Iterator)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_ITERATOR_TAG,0x02);
	if (Iterator == NULL)
		return ISL_FAIL;

	if (Iterator->Node)
	{
		isl_FreeMemory(Iterator->pArchive, Iterator->Node);
	}

	isl_FreeMemory(Iterator->pArchive, Iterator);
	
	return ISL_OK;
}
