/*-----------------------------------------------------------------------
 * File: MANIFEST.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

/*
 *
 * EISL_GetManifestSignatureRoot
 * EISL_VerifyAndLoadModule
 * EISL_VerifyLoadedModule
 * EISL_FindManifestSectionAttribute
 * EISL_CreateManifestSectionAttributeEnumerator
 */

#include "eislapi.h"
#include "isl_internal.h"

#ifdef TEST
#include <stdio.h>
#include "islutil.h"
#include <cssmMemTrack.h>
#endif

#ifdef EISL
static ISL_STATUS
VerifyModuleCommon(
	ISL_MODULE_PTR	pModule,
	uint32			Memory);
#endif

/*
 * EISL_GetManifestSignatureRoot
 */
ISL_VERIFIED_SIGNATURE_ROOT_PTR 
EISL_GetManifestSignatureRoot(
	const ISL_MANIFEST_SECTION_PTR Section)
{
	ISL_ARCHIVE_CONTEXT_PTR pArchive = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x00);
	if (NULL == Section)
		return NULL;

	pArchive = Section->Parent;
	if (NULL == pArchive || NULL == pArchive->Signatures ||
			NULL == pArchive->Signatures->Signature)
		return NULL;

	return (pArchive->Signatures->Signature->Signers);
}


/*
 * EISL_FindManifestSectionAttribute
 *
 * This function updates the length and pointer to refer to the 
 * Manifest Section Attribute (or metadata) Value corresponding to the given name, 
 * or returns ISL_FAIL if there is no such attribute.
 */

ISL_STATUS 
EISL_FindManifestSectionAttribute(
	ISL_MANIFEST_SECTION_PTR Section,
	ISL_CONST_DATA Name,
	ISL_CONST_DATA_PTR Value)
{
	ISL_NAME_VALUE_PTR pAttribute = NULL;
	ISL_SECTION_PTR SectionInfo;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x01);
	if (NULL == Section || NULL == Value)
		return ISL_FAIL;

	SectionInfo = &Section->SectionInfo;
	for(pAttribute = SectionInfo->Attribute;
		pAttribute;
		pAttribute = pAttribute->Next)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x02);
		if (IS_EQUAL(Name, pAttribute->Name))
		{
			Value->Length = pAttribute->Value.Length;
			Value->Data   = pAttribute->Value.Data;
			return ISL_OK;
		}
	}
	return ISL_FAIL;
}

/*
 * EISL_CreateManifestSectionAttributeEnumerator
 */
ISL_ITERATOR_PTR 
EISL_CreateManifestSectionAttributeEnumerator(
	const ISL_MANIFEST_SECTION_PTR Section)
{
	ISL_ITERATOR_PTR pItr = NULL;
	ISL_ARCHIVE_CONTEXT_PTR pArchive;
	ISL_DATA				AttributePointerArray = {0, NULL};
	ISL_NAME_VALUE_PTR		*Attributes = NULL;
	ISL_NAME_VALUE_PTR		pAttribute = NULL;
	uint32					i = 0;
	ISL_SECTION_PTR SectionInfo;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x03);
	if (!Section)
		return NULL;

	pArchive = Section->Parent;
	SectionInfo = &Section->SectionInfo;
	ASSERT(pArchive);

	AttributePointerArray.Length = sizeof(ISL_NAME_VALUE_PTR) * SectionInfo->AttributeCount;
	Attributes = isl_AllocateMemory(pArchive, AttributePointerArray.Length);
	if (!Attributes) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_COPY_MEMORY);
		sprintf(gErrBuf, gErrStr, pArchive, AttributePointerArray.Length);
		isl_ReportError(pArchive, ISL_NO_MEMORY, gErrBuf);
#endif
		return NULL;
	}

	AttributePointerArray.Data = (void*)Attributes;

	for(pAttribute = SectionInfo->Attribute, i = 0;
		pAttribute;
		pAttribute = pAttribute->Next, i++)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x04);
		Attributes[i] = pAttribute;
	}

	pItr = isl_CreateIterator(pArchive, &AttributePointerArray);
	isl_FreeMemory(pArchive, Attributes);
	if (!pItr) {
		REPORT_CREATE_ITERATOR_ERROR(pArchive, &AttributePointerArray);
		return NULL;
	}
	return pItr;
}

ISL_STATUS 
EISL_GetNextManifestSectionAttribute(
	const ISL_ITERATOR_PTR Iterator,
	ISL_CONST_DATA_PTR Name,
	ISL_CONST_DATA_PTR Value)
{
	ISL_NAME_VALUE_PTR node = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x05);
	node = (ISL_NAME_VALUE_PTR) isl_GetNextNode(Iterator);
	if (!node)
		return ISL_FAIL;

	Name->Length  = node->Name.Length;
	Name->Data	  = node->Name.Data;
	Value->Length = node->Value.Length;
	Value->Data   = node->Value.Data;
	return ISL_OK;
}

ISL_STATUS 
EISL_RecycleManifestSectionAttributeEnumerator(
	const ISL_ITERATOR_PTR Iterator)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x06);
	return isl_RecycleIterator(Iterator);
}



#ifdef EISL
#define WORKFACTORSHIFT 12
/* 
 *	EISL_ContinueVerification
 *
 *	This function continues increment verification of a memory manifest section.
 *
 */
uint32 
EISL_ContinueVerification(
	ISL_VERIFIED_MODULE_PTR pModule,
	uint32 WorkFactor)
{
	ISL_GET_DATA_SERVICE_CONTEXT	*context = NULL;
	EICLSHADigest					digest;
	EICLSHAState						*pState;
	EICLData							message		= {0, NULL};	
	ISL_ALG_INFO_PTR				pNode		= NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x0c);

	if (!pModule || !pModule->hModule)
		return 0L;	/* not a loaded module */

	if (!pModule->Continuation) {
		pModule->Continuation = isl_AllocateMemory(pModule->ManSect->Parent, sizeof(struct isl_verify_continuation));
		if (!pModule->Continuation) {
			return 0;
		}
		pModule->Continuation->Count = 1;
		pState = (EICLSHAState*) isl_AllocateMemory(pModule->ManSect->Parent, sizeof(EICLSHAState));
		pModule->Continuation->DigestContext = pState;		
		if (!pState) {
			return 0;
		}
		EICL_SHABegin(pState);
		context = (ISL_GET_DATA_SERVICE_CONTEXT*)
			  isl_AllocateMemory(	pModule->ManSect->Parent, 
									isl_GetDllSizeofObject());
		if (!context) {
			return 0;
		}
		pModule->Continuation->GetDataContext = context;
#ifdef VMS
                if (isl_GetDllInitMemory(context, (void *)pModule->Path.Data) != ISL_OK)
#else
		if (isl_GetDllInitMemory(context, pModule->hModule) != ISL_OK)
#endif
			return 0L;
		pModule->Continuation->PartialData.Length = 0;
	} else {
		pState = (EICLSHAState*) pModule->Continuation->DigestContext;
		context = (ISL_GET_DATA_SERVICE_CONTEXT*) pModule->Continuation->GetDataContext;
	}
	
	while(WorkFactor != 0)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x0d);
		
		if (pModule->Continuation->PartialData.Length == 0) {
			if (isl_GetDllUpdateMemory(context, &pModule->Continuation->PartialData) != ISL_OK)
				return 0;
			if (pModule->Continuation->PartialData.Length == 0)
				break;
		}
		message.value = (EICLByte *) pModule->Continuation->PartialData.Data;
		message.length = pModule->Continuation->PartialData.Length;
		if ((unsigned) message.length > (WorkFactor << WORKFACTORSHIFT)) {
			message.length = (WorkFactor << WORKFACTORSHIFT);
		}
		pModule->Continuation->PartialData.Data += message.length;
		pModule->Continuation->PartialData.Length -= message.length;
		WorkFactor -= (message.length >> WORKFACTORSHIFT);
		EICL_SHAProcess(&message, pState);
		
	}
	if (WorkFactor == 0) 
		return pModule->Continuation->Count;

	EICL_SHAEnd(pState, digest);
	if (isl_GetDllRecycleMemory(context) != ISL_OK)
			return 0L;

	EICL_SHABegin(pState);
#ifdef VMS
        if (isl_GetDllInitMemory(context, (void *)pModule->Path.Data) != ISL_OK)
#else
	if (isl_GetDllInitMemory(context, pModule->hModule) != ISL_OK)
#endif
            return 0L;

	pNode = isl_FindSHA(pModule->ManSect->SectionInfo.Algorithm);
	if (!pNode)
		return 0L;

	if (pNode->Hash->Value.Length != 20 ||
		memcmp(pNode->Hash->Value.Data, digest, 20) != 0)
		return 0L;
	if (++pModule->Continuation->Count == 0) // watch out for wrap-around
		pModule->Continuation->Count = 1;
	return pModule->Continuation->Count;
}
#endif /* EISL */
