/*-----------------------------------------------------------------------
 * File: MEMORY.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 


#include "isl_internal.h"
#include <isl_memtrack.h>

/*
 * Protected Methods
 *
 * isl_CopyMemory
 * isl_AllocateMemory
 * isl_FreeMemory
 */

/*
 * isl_CopyMemory
 *
 */
void *
isl_CopyMemory(
	ISL_ARCHIVE_CONTEXT_PTR pArchive, 
	const void *buffer, 
	ISL_SIZE size)
{
	void * ptr = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MEMORY_TAG,0x02);
	ptr = isl_AllocateMemory(pArchive, size);
	if (!ptr) return NULL;
	memcpy(ptr, buffer, size);
	return ptr;
}

/*
 * isl_AllocateMemory
 *
 */
void *
isl_AllocateMemory(
	ISL_ARCHIVE_CONTEXT_PTR pArchive, 
	ISL_SIZE size)
{
	ISL_MEMORY_BUFFER_PTR pMemNode = NULL;
	void *ptr = NULL;
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MEMORY_TAG,0x03);
	
	ASSERT(pArchive);														/* parameter checking */
	pMemNode = (ISL_MEMORY_BUFFER_PTR) malloc(sizeof(ISL_MEMORY_BUFFER));	/* allocating memory for book keeping*/
	if (!pMemNode) return NULL;												/* checking for valid memory */
	memset(pMemNode, 0, sizeof(ISL_MEMORY_BUFFER));							/* initialize to zero */

	ptr = malloc(size);														/* allocating requested memory */
	if (!ptr)																/* checking for valid memory */
	{																		/* memory not valid */
		free(pMemNode);														/* freeing book keeping memory */
		return NULL;														
	}

	pMemNode->Next = pArchive->Buffers;										/* pointing to allocated list */
	pMemNode->Buffer = ptr;													/* recording buffer */
	pArchive->Buffers = pMemNode;											/* adding to list */
	return ptr;
}

/*
 * isl_FreeMemory
 *
 */
void 
isl_FreeMemory(
	ISL_ARCHIVE_CONTEXT_PTR pArchive, 
	void *buffer)
{
	ISL_MEMORY_BUFFER_PTR	Curr = NULL;
	ISL_MEMORY_BUFFER_PTR	Prev = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MEMORY_TAG,0x04);
	for(Curr = pArchive->Buffers;											/* looping through all buffers */
		Curr;																/* exit condition */
		Curr = Curr->Next)													/* getting next buffer */
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MEMORY_TAG,0x05);
		if (buffer == Curr->Buffer)											/* match pointers */
		{
			if (!Prev)														/* testing for head of list */
				pArchive->Buffers = Curr->Next;								/* head of list is next buffer */
			else
				Prev->Next = Curr->Next;									/* pointer previous bufffer to next buffer */
			
			free(buffer);													/* free buffer */
			free(Curr);														/* free book keeping */
			return;
		}
		Prev = Curr;														/* save current pointer */
	}
	return;
}
