/*-----------------------------------------------------------------------
 * File: PKCS.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

#include "isl_internal.h"
#include <isl_memtrack.h>

static
ISL_STATUS
ParseSignedData(
	ISL_ARCHIVE_CONTEXT_PTR	pArchive,
	BER_PARSED_ITEM			SignedData,
    ISL_PARSED_PKCS         *pPKCS);

ISL_STATUS
isl_ParsePKCS7(
    ISL_ARCHIVE_CONTEXT_PTR pArchive,
	ISL_CONST_DATA	        Image,
    ISL_PARSED_PKCS         *pPKCS)
{
    ISL_STATUS              status = ISL_FAIL;
	BER_PARSED_ITEM		    SignedData;	
	BER_PARSED_ITEM		    msgitem;
	BER_PARSED_ITEM         ContentType;
    BER_PARSED_ITEM         Content;
	const uint8				*currentPosition = NULL;
	uint32			        remainingInput = 0;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x00);
    
	currentPosition = (uint8 *)Image.Data;
	remainingInput =  Image.Length;

	currentPosition = BER_ExpandItem(currentPosition,
                                     remainingInput, 
					                 &msgitem);
	if (!currentPosition) 
		return ISL_FAIL;
	currentPosition = msgitem.Content;
	remainingInput  = msgitem.ContentLength;
	currentPosition = BER_ExpandItem(currentPosition, 
					                remainingInput, 
					                &ContentType);
	if (!currentPosition)
		return ISL_FAIL;
	remainingInput  = msgitem.ContentLength - (currentPosition - msgitem.Content);
	currentPosition = BER_ExpandItem(currentPosition, 
					                 remainingInput, 
					                 &Content);
	if (!currentPosition)
		return ISL_FAIL;
	currentPosition = Content.Content;
	remainingInput  = Content.ContentLength;
	currentPosition = BER_ExpandItem(currentPosition, 
					                 remainingInput, 
					                 &SignedData);
	if (!currentPosition)
	        return ISL_FAIL;
    status = ParseSignedData(pArchive, SignedData, pPKCS);
    if (status != ISL_OK)
        return status;

	return ISL_OK;
}

static
ISL_STATUS
ParseSignedData(
	ISL_ARCHIVE_CONTEXT_PTR	pArchive,
	BER_PARSED_ITEM			SignedData,
    ISL_PARSED_PKCS         *pPKCS)
{
    BER_PARSED_ITEM		    *pNode = NULL;	
	const uint8			    *currentPosition = NULL;
	uint32				    remainingInput = 0;
	uint32				    Count = 0;
	uint32                  i = 0;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x01);
	currentPosition = SignedData.Content;
	remainingInput  = SignedData.ContentLength;
	Count = BER_CountItems(currentPosition, remainingInput);
	if (Count <= 0) return ISL_FAIL;
	pNode = isl_AllocateMemory(pArchive, Count * sizeof(BER_PARSED_ITEM));
	if (!pNode) {
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive, Count * sizeof(BER_PARSED_ITEM));
		return ISL_FAIL;
	}
	for (i=0; i < Count; i++)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x02);

		currentPosition = BER_ExpandItem(currentPosition, 
						                 remainingInput, 
						                 pNode + i);
		if (!currentPosition)
			return ISL_FAIL;
		remainingInput  = SignedData.ContentLength - (currentPosition - SignedData.Content);
	}

    pPKCS->Node  = pNode;
	pPKCS->Count = Count;
	return ISL_OK;
}

ISL_STATUS
isl_GetCertsFromPKCS(
	ISL_ARCHIVE_CONTEXT_PTR pArchive,
	ISL_PARSED_PKCS		    PKCS,
	ISL_CONST_DATA_PTR	    *pCerts,
	uint32			        *pCount)	
{
    uint32				i = 0;
	const uint8				*currentPosition = NULL;
	BER_PARSED_ITEM		*pNode;
	uint32				Count;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x03);

	if (!pCount || !pCerts)
		return ISL_FAIL;

	*pCerts = NULL;
    for(i= 0; i < PKCS.Count; i++)
    {
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x04);

        pNode = PKCS.Node + i;
	    if (pNode->Tag[0] == (BER_CONSTRUCTED | BER_CLASS_CONTEXTSPECIFIC | 0))
	        goto PARSE;
    }
	*pCount = 0;		/* no embedded certificates */
    return ISL_OK;

PARSE:
	Count = BER_CountItems(pNode->Content, pNode->ContentLength);
	if (Count < 0)  return ISL_FAIL;

	*pCount = Count;
	if (*pCount == 0) return ISL_OK;
	*pCerts = isl_AllocateMemory(pArchive, *pCount * sizeof(ISL_CONST_DATA));
	if (!(*pCerts)){
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive, *pCount * sizeof(ISL_CONST_DATA));
		return ISL_FAIL;
	}

	currentPosition = pNode->Content;
	for(i=0; i < *pCount; i++)
	{
        uint32  len = 0;

        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x05);

        len = BER_SizeofObject(currentPosition);
        if (len == 0) return ISL_FAIL;
		(*pCerts)[i].Data = currentPosition;
		(*pCerts)[i].Length = len;
		currentPosition += len;
	}

	return ISL_OK;
}

ISL_STATUS
isl_GetSignedContentFromPKCS(
	ISL_PARSED_PKCS	    PKCS,
	ISL_CONST_DATA_PTR	pSignedContent)
{
	const uint8			*currentPosition = NULL;
	uint32			remainingInput  = 0;
	BER_PARSED_ITEM Node;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x06);
    currentPosition = PKCS.Node[2].Content;
    remainingInput = PKCS.Node[2].ContentLength;
	currentPosition = BER_ExpandItem(currentPosition, remainingInput, &Node);
    if (!currentPosition) return ISL_FAIL;
	remainingInput  = PKCS.Node[2].ContentLength - (currentPosition - PKCS.Node[2].Content);
    currentPosition = BER_ExpandItem(currentPosition, remainingInput, &Node);
    currentPosition = Node.Content;
    remainingInput = Node.ContentLength;
    currentPosition = BER_ExpandItem(currentPosition, remainingInput, &Node);
	pSignedContent->Data = Node.Content;
	pSignedContent->Length = Node.ContentLength;
	return ISL_OK;
}

ISL_STATUS
isl_GetSignersFromPKCS(
	ISL_ARCHIVE_CONTEXT_PTR	pArchive,
    ISL_PARSED_PKCS         PKCS,
    ISL_CONST_DATA_PTR	    *pSignerImage,
	uint32				    *pSignerCount)
{
	uint32	                       count = 0;
	uint32	                       iSigners = 0;
	BER_PARSED_ITEM                *pSignerNodes = NULL;
    BER_PARSED_ITEM                *pNode;
    uint32                         NodeCount;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x07);

    *pSignerCount = 0;
	*pSignerImage  = NULL;
    pNode = PKCS.Node;
    NodeCount = PKCS.Count;

	/* skip over optional certificates and CRL which have context-specific tags */
	for (iSigners = 3; iSigners < NodeCount; iSigners++) {
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x08);

		if (pNode[iSigners].Tag[0] == (BER_CONSTRUCTED | BER_SET)) 
			break;
    }

	if (iSigners >= NodeCount)
		return ISL_FAIL;

	count = BER_CountItems(pNode[iSigners].Content, pNode[iSigners].ContentLength);
	if (count <= 0)
		return ISL_OK;

	pSignerNodes = isl_AllocateMemory(pArchive, count * sizeof(BER_PARSED_ITEM));
	if (!pSignerNodes) {
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive, count * sizeof(BER_PARSED_ITEM));
		return ISL_FAIL;
	}
	*pSignerCount = BER_ExpandSet(
                      pNode[iSigners].Content,
				      pNode[iSigners].ContentLength,
				      count,
				      0,
				      NULL,
				      pSignerNodes);
	
	if (*pSignerCount == 0)
		return ISL_OK;

	*pSignerImage = isl_AllocateMemory(pArchive, *pSignerCount * sizeof(ISL_CONST_DATA));
	if (!*pSignerImage) {
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive, *pSignerCount * sizeof(ISL_CONST_DATA));
		return ISL_FAIL;
	}
	for(iSigners=0; iSigners < *pSignerCount; iSigners++)
    {
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PKCS_TAG,0x09);
	    pSignerImage[iSigners]->Data = pSignerNodes[iSigners].Content;
        pSignerImage[iSigners]->Length = pSignerNodes[iSigners].ContentLength;
    }
    return ISL_OK;
}
