/*-----------------------------------------------------------------------
 * File: FILE_GET.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/*
**	relative file algorithm glue
*/
#include "isl_internal.h"
#include "integapi.h"
#include <stdio.h>
#include <isl_memtrack.h>

extern struct isl_archive_methods ArchiveMethods;
extern ISL_STATUS GetPathName(
 	ISL_MANIFEST_SECTION *Context,	
    ISL_CONST_DATA_PTR pathName);

static void identify(
	ISL_SERVICE_CLASS *algClass,				
	ISL_CONST_DATA *algID,				
	ISL_CONST_DATA *serviceName)
{
	*algClass = ISL_ServiceGetData;
	algID->Data = (const unsigned char *)"file";                    /* ifdef VMS apply cast */
	algID->Length = strlen((const char *)algID->Data);              /* ifdef VMS apply cast */
	serviceName->Data = (const unsigned char *)"file";              /* ifdef VMS apply cast */
	serviceName->Length = strlen((const char *)serviceName->Data);  /* ifdef VMS apply cast */
}
struct fileGetContext {
	FILE *f;
	char buffer[8192];
};
static ISL_SIZE size()
{
	return sizeof(struct fileGetContext);
}

static ISL_STATUS init(
	ISL_GET_DATA_SERVICE_CONTEXT *memory,
	ISL_CONST_DATA parameters)
{
	struct fileGetContext *result;

	result = (struct fileGetContext *) memory;
	if (parameters.Length > 8191)
		return 0;
	memcpy(result->buffer, parameters.Data, parameters.Length);
	result->buffer[parameters.Length] = 0;
	result->f = fopen(result->buffer, "rb");
	if (!result->f) return ISL_FAIL;
	return ISL_OK;
}

static ISL_ERROR_CODE update(
	ISL_GET_DATA_SERVICE_CONTEXT *context,
	ISL_CONST_DATA *data)
{
	struct fileGetContext *c = (struct fileGetContext*) context;
	data->Length = fread(c->buffer, 1, sizeof(c->buffer), c->f);
	data->Data = (const unsigned char *)c->buffer;
	if (data->Length > 0) return INTEGRITY_OK;
	if (feof(c->f)) return INTEGRITY_OK;
	return INTEGRITY_IO_ERROR;
}

static ISL_ERROR_CODE recycle(
	ISL_GET_DATA_SERVICE_CONTEXT *context)
{
	struct fileGetContext *c = (struct fileGetContext*) context;
	fclose(c->f);
	return INTEGRITY_OK;
}

static ISL_STATUS initializeWithClass(
		ISL_GET_DATA_SERVICE_CONTEXT *Memory,
		ISL_CONST_DATA Filename,					
		ISL_CLASS_PTR Class,						/* class structure */
		ISL_MANIFEST_SECTION_PTR Section){
	
	struct fileGetContext *result;
	ISL_CONST_DATA filePath;

	result = (struct fileGetContext *) Memory;
	if (Filename.Length > 8191)
		return ISL_FAIL;
	filePath.Data = (uint8*)&result->buffer[0];
	filePath.Length = 8192;
	if (GetPathName(Section, &filePath) != ISL_OK)
		return ISL_FAIL;
	result->f = fopen(result->buffer, "rb");
	if (!result->f) return ISL_FAIL;
	return ISL_OK;
}

ISL_STATUS
GetPathName(
 	ISL_MANIFEST_SECTION *Context,		/* context */
    ISL_CONST_DATA_PTR pathName)
{
	if (!Context || !Context->Parent)
		return ISL_FAIL;

	if (!pathName) {
		return ISL_FAIL;
	}
	
	memset((void *)pathName->Data, 0, 256);
	if (Context->Parent->PathName.Length + Context->Parameters.Length > 250) {
		return ISL_FAIL;	// path too long
	}

	memcpy((void*)pathName->Data, (char *)Context->Parent->PathName.Data, 
		Context->Parent->PathName.Length);
	pathName->Length = Context->Parent->PathName.Length;
	while (pathName->Length > 0 && pathName->Data[--pathName->Length] != '\\');
	if (pathName->Length > 0)
		((char*)pathName->Data)[pathName->Length++] = '\\';
	memcpy((char*)pathName->Data + pathName->Length, Context->Parameters.Data,
		Context->Parameters.Length);
	pathName->Length += Context->Parameters.Length;
	((char*)pathName->Data)[pathName->Length] = 0; 
    return ISL_OK;
}  

ISL_GET_DATA_METHODS  FileGetDataMethods = {
	{ identify },
	size,
	init,
	update,
	recycle,
	initializeWithClass
};
