/*-----------------------------------------------------------------------
* File: SETUP.C
*
* Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
*-----------------------------------------------------------------------
*/
/*
* INTEL CONFIDENTIAL 
* This file, software, or program is supplied under the terms of a 
* license agreement or nondisclosure agreement with Intel Corporation 
* and may not be copied or disclosed except in accordance with the 
* terms of that agreement. This file, software, or program contains 
* copyrighted material and/or trade secret information of Intel 
* Corporation, and must be treated as such. Intel reserves all rights 
* in this material, except as the license agreement or nondisclosure 
* agreement specifically indicate. 
*/
/* 
* WARNING: EXPORT RESTRICTED. 
* This software listing contains cryptographic methods and technology. 
* It is export restricted by the Office of Defense Trade Controls, United 
* States Department of State and cannot be downloaded or otherwise 
* exported or re-exported (i) into (or to a national or resident of) Cuba, 
* Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
* to which the US has embargoed goods; or (ii) to anyone on the US 
* Treasury Department's list of Specially Designated Nationals or the US 
* Commerce Department's Table of Denial Orders. By downloading or using 
* this product, you are agreeing to the foregoing and you are representing 
* and warranting that you are not located in, under the control of, or a 
* national or resident of any such country or on any such list. 
*/

#include "cssm.h"
#include "memfuncs.h"
#include "stdio.h"
#define CSSM_INITGUID
#include "guids.h"

CSSM_UPCALLS MemoryFunctions = {_spi_malloc, 
                                _spi_free, 
                                _spi_realloc, 
                                _spi_calloc,
				0,
				0};

static CSSM_API_MEMORY_FUNCS APIMemoryFunctions = {_app_malloc, 
                                            _app_free, 
                                            _app_realloc, 
                                            _app_calloc,
					    0};

extern CSSM_CSP_HANDLE CSPHandle;
extern CSSM_CL_HANDLE CLHandle;
extern CSSM_GUID CSPGuid;

CSSM_RETURN CertGen_Init(void)
{

    CSSM_VERSION cssm_version = {CSSM_MAJOR, CSSM_MINOR};
	CSSM_RETURN	ret = CSSM_OK;
    CSSM_PVC_MODE PvcPolicy = CSSM_PVC_SP;
    CSSM_GUID CLGuid = intel_x509v3_clm_guid_3_0_0_init;
	/* Initialize CSSM */

	ret = CSSM_Init (
					&cssm_version, 
					CSSM_PRIVILEGE_SCOPE_NONE, 
					NULL, /* must there be a guid specified here? */
					CSSM_KEY_HIERARCHY_NONE, 
					&PvcPolicy, 
					NULL);
	
    if (CSSM_OK != ret)
    {
		fprintf(stderr, "CSSM_Init failed with error code %d\n",ret);
        return ret;
	}

	
    /* Initialize CL */

    CLHandle = 0;
    ret = CSSM_ModuleLoad(
					&CLGuid, /* GUID */
					CSSM_KEY_HIERARCHY_NONE, 
                    NULL, 
                    NULL);
    if (CSSM_OK != ret)
	{
        fprintf(stderr, "CL CSSM_ModuleLoad failed. Error Code:%d\n", ret);
        return ret;
	}


	ret = CSSM_ModuleAttach (
				&intel_x509v3_clm_guid, 
				&cssm_version,
                &APIMemoryFunctions, 
				0, 
				CSSM_SERVICE_CL, 
				0,
				CSSM_KEY_HIERARCHY_NONE,
				NULL,
				0, 
				NULL,
				&CLHandle);
	
	if (CSSM_OK != ret)
    {
        fprintf(stderr, "CL CSSM_ModuleAttach failed. Error Code:%d\n", ret);
        return ret;
	}

    /* CSP Init*/
	CSPHandle = 0;

	ret = CSSM_ModuleLoad(
					&CSPGuid, /* GUID */
					CSSM_KEY_HIERARCHY_NONE, 
                    NULL, 
                    NULL);

    if (CSSM_OK != ret)
	{
        fprintf(stderr, "CSP CSSM_ModuleLoad failed. Error Code:%d\n", ret);
        return ret;
	}


	ret = CSSM_ModuleAttach (
				&CSPGuid, 
				&cssm_version,
                &APIMemoryFunctions, 
				0, 
				CSSM_SERVICE_CSP, 
				0,
				CSSM_KEY_HIERARCHY_NONE,
				NULL,
				0, 
				NULL,
				&CSPHandle);
	
	if (CSSM_OK != ret) 
    {
        fprintf(stderr, "CSP CSSM_ModuleAttach failed. Error Code:%d\n", ret);
        return ret;
	}
    
    /* All init succeeded */
    
    return CSSM_OK;
    
}


CSSM_RETURN CertGen_UnInit(void)
{

    CSSM_RETURN ret = CSSM_OK;

    ret = CSSM_ModuleDetach(CLHandle);
    if (CSSM_OK != ret) 
    {
        fprintf(stderr, "CL CSSM_ModuleDetach failed. Error Code:%d\n", ret);
	}
        
    ret = CSSM_ModuleUnload(&intel_x509v3_clm_guid, NULL, NULL);

    if (CSSM_OK != ret) 
    {
        fprintf(stderr, "CL CSSM_ModuleUnload failed. Error Code:%d\n", ret);
	}
    

    ret = CSSM_ModuleDetach(CSPHandle);
    if (CSSM_OK != ret) 
    {
        fprintf(stderr, "CSP CSSM_ModuleDetach failed. Error Code:%d\n", ret);
	}
        
    ret = CSSM_ModuleUnload(&CSPGuid, NULL, NULL);

    if (CSSM_OK != ret) 
    {
        fprintf(stderr, "CSP CSSM_ModuleUnload failed. Error Code:%d\n", ret);
	}
    
    ret = CSSM_Terminate();
    
    if (CSSM_OK != ret) 
    {
        fprintf(stderr, "CSSM_Terminate failed. Error Code:%d\n", ret);
        return ret;
	}
    return CSSM_OK;

}
