
#include <cssm.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int CSSMDataToOIDDotNotation(CSSM_DATA_PTR pData,char* buffer,int maxLength)
{
	
	if((pData == NULL) || (pData->Data == NULL)|| (pData->Length == 0))
	{
		return 0;
	}

	// allocate enough space to unpack the OID into an array of ints
	// it won't be longer than byte array of the oid +1, but it might be
	// shorter.
	
    unsigned char* Octets = pData->Data;
	uint32 dataLength = pData->Length;
	uint32 arrayLength = dataLength+1;
    uint32 *Numbers = new uint32[arrayLength];
	uint32 count = 0;

	// decode BER number into an array of integers
    /* first byte contains two multiplicatively packed numbers */
    dataLength--;
    Numbers[count++] = *Octets / 40;
    Numbers[count++] = *Octets++ % 40;
    while (dataLength > 0) 
	{
        if (count >= arrayLength) 
        {
			delete []Numbers;
			return 0;
		}
        Numbers[count] = 0;
        do 
		{
            if (dataLength-- <= 0)
            {
				delete []Numbers;
				return 0;
			}
			Numbers[count] = Numbers[count] * 128 + (*Octets & 127);
        } 
		while ((*Octets++ & 0x80) != 0);
        count++;
    }


	// now convert each integer in array into a string
	// and separate by dots
	buffer[0] = 0;
	char temp[32];

	for ( uint32 i = 0; i < count; i++ )
	{	
		sprintf(temp,"%d",Numbers[i]);
		strcat(buffer,temp);
		
		if ( i+1 < count )
		{
			strcat(buffer,".");
		}
	}

	delete[]Numbers;
	return strlen(buffer);
}

CSSM_DATA_PTR DotOidToCSSMData(char* pOidStr)
{
	int cntr;

	if(pOidStr == NULL)
	{
		return NULL;
	}

	// how many integers?
	int strLength = strlen(pOidStr);
	int numInts = 1;

	for ( cntr = 0; cntr < strLength; cntr++)
	{
		if(pOidStr[cntr] == '.')
		{
			numInts ++;
		}
	}

	if(numInts < 2)
	{
		return NULL;
	}

	// allocate an array of ints
	uint32 * Numbers = new uint32[numInts];

	// convert the string to an array of ints
	char* pCurr = pOidStr;
	char* pNext = pOidStr;

	for( cntr = 0; cntr < numInts; cntr++)
	{
		while((*pNext != '\0') && (*pNext != '.'))
		{
			pNext++;
		}

		*pNext = '\0';
		Numbers[cntr] = atoi(pCurr);
		
		pNext++;
		pCurr = pNext;
	}

	// now we have an array of integers, 
	// package them as a BER num
	sint16 size, i, j;
    uint32 value;

	/* precompute the sizes */
    size = 1;   /* first two ints turns into one BER byte*/
    for (i = 2; i < numInts; i++) {
        for (j = 1; j < 5; j++)
            if ((1UL << (j * 7)) > Numbers[i]) break;
        size += j;
    }
    
	// package the whole thing as a CSSM data
	CSSM_DATA_PTR pOID = (CSSM_DATA_PTR)malloc( sizeof(CSSM_DATA));
	pOID->Length = size;
	pOID->Data = (unsigned char *)malloc( size);
	unsigned char* Octets = pOID->Data;

	 *Octets++ = (uint8) (Numbers[0] * 40 + Numbers[1]);
    for (i = 2; i < numInts; i++) 
	{
        value = Numbers[i];
        for (j = 4; j > 0; j--) 
            if ((1UL << (j * 7)) <= value) break;
        if (j == 4) 
		{   /* special for MSFT C++ bug */
            *Octets++ = (uint8) (((value >> 28) & 15) | 128);
            j--;
        }
        while (j > 0) 
		{
            *Octets++ = (uint8) (((value >> (j * 7)) & 127) | 128);
            j--;
        }
        *Octets++ = (uint8) (value & 127);
    }
	
	delete [] Numbers;
	return pOID;
}

