/*-----------------------------------------------------------------------
 * File: ATTRIBUTELIST.H
 *
 * Copyright (c) 1998-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// AttributeList.h
//
// This file contains the class definition for the CAttributeListNode
//  and CattributeList classes.  These classes implement an 
//  attribute/value pair list.
//
//////////////////////////////////////////////////////////////////////
#ifndef ATTRIBUTELIST_H
#define ATTRIBUTELIST_H

#include "wf_type.h"
#include "mrm_defs.h"

//////////////////////////////////////////////////////////////////////
// Class description: CAttributeListNode
//  Members:
//      mAttribute(public)      
//          name of attribute
//      mValue(public)
//          value associated with the attribute
//      mNext(public)
//          pointer to the next node in the list
//  Methods:
//      none
/////////////////////////////////////////////////////////////////////
class DllExport CAttributeListNode  
{
public:
	// constructors
	CAttributeListNode(WF_CONST_DATA_PTR Attribute, WF_CONST_DATA_PTR Value);
    CAttributeListNode(const CAttributeListNode *Node);
	CAttributeListNode();
	~CAttributeListNode();

    // public members
    WF_CONST_DATA mAttribute;
    WF_CONST_DATA mValue;
    CAttributeListNode *mNext;
		
};


//////////////////////////////////////////////////////////////////////
// Class description: CAttributeList
//  Members:
//      mHead(public)      
//          pointer to the head node of the list
//      mTail(public)      
//          pointer to the tail node of the list
//      mNumberOfNodes(private)
//          count of the number of nodes in the list
//  Methods:
//      AddNode(public)
//          appends a node to the end of a list
//      GetNumberOfNodes(public)
//          returns the total number of nodes in the list
/////////////////////////////////////////////////////////////////////
class DllExport CAttributeList  
{
public:
	// constructors
	CAttributeList(const CAttributeList *List);
	CAttributeList(CAttributeListNode *node);
	CAttributeList();
	~CAttributeList();

    // public members
	CAttributeListNode *mHead;
	
	// public methods
	void AddNode( CAttributeListNode *NodeToAdd );
	unsigned int GetNumberOfNodes( );

private:
	// private members
	CAttributeListNode *mTail;
	unsigned int mNumberOfNodes;
};

#endif // !def ATTRIBUTELIST_H
