/*-----------------------------------------------------------------------
 * File: OBJECTLIST.H
 *
 * Copyright (c) 1998-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// ObjectList.h
//
// This file contains the class definition for the CObjectList
//  and CObjectListNode classes. These classes implement a list
//  of CObjectRef objects.
//
//////////////////////////////////////////////////////////////////////
#ifndef OBJECTLIST_H
#define OBJECTLIST_H

#include "ObjectRef.h"

//////////////////////////////////////////////////////////////////////
// Class description: CObjectListNode  
//  Members:
//      mRef(public)      
//          reference for this node
//      mNext(public)
//          pointer to the next node in the list
//  Methods:
//      none
/////////////////////////////////////////////////////////////////////
class DllExport CObjectListNode  
{
public:
	CObjectListNode *mNext;
	CObjectRef *mRef;
	
	// constructors
	CObjectListNode(const CObjectListNode *Node);
	CObjectListNode(CObjectRef *Ref);
	CObjectListNode();
	~CObjectListNode();

};

//////////////////////////////////////////////////////////////////////
// Class description: CObjectList  
//  Members:
//      mHead(public)      
//          pointer to the head node of the list
//      mTail(private)      
//          pointer to the tail node of the list
//      mNumberOfNodes(private)
//          count of the number of nodes in the list
//  Methods:
//      AddNode(public)
//          appends a node to the end of a list
//      GetNumberOfNodes(public)
//          returns the total number of nodes in the list
/////////////////////////////////////////////////////////////////////
class DllExport CObjectList  
{
public:
	CObjectListNode *mHead;
	
	// constructors
	CObjectList(const CObjectList *List);
	CObjectList(CObjectListNode *node);
	CObjectList();
	~CObjectList();

	// public methods
	void AddNode( CObjectListNode *NodeToAdd );
	unsigned int GetNumberOfNodes( );

private:
	// private vars
	CObjectListNode *mTail;
	unsigned int mNumberOfNodes;
};

#endif // !def OBJECTLIST_H
