/*-----------------------------------------------------------------------
 * File: CREDUTIL.Cpp
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
/*
 * INTEL CONFIDENTIAL 
 * This file, software, or program is supplied under the terms of a 
 * license agreement or nondisclosure agreement with Intel Corporation 
 * and may not be copied or disclosed except in accordance with the 
 * terms of that agreement. This file, software, or program contains 
 * copyrighted material and/or trade secret information of Intel 
 * Corporation, and must be treated as such. Intel reserves all rights 
 * in this material, except as the license agreement or nondisclosure 
 * agreement specifically indicate. 
 */
/* 
 * WARNING: EXPORT RESTRICTED. 
 * This software listing contains cryptographic methods and technology. 
 * It is export restricted by the Office of Defense Trade Controls, United 
 * States Department of State and cannot be downloaded or otherwise 
 * exported or re-exported (i) into (or to a national or resident of) Cuba, 
 * Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
 * to which the US has embargoed goods; or (ii) to anyone on the US 
 * Treasury Department's list of Specially Designated Nationals or the US 
 * Commerce Department's Table of Denial Orders. By downloading or using 
 * this product, you are agreeing to the foregoing and you are representing 
 * and warranting that you are not located in, under the control of, or a 
 * national or resident of any such country or on any such list. 
 */

#include "cssmport.h"
#include "credutil.h"

CSSM_ACCESS_CREDENTIALS* CSSMAPI CREDUTIL_CreatePasswordCredentials(
						CREDUTIL_PASSWORD_ACCESS_CREDENTIAL *pCredStruct,
						char *Password,
						CSSM_CHALLENGE_CALLBACK ChallengeCallback,
						void *ChallengeCallbackCtx )
{
	/* Parameter checks */
	if ( ( !pCredStruct ) ||
		 ( !Password && !ChallengeCallback ) ||
		 ( Password && ( ChallengeCallback || ChallengeCallbackCtx ) ) )
	{
		return NULL;
	}

	/* Clear the context structure */
	memset( pCredStruct, 0, sizeof(*pCredStruct) );

	/* Fill in the callback values. They should be valid based on the 
	 * initial parameter checks.
	 */
	pCredStruct->AccessCred.Callback = ChallengeCallback;
	pCredStruct->AccessCred.CallerCtx = ChallengeCallbackCtx;

	/* The sample list is only required if the password is an immediate 
	 * value.
	 */
	if ( Password )
	{
		/* Fill in the sample list nodes */
		pCredStruct->CredTypeElement.ElementType = CSSM_LIST_ELEMENT_WORDID;
		pCredStruct->CredTypeElement.WordID = CSSM_SAMPLE_TYPE_PASSWORD;

		pCredStruct->PasswordElement.ElementType = CSSM_LIST_ELEMENT_DATUM;
		pCredStruct->PasswordElement.Element.Word.Data = (uint8*)Password;
		pCredStruct->PasswordElement.Element.Word.Length = strlen( Password );

		/* Link the sample list nodes together */
		pCredStruct->AccessCred.Samples.NumberOfSamples = 1;
		pCredStruct->AccessCred.Samples.Samples =
											&pCredStruct->PasswordSample;
		pCredStruct->PasswordSample.TypedSample.ListType = 
													CSSM_LIST_TYPE_SEXPR;
		pCredStruct->PasswordSample.TypedSample.Head =
			pCredStruct->PasswordSample.TypedSample.Tail =
											&pCredStruct->CredTypeElement;
		pCredStruct->CredTypeElement.NextElement = 
											&pCredStruct->PasswordElement;
	}

	return &pCredStruct->AccessCred;
}

