/*-----------------------------------------------------------------------
 * File: OBJECTLIST.CPP
 *
 * Copyright (c) 1998-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// ObjectList.cpp: implementation of the CObjectListNode class.
//
//////////////////////////////////////////////////////////////////////

#ifdef CSSM_MEMTRACK
#undef new
#include <crtdbg.h>
static char SZ_FILE_NAME[] = __FILE__;
#define new new(_NORMAL_BLOCK, SZ_FILE_NAME, __LINE__)  /*need space*/
#endif /* ifdef VMS comment this out - CSSM_MEMTRACK */

#include "ObjectList.h"
#include <scMemTrack.h>

//////////////////////////////////////////////////////////////////////
// CObjectListNode::CObjectListNode()
//  constructs an empty CObjectListNode object
//
// Parameters:
//  none
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CObjectListNode::CObjectListNode()
{
    mRef = NULL;
}

//////////////////////////////////////////////////////////////////////
// CObjectListNode::~CObjectListNode()
//  destructs a CObjectListNode object
//
// Parameters:
//  none
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CObjectListNode::~CObjectListNode()
{
    if(mRef)
        delete mRef;
}

//////////////////////////////////////////////////////////////////////
// CObjectListNode::CObjectListNode(const CObjectListNode *node)
//  constructs a CObjectListNode object which is a copy of the one 
//  passed 
//
// Parameters:
//  node(input)         
//      pointer to the CObjectListNode object to copy
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CObjectListNode::CObjectListNode(const CObjectListNode *node)
{
	CObjectRef* ref = new CObjectRef(node->mRef);
	mRef = ref;
	mNext = NULL;
}

//////////////////////////////////////////////////////////////////////
// CObjectListNode::CObjectListNode(CObjectRef *ref)
//  constructs a CObjectListNode object with the reference 
//  field filled with the passed ref
//
// Parameters:
//  ref(input)
//      reference to be associated with this node
// Returns:
//  none
// Side effects:
//  ref object is copied before construction
//////////////////////////////////////////////////////////////////////
CObjectListNode::CObjectListNode(CObjectRef *ref)
{
	mRef = new CObjectRef(ref);
    mNext = 0;
}



//////////////////////////////////////////////////////////////////////
// CObjectList::CObjectList()
//  constructs an empty CObjectList object
//
// Parameters:
//  none
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CObjectList::CObjectList()
{
    mHead = 0;
	mTail = 0;
	mNumberOfNodes = 0;
}

//////////////////////////////////////////////////////////////////////
// CObjectList::~CObjectList()
//  destructs a CObjectList object
//
// Parameters:
//  none
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CObjectList::~CObjectList()
{
    CObjectListNode *current_node = mHead, *next_node;

	// delete all nodes
	for( unsigned int i=0; i<mNumberOfNodes; i++ )
	{
		next_node = current_node->mNext;
		delete current_node;
		current_node = next_node;
	}
}

//////////////////////////////////////////////////////////////////////
// CObjectList::CObjectList(const CObjectList *list
//  constructs a CObjectList object which is a copy of the one 
//  passed 
//
// Parameters:
//  list(input)         
//      pointer to the CObjectList object to copy
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CObjectList::CObjectList(const CObjectList *list)
{
	CObjectListNode *new_node, *old_node;
	
	// setup head ptr
	mHead = new CObjectListNode(list->mHead);
	mTail = mHead;
	new_node = mHead;
    old_node = list->mHead;
	// copy all nodes
	for( unsigned int i=1; i<list->mNumberOfNodes; i++ )
	{
		new_node->mNext = new CObjectListNode( old_node->mNext );
		new_node = new_node->mNext;
		mTail = new_node;
		old_node = old_node->mNext;
	}
	// intialize number of nodes
	mNumberOfNodes = list->mNumberOfNodes;
}

//////////////////////////////////////////////////////////////////////
// CObjectList::CObjectList(CObjectListNode *node)
//  constructs a CObjectList object containing the node passed
//
// Parameters:
//  node(input)         
//      pointer to the CObjectListNode object to embed in the list
// Returns:
//  none
// Side effects:
//  node will be copied prior to insertion
//////////////////////////////////////////////////////////////////////
CObjectList::CObjectList(CObjectListNode *node)
{
	mHead = new CObjectListNode( node );
	mTail = mHead;
	mNumberOfNodes = 1;
}

//////////////////////////////////////////////////////////////////////
// CObjectList::AddNode( CObjectListNode *NodeToAdd )
//  appends NodeToAdd to the end of the list
//
// Parameters:
//  NodeToAdd(input)         
//      pointer to the CObjectListNode object to add to the list
// Returns:
//  none
// Side effects:
//  node will be copied prior to addition
//////////////////////////////////////////////////////////////////////
void CObjectList::AddNode( CObjectListNode *NodeToAdd )
{
    if( !mHead )
    {
        mHead = new CObjectListNode( NodeToAdd );
        mTail = mHead;
    }
    else
    {
        mTail->mNext = new CObjectListNode( NodeToAdd );
	    mTail = mTail->mNext;
	    mTail->mNext = 0;
    }
	mNumberOfNodes++;
	return;
}

//////////////////////////////////////////////////////////////////////
// CObjectList::GetNumberOfNodes( void )
//  gets the total number of elements on the list
//
// Parameters:
//  none
// Returns:
//  number of elements in the list
// Side effects:
//  node will be copied prior to addition
//////////////////////////////////////////////////////////////////////
unsigned int CObjectList::GetNumberOfNodes( void )
{
	return mNumberOfNodes;
}


