/*-----------------------------------------------------------------------
 * File: SIGNERLIST.CPP
 *
 * Copyright (c) 1998-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// SignerList.cpp
//
//////////////////////////////////////////////////////////////////////
#ifdef CSSM_MEMTRACK
#undef new
#include <crtdbg.h>
static char SZ_FILE_NAME[] = __FILE__;
#define new new(_NORMAL_BLOCK, SZ_FILE_NAME, __LINE__)  /*need space*/
#endif /* ifdef VMS comment this out - CSSM_MEMTRACK */

#include "SignerList.h"
#ifndef VMS
#pragma warning (disable:4201 4514 4214 4115)
#endif
#include "isl.h"
#include <scMemTrack.h>
#ifndef VMS
#pragma warning (default:4201 4214 4115)
#endif
#include "cssmport.h"

//======================//
//                      //
//  CSignerListNode     //
//                      //
//======================//

//////////////////////////////////////////////////////////////////////
// CSignerListNode::CSignerListNode(const CSignerListNode *node)
//  constructs a CSignerListNode object which is a copy of the one 
//  passed 
//
// Parameters:
//  node(input)         
//      pointer to the CSignerListNode object to copy
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CSignerListNode::CSignerListNode(const CSignerListNode *node)
{
    mSigBlock.Data = (unsigned char*)malloc(node->mSigBlock.Length);
    memcpy((void*)mSigBlock.Data, node->mSigBlock.Data, node->mSigBlock.Length);
    mSigBlock.Length = node->mSigBlock.Length;
    mNext = NULL;
    if(node->mCounterSigners)
        mCounterSigners = new CSignerList( node->mCounterSigners );
    else 
        mCounterSigners = NULL;
}

//////////////////////////////////////////////////////////////////////
// CSignerListNode::CSignerListNode(WF_SIGNATURE_CONTEXT *Context)
//  constructs a CSignerListNode object based on the passed signature
//  context
//
// Parameters:
//  Context(input)         
//      pointer to the active signature context
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CSignerListNode::CSignerListNode(WF_SIGNATURE_CONTEXT *Context)
{
    mCounterSigners = NULL;
    mNext = NULL;
    ISL_CONST_DATA Block;
    (*PKCS7SignatureMethods.Flatten)( Context, &Block );
    mSigBlock.Data = (unsigned char*)malloc(Block.Length);
    memcpy((void*)mSigBlock.Data, Block.Data, Block.Length);
    mSigBlock.Length = Block.Length;
}

//////////////////////////////////////////////////////////////////////
// CSignerListNode::~CSignerListNode()
//  destructs a CSignerListNode object
//
// Parameters:
//  none
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CSignerListNode::~CSignerListNode()
{
    free((void*)mSigBlock.Data);
}



//==============//
//              //
//  CSignerList //
//              //
//==============//

//////////////////////////////////////////////////////////////////////
// CSignerList::CSignerList()
//  constructs an empty CSignerList object
//
// Parameters:
//  none
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CSignerList::CSignerList()
{
    mHead = 0;
	mTail = 0;
	mNumberOfNodes = 0;
}

//////////////////////////////////////////////////////////////////////
// CSignerList::~CSignerList()
//  destructs a CSignerList object
//
// Parameters:
//  none
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CSignerList::~CSignerList()
{
    CSignerListNode *current_node = mHead, *next_node;

	// delete all nodes
	for( int i=0; i<mNumberOfNodes; i++ )
	{
		next_node = current_node->mNext;
		delete current_node;
		current_node = next_node;
	}
}

//////////////////////////////////////////////////////////////////////
// CSignerList::CSignerList(const CSignerList *list)
//  constructs a CSignerList object which is a copy of the one 
//  passed 
//
// Parameters:
//  list(input)         
//      pointer to the CSignerList object to copy
// Returns:
//  none
// Side effects:
//  none
//////////////////////////////////////////////////////////////////////
CSignerList::CSignerList(const CSignerList *list)
{
	CSignerListNode *new_node, *old_node;
	
	// setup head ptr
	mHead = new CSignerListNode(list->mHead);
	mTail = mHead;
	new_node = mHead;
	old_node = list->mHead;
	// copy all nodes
	for( int i=1; i<list->mNumberOfNodes; i++ )
	{
		new_node->mNext = new CSignerListNode( old_node->mNext );
		new_node = new_node->mNext;
		mTail = new_node;
		old_node = old_node->mNext;
	}
	// intialize number of nodes
	mNumberOfNodes = list->mNumberOfNodes;
}

//////////////////////////////////////////////////////////////////////
// CSignerList::CSignerList(CSignerListNode *node)
//  constructs a CSignerList object containing a single node
//
// Parameters:
//  node(input)         
//      pointer to the CSignerListNode object to be used to build the 
//      list
// Returns:
//  none
// Side effects:
//  node will be copyed prior to being embedded in the list object
//////////////////////////////////////////////////////////////////////
CSignerList::CSignerList(CSignerListNode *node)
{
	mHead = new CSignerListNode( node );
	mTail = mHead;
	mNumberOfNodes = 1;
}

//////////////////////////////////////////////////////////////////////
// CSignerList::AddNode( CSignerListNode *NodeToAdd )
//  appends NodeToAdd to the end of the list
//
// Parameters:
//  NodeToAdd(input)         
//      pointer to the CSignerList object to add to the list
// Returns:
//  none
// Side effects:
//  NodeToAdd will be copied prior to addition
//////////////////////////////////////////////////////////////////////
WF_RETURN CSignerList::AddNode( CSignerListNode *NodeToAdd )
{
	if( !mHead )
    {
        mHead = new CSignerListNode( NodeToAdd );
        mTail = mHead;
    }
    else
    {
        mTail->mNext = new CSignerListNode( NodeToAdd );
	    mTail = mTail->mNext;
	    mTail->mNext = 0;
    }
	mNumberOfNodes++;
	return WF_OK;
}
